@@ -11,9 +11,7 @@ my %ignore = (
 	"scheduler" => 0,
 	"sizer" => 0,
 	"indexer" => 1,
-	"file-send" => 1,
-	"file-recv" => 1,
-	"recv-index" => 1,
+	"shcp" => 1,
 	"send-indices" => 0
 );
 
