@@ -7,9 +7,10 @@
 #include "sherlock/sherlock.h"
 #include "sherlock/index.h"
 #include "search/sherlockd.h"
+#include "search/refs.h"
 
 byte *custom_file_type_names[MAX_FILE_TYPES] = {
-  "unknown", "html", "pdf", "text", "msword", "excel", "RFU6", "RFU7", 
+  "unknown", "html", "pdf", "text", "msword", "excel", "RFU6", "RFU7",
   "jpeg", "png", "gif",  "RFU11", "RFU12", "RFU13", "RFU14", "RFU15" };
 
 byte *lng_parse(struct query *q, enum custom_op op, byte *val, uns intval)
@@ -22,7 +23,7 @@ byte *lng_parse(struct query *q, enum custom_op op, byte *val, uns intval)
   return NULL;
 }
 
-int custom_match(struct query *q, struct card_attr *ca, int *QQ)
+int custom_match(struct query *q, struct ref_context *c, struct card_attr *ca, int *QQ)
 {
   q->lng_value = CA_GET_FILE_LANG(ca);
   if (q->lng_only >= 0 && (int)q->lng_value != q->lng_only)
