@@ -15,18 +15,19 @@ uns filter_tree_limit = 4;
 uns filter_optimize = 1;
 byte *filter_dump_to = NULL;
 
-static struct cfitem fconfig[] = {
-  { "Filter",		CT_SECTION,	NULL },
-  { "Trace",		CT_INT,		&filter_trace },
-  { "HashLimit",	CT_INT,		&filter_hash_limit },
-  { "TrieLimit",	CT_INT,		&filter_trie_limit },
-  { "TreeLimit",	CT_INT,		&filter_tree_limit },
-  { "Optimize",		CT_INT,		&filter_optimize },
-  { "DumpFilterTo",	CT_STRING,	&filter_dump_to },
-  { NULL,		CT_STOP,	NULL }
+static struct cf_section fconfig = {
+	CF_ITEMS {
+		CF_UNS("Trace", &filter_trace),
+		CF_UNS("HashLimit", &filter_hash_limit),
+		CF_UNS("TrieLimit", &filter_trie_limit),
+		CF_UNS("TreeLimit", &filter_tree_limit),
+		CF_UNS("Optimize", &filter_optimize),
+		CF_STRING("DumpFilterTo", &filter_dump_to),
+		CF_END
+	}
 };
 
 static void CONSTRUCTOR fconfig_init(void)
 {
-  cf_register(fconfig);
+	cf_declare_section("Filter", &fconfig, 0);
 }
