@@ -7,6 +7,7 @@
 
 #include "sherlock/sherlock.h"
 #include "lib/conf.h"
+#include "lib/getopt.h"
 #include "lib/mempool.h"
 #include "lib/url.h"
 #include "lib/fastbuf.h"
@@ -34,7 +35,6 @@ filter_load(byte *name, struct filter_variable *var, struct filter_binding *bind
 	f->var = var;
 	f->bind = bind;
 	f->func = func;
-	f->cfg_undos = cf_item_count();
 	f->user_vars = 0;
 	f->decl = f->deleted_decl = NULL;
 	f->body = NULL;
@@ -69,7 +69,6 @@ filter_intr_new(struct filter *f)
 {
 	struct filter_args *a = xmalloc(sizeof(struct filter_args));
 	a->filter = f;
-	a->cfg_undo = xmalloc_zero(a->filter->cfg_undos*sizeof(struct filter_undo_pool));
 	if (a->filter->user_vars)
 		a->user_var = xmalloc_zero(a->filter->user_vars*sizeof(union filter_raw_value));
 	else
@@ -84,8 +83,6 @@ filter_intr_new(struct filter *f)
 void
 filter_intr_delete(struct filter_args *a)
 {
-	xfree(a->cfg_undo);
-	a->cfg_undo = NULL;
 	xfree(a->user_var);
 	a->user_var = NULL;
 	xfree(a);
@@ -136,9 +133,9 @@ filter_get_lvalue(struct filter_value *dest, struct filter_args *args, struct fi
 			break;
 		case F_LVC_CONF:
 			if (lv->type == F_ET_INT)
-				dest->v.i = * (int*) (lv->v.cfg->var);
+				dest->v.i = * (int*) (lv->v.cfg->ptr);
 			else if (lv->type == F_ET_STRING)
-				dest->v.s = * (byte**) (lv->v.cfg->var);
+				dest->v.s = * (byte**) (lv->v.cfg->ptr);
 			else
 				ASSERT(0);
 			break;
@@ -160,7 +157,6 @@ static void
 filter_set_lvalue(struct filter_lvalue *dest, struct filter_args *args, struct filter_value *v, int append)
 {
 	byte tmp[32], *tmp1;
-	struct filter_undo_pool *undo;
 
 	if (dest->undef)
 		return;
@@ -221,21 +217,11 @@ filter_set_lvalue(struct filter_lvalue *dest, struct filter_args *args, struct f
 			}
 			break;
 		case F_LVC_CONF:
-			for (undo = args->cfg_undo; undo->cfg && undo->cfg != dest->v.cfg; undo++);
-			if (!undo->cfg)
-			{
-				undo->cfg = dest->v.cfg;
-				if (undo->cfg->type == CT_INT)
-					undo->v.i = * (int*) undo->cfg->var;
-				else
-					undo->v.s = * (byte**) undo->cfg->var;
-				undo++;
-				undo->cfg = NULL;
-			}
+			cf_journal_block(dest->v.cfg->ptr, sizeof(void*));
 			if (v->type == F_ET_INT)
-				* (int*) (dest->v.cfg->var) = v->v.i;
+				* (int*) (dest->v.cfg->ptr) = v->v.i;
 			else if (v->type == F_ET_STRING)
-				* (byte**) (dest->v.cfg->var) = v->v.s;
+				* (byte**) (dest->v.cfg->ptr) = v->v.s;
 			else
 				ASSERT(0);
 			break;
@@ -440,6 +426,7 @@ compare_int(struct filter_value *l, int unsign, int op, struct filter_value *r)
 			default:
 				ASSERT(0);
 		}
+	return UNDEFINED;
 }
 
 static inline int
@@ -474,6 +461,7 @@ compare_string(struct filter_value *l, int icase, int op, struct filter_value *r
 		default:
 			ASSERT(0);
 	}
+	return UNDEFINED;
 }
 
 static inline int
@@ -490,6 +478,7 @@ log_not(int x)
 		default:
 			ASSERT(0);
 	}
+	return UNDEFINED;
 }
 
 int
@@ -576,14 +565,14 @@ filter_eval_cond(struct filter_args *args, struct filter_cond *cond)
 			switch (cond->o.bin.op)
 			{
 				case AND:
-					return l==TRUE && r==TRUE 
-						? TRUE 
+					return l==TRUE && r==TRUE
+						? TRUE
 						: l==FALSE || r==FALSE
 							? FALSE
 							: UNDEFINED;
 				case OR:
-					return l==FALSE && r==FALSE 
-						? FALSE 
+					return l==FALSE && r==FALSE
+						? FALSE
 						: l==TRUE || r==TRUE
 							? TRUE
 							: UNDEFINED;
@@ -591,13 +580,13 @@ filter_eval_cond(struct filter_args *args, struct filter_cond *cond)
 					return l==UNDEFINED || r==UNDEFINED
 						? UNDEFINED
 						: l == r
-							? TRUE 
+							? TRUE
 							: FALSE;
 				case NE:
 					return l==UNDEFINED || r==UNDEFINED
 						? UNDEFINED
 						: l != r
-							? TRUE 
+							? TRUE
 							: FALSE;
 				default:
 					ASSERT(0);
@@ -784,29 +773,14 @@ filter_intr_undo_init(struct filter_args *a)
 {
 	/* All the changes will be commited, because the undo pool is forgotten
 	 * and reset to the initial state.  */
-	a->cfg_undo[0].cfg = NULL;
+	a->oldj = cf_journal_new_transaction(0);
 }
 
 void
 filter_intr_undo(struct filter_args *a)
 {
-	struct filter_undo_pool *p;
-
-	for (p = a->cfg_undo; p->cfg; p++)
-	{
-		switch (p->cfg->type)
-		{
-			case CT_INT:
-				* (int*) p->cfg->var = p->v.i;
-				break;
-			case CT_STRING:
-				* (byte**) p->cfg->var = p->v.s;
-				break;
-			default:
-				ASSERT(0);
-		}
-	}
-	a->cfg_undo[0].cfg = NULL;
+	cf_journal_rollback_transaction(0, a->oldj);
+	a->oldj = NULL;
 }
 
 int
@@ -816,7 +790,11 @@ filter_intr_run(struct filter_args *a)
 
 	a->msg = NULL;
 	filter_user_var_init(a);
+	if (!a->want_config_changes)
+		filter_intr_undo_init(a);
 	res = filter_eval_cmd(a, a->filter->body);
+	if (!a->want_config_changes)
+		filter_intr_undo(a);
 	if (res != REJECT && res != ACCEPT)
 	{
 		log(L_ERROR, "filter: Neither ACCEPT nor REJECT issued, defaulting to REJECT.");
