@@ -2,10 +2,17 @@
  *	Sherlock Gatherer -- Content Encoding and Type Identification
  *
  *	(c) 1997--2002 Martin Mares <mj@ucw.cz>
- *	(c) 2001--2004 Robert Spalek <robert@ucw.cz>
+ *	(c) 2001--2006 Robert Spalek <robert@ucw.cz>
  */
 
 #include "sherlock/sherlock.h"
+#include "lib/clists.h"
+#include "lib/simple-lists.h"
+#include "lib/conf.h"
+#include "lib/chartype.h"
+#include "lib/fastbuf.h"
+#include "lib/url.h"
+#include "gather/gather.h"
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -13,42 +20,18 @@
 #include <fcntl.h>
 #include <unistd.h>
 
-#include "lib/conf.h"
-#include "lib/chartype.h"
-#include "lib/fastbuf.h"
-#include "lib/url.h"
-
-#include "gather/gather.h"
-
 /* Structures */
 
-struct intype_filter {			/* Input content type / content encoding filter */
-	node n;
-	byte *src, *dest;
-};
-
-struct encoding {			/* Encoding */
-	node n;
-	byte *encoding;
-	byte *drop_suffix;
-};
-
-struct type_to_enc {			/* Content type => encoding conversion */
-	node n;
-	byte *type;
-	byte *encoding;
-};
-
-enum cbmode { CBMODE_SUFFIX, CBMODE_WILDCARD, CBMODE_EXTENSION, CBMODE_BINARY, CBMODE_IS_ASCII, CBMODE_END };
+static byte *mode_names[] = { "suffix", "wildcard", "bytes", "isascii", NULL };	// doesn't contain "extension" for purpose
+enum cbmode { CBMODE_SUFFIX, CBMODE_WILDCARD, CBMODE_BINARY, CBMODE_IS_ASCII, CBMODE_EXTENSION, CBMODE_END };
 enum cbresult { CBRES_DONT_KNOW, CBRES_TRUE, CBRES_FALSE, CBRES_END };
 
 struct crystal_ball {			/* Magic rules for datomanthy */
-	node n;
-	uns order;			/* Order of the rule */
-	enum cbmode mode;		/* Crystal ball type */
-	byte *rule;			/* Name or contents pattern */
-	uns rule_len;
+	cnode n;
+	int mode;			/* Crystal ball type */
 	byte *type;			/* Resulting type */
+	byte **rule;			/* Name or contents pattern */
+	byte **compiled;		/* Compiled rule */
 };
 
 /* Configuration */
@@ -58,13 +41,16 @@ static uns trace_contents;
 #define XTRACE(x,y...) do { if (trace_contents > 1) log(L_DEBUG, x,##y); } while (0)
 #define XXTRACE(x,y...) do { if (trace_contents > 2) log(L_DEBUG, x,##y); } while (0)
 
-static list inenc_filters, encodings, type_to_encs, intype_filters;
-struct list crystal_balls;	/* contains only modes != CBMODE_EXTENSION */
-static uns cb_order = 1;
+static clist
+	inenc_filters,		/* Input content type / content encoding mapping */
+	intype_filters,
+	encodings,		/* Encoding => drop suffix */
+	type_to_encs;		/* Content type => encoding mapping */
+struct clist crystal_balls;	/* contains only modes != CBMODE_EXTENSION */
+struct clist ext_crystal_balls;	/* contains only modes == CBMODE_EXTENSION */
 static int ascii_fraction = 1000;
 
 struct extension_record {
-	uns order;
 	byte *extension;	/* key */
 	byte *content_type;
 };
@@ -73,9 +59,10 @@ struct extension_record {
 #define HASH_KEY_STRING extension
 #define HASH_WANT_FIND
 #define HASH_WANT_NEW
-#define HASH_USE_POOL cfpool
+#define HASH_USE_POOL cf_pool
 #define HASH_NOCASE
 #define HASH_CONSERVE_SPACE
+#define HASH_TABLE_ALLOC
 #include "lib/hashtable.h"
 
 struct content_type_record {
@@ -88,101 +75,27 @@ struct content_type_record {
 #define HASH_WANT_FIND
 #define HASH_WANT_FIND_NEXT
 #define HASH_WANT_NEW
-#define HASH_USE_POOL cfpool
+#define HASH_USE_POOL cf_pool
 #define HASH_CONSERVE_SPACE
+#define HASH_TABLE_ALLOC
 #include "lib/hashtable.h"
 
 static byte *snerr = "Syntax error";
 
-static inline byte *
-canonicalize_content_type(byte *ct)
+static byte *
+commit_ball(struct crystal_ball *f)
 {
-	if (!ct || !strcmp(ct, "UNKNOWN"))
+	uns size = f->rule ? DARY_LEN(f->rule) : 0;
+	if (f->mode != CBMODE_IS_ASCII && !size)
+		return "Expecting at least one rule";
+	if (f->mode != CBMODE_BINARY)
 		return NULL;
-	else
-		return ct;
-
-}
-
-static byte *
-parse_inenc(struct cfitem *c UNUSED, byte *l)
-{
-	byte *w[2];
-	struct intype_filter *f = cfg_malloc(sizeof(struct intype_filter));
-
-	if (wordsplit(l, w, 2) != 2)
-		return snerr;
-	f->src = w[0];
-	f->dest = w[1];
-	add_tail(&inenc_filters, &f->n);
-	return NULL;
-}
-
-static byte *
-parse_encoding(struct cfitem *c UNUSED, byte *l)
-{
-	byte *w[2];
-	struct encoding *f = cfg_malloc(sizeof(struct encoding));
-	int z;
-
-	z = wordsplit(l, w, 2);
-	if (z != 1 && z != 2)
-		return snerr;
-	f->encoding = w[0];
-	f->drop_suffix = (z == 2) ? w[1] : NULL;
-	add_tail(&encodings, &f->n);
-	return NULL;
-}
-
-static byte *
-parse_dtenc(struct cfitem *c UNUSED, byte *l)
-{
-	byte *w[2];
-	struct type_to_enc *f = cfg_malloc(sizeof(struct type_to_enc));
-
-	if (wordsplit(l, w, 2) != 2)
-		return snerr;
-	f->type = w[0];
-	f->encoding = w[1];
-	add_tail(&type_to_encs, &f->n);
-	return NULL;
-}
-
-static byte *
-parse_intype(struct cfitem *c UNUSED, byte *l)
-{
-	byte *w[2];
-	struct intype_filter *f = cfg_malloc(sizeof(struct intype_filter));
-
-	if (wordsplit(l, w, 2) != 2)
-		return snerr;
-	f->src = w[0];
-	f->dest = w[1];
-	add_tail(&intype_filters, &f->n);
-	return NULL;
-}
-
-static byte *
-parse_filter(struct cfitem *c, byte *l)
-{
-	byte *w[2];
-	struct crystal_ball *f = cfg_malloc(sizeof(struct crystal_ball));
-	struct content_type_record *ctr;
-
-	if (wordsplit(l, w, 2) != 2)
-		return snerr;
-	switch (c->name[0])
-	{
-		case 'S': f->mode = CBMODE_SUFFIX; break;
-		case 'W': f->mode = CBMODE_WILDCARD; break;
-		case 'E': f->mode = CBMODE_EXTENSION; break;
-		case 'B': f->mode = CBMODE_BINARY; break;
-		default: ASSERT(0);
-	}
-	if (f->mode == CBMODE_BINARY)
+	CF_JOURNAL_VAR(f->compiled);
+	f->compiled = cf_malloc(size * sizeof(byte*));
+	for (uns idx=0; idx<size; idx++)
 	{
 		int z, b = 0;
-		byte *c = w[0];
+		byte *c = f->rule[idx];
 		byte *d;
 
 		while (*c)			/* Estimate length of compiled pattern */
@@ -222,8 +135,8 @@ parse_filter(struct cfitem *c, byte *l)
 			}
 		}
 
-		d = f->rule = cfg_malloc(b + 1);	/* Compile the pattern */
-		c = w[0];
+		d = f->compiled[idx] = cf_malloc(b + 1);	/* Compile the pattern */
+		c = f->rule[idx];
 		while (*c)
 		{
 			b = 0;
@@ -256,108 +169,133 @@ parse_filter(struct cfitem *c, byte *l)
 				c++;
 		}
 		*d = 0xff;
-		f->rule_len = d - f->rule;
 	}
-	else			/* CBMODE_WILDCARD || CBMODE_SUFFIX || CBMODE_EXTENSION */
-	{
-		f->rule = w[0];
-		f->rule_len = strlen(f->rule);
-	}
-	f->type = w[1];
-	f->order = cb_order;
-	if (f->mode == CBMODE_EXTENSION)
-	{
-		struct extension_record *rec = ext_new(f->rule);
-		rec->content_type = w[1];
-		rec->order = cb_order;
-	}
-	else
-		add_tail(&crystal_balls, &f->n);
-	cb_order++;
-	ctr = ct_new(f->type);
-	ctr->cb = f;
 	return NULL;
 }
 
 static byte *
-parse_isascii(struct cfitem *c UNUSED, byte *l)
+construct_hash_tables(void *ptr UNUSED)
 {
-	struct crystal_ball *f = cfg_malloc(sizeof(struct crystal_ball));
-	struct content_type_record *ctr;
-
-	f->mode = CBMODE_IS_ASCII;
-	f->rule = NULL;
-	f->rule_len = 0;
-	f->type = l;
-	f->order = cb_order++;
-	add_tail(&crystal_balls, &f->n);
-	ctr = ct_new(f->type);
-	ctr->cb = f;
+	struct crystal_ball *f;
+	CF_JOURNAL_VAR(ext_table);
+	ext_init();
+	CLIST_WALK(f, ext_crystal_balls)
+	{
+		f->mode = CBMODE_EXTENSION;		// no need to journal, this should have been done in the init-hook and it is constant
+		uns size = f->rule ? DARY_LEN(f->rule) : 0;
+		for (uns idx=0; idx<size; idx++)
+		{
+			struct extension_record *rec = ext_new(f->rule[idx]);
+			rec->content_type = f->type;
+		}
+	}
+	CF_JOURNAL_VAR(ct_table);
+	ct_init();
+	CLIST_WALK(f, crystal_balls)
+	{
+		struct content_type_record *ctr;
+		ctr = ct_new(f->type);
+		ctr->cb = f;
+	}
 	return NULL;
 }
 
-static struct cfitem content_config[] = {
-	{ "Content",	CT_SECTION,	NULL },
-	{ "Trace",	CT_INT,		&trace_contents },
-	{ "InEnc",	CT_FUNCTION,	parse_inenc },
-	{ "Encoding",	CT_FUNCTION,	parse_encoding },
-	{ "TypeEnc",	CT_FUNCTION,	parse_dtenc },
-	{ "InType",	CT_FUNCTION,	parse_intype },
-	{ "Suffix",	CT_FUNCTION,	parse_filter },
-	{ "WildCard",	CT_FUNCTION,	parse_filter },
-	{ "Extension",	CT_FUNCTION,	parse_filter },
-	{ "Bytes",	CT_FUNCTION,	parse_filter },
-	{ "IsAscii",	CT_FUNCTION,	parse_isascii },
-	{ "AsciiFraction",CT_INT,	&ascii_fraction },
-	{ NULL,		CT_STOP,	NULL }
+static byte *
+check_destination(simp2_node *ptr)
+{
+	return ptr->s2 ? NULL : "Missing destination";
+}
+
+static struct cf_section cball_config = {
+	CF_TYPE(struct crystal_ball),
+	CF_COMMIT(commit_ball),
+#define F(x)	PTR_TO(struct crystal_ball, x)
+	CF_ITEMS {
+		CF_LOOKUP("Mode", F(mode), mode_names),
+		CF_STRING("Type", F(type)),
+		CF_STRING_DYN("rule", F(rule), 100),
+		CF_END
+	}
+#undef F
+};
+static struct cf_section ext_cball_config = {
+	CF_TYPE(struct crystal_ball),
+#define F(x)	PTR_TO(struct crystal_ball, x)
+	CF_ITEMS {
+		CF_STRING("Type", F(type)),
+		CF_STRING_DYN("rule", F(rule), 100),
+		CF_END
+	}
+#undef F
+};
+
+static struct cf_section mapping_config;	// initialized in the constructor
+static struct cf_section content_config = {
+	CF_COMMIT(construct_hash_tables),
+	CF_ITEMS {
+		CF_UNS("Trace", &trace_contents),
+		CF_LIST("InEnc", &inenc_filters, &mapping_config),
+		CF_LIST("InType", &intype_filters, &mapping_config),
+		CF_LIST("Encoding", &encodings, &cf_2string_list_config),
+		CF_LIST("TypeEnc", &type_to_encs, &mapping_config),
+		CF_LIST("Rule", &crystal_balls, &cball_config),
+		CF_LIST("ExtRule", &ext_crystal_balls, &ext_cball_config),
+		CF_INT("AsciiFraction", &ascii_fraction),
+		CF_END
+	}
 };
 
 static void CONSTRUCTOR
 content_init_config(void)
 {
-	cf_register(content_config);
-	init_list(&inenc_filters);
-	init_list(&encodings);
-	init_list(&type_to_encs);
-	init_list(&intype_filters);
-	init_list(&crystal_balls);
-	ext_init();
-	ct_init();
+	mapping_config = cf_2string_list_config;
+	mapping_config.commit = (cf_hook*) check_destination;
+	cf_declare_section("Content", &content_config, 0);
 }
 
 /* Perform input filtering of content types and encodigs */
 
 static byte *enc_m, *type_m, *rename_m;		/* Logging messages */
 
-static int
-perform_translation(struct list *filters, byte **type, byte *which)
+static inline byte *
+canonicalize_content_type(byte *ct)
 {
-	struct intype_filter *f;
+	if (!ct || !strcmp(ct, "UNKNOWN"))
+		return NULL;
+	else
+		return ct;
+
+}
+
+static int
+perform_translation(clist *filters, byte **type, byte *which)
+{
+	simp2_node *f;
 	int i = 0;
 
 	if (!*type)
 		return 0;
-	DO_FOR_ALL(f, *filters)
+	CLIST_WALK(f, *filters)
 	{
 		int res;
 		i++;
 		if (which[8] == 'T')
-			res = match_ct_patt(f->src, *type);
+			res = match_ct_patt(f->s1, *type);
 		else if (which[8] == 'E')
-			res = !strcasecmp(f->src, *type);
+			res = !strcasecmp(f->s1, *type);
 		else
 			ASSERT(0);
 		if (res)
 		{
-			XTRACE("Rule #%d: In filter: %s=`%s'", i, which, f->dest);
-			if (!strcmp(f->dest, "ERROR"))
+			XTRACE("Rule #%d: In filter: %s=`%s'", i, which, f->s2);
+			if (!strcmp(f->s2, "ERROR"))
 			{
 				log(L_ERROR_R, "Suspicious %s: %s (rule %d)", which, *type, i);
 				*type = NULL;
 			}
 			else
 			{
-				*type = canonicalize_content_type(f->dest);
+				*type = canonicalize_content_type(f->s2);
 			}
 			return 1;
 		}
@@ -368,19 +306,19 @@ perform_translation(struct list *filters, byte **type, byte *which)
 static int
 maybe_encoding(byte **enc,byte **type)			/* Maybe our content type is just an encoding? */
 {
-	struct type_to_enc *f;
+	simp2_node *f;
 	int i = 0;
 
 	if(*enc || !*type)
 		return 0;
-	DO_FOR_ALL(f, type_to_encs)
+	CLIST_WALK(f, type_to_encs)
 	{
 		i++;
-		if (match_ct_patt(f->type, *type))
+		if (match_ct_patt(f->s1, *type))
 		{
-			XTRACE("Rule #%d: Content type %s -> Encoding %s", i, *type, f->encoding);
+			XTRACE("Rule #%d: Content type %s -> Encoding %s", i, *type, f->s2);
 			*type = NULL;
-			*enc = f->encoding;
+			*enc = f->s2;
 			enc_m = "[from content-type] ";
 			return 1;
 		}
@@ -391,16 +329,16 @@ maybe_encoding(byte **enc,byte **type)			/* Maybe our content type is just an en
 static int
 encoding_to_type(byte *enc,byte **type)			/* Inverse conversion for checking the Content-Type */
 {
-	struct type_to_enc *f;
+	simp2_node *f;
 	int i = 0;
 
-	DO_FOR_ALL(f, type_to_encs)
+	CLIST_WALK(f, type_to_encs)
 	{
 		i++;
-		if (!strcasecmp(f->encoding, enc))
+		if (!strcasecmp(f->s2, enc))
 		{
-			XTRACE("Rule #%d: Encoding %s -> Content type %s", i, enc, f->type);
-			*type = f->type;
+			XTRACE("Rule #%d: Encoding %s -> Content type %s", i, enc, f->s1);
+			*type = f->s1;
 			return 1;
 		}
 	}
@@ -428,16 +366,16 @@ chop_off_suffix(byte *base, byte *s)	/* Chop off filename suffix when decoding *
 void
 cut_inenc_suffix(byte *name, byte *enc)	/* Cut the optional file suffix and return the encoding number */
 {
-	struct encoding *e;
+	struct simp2_node *e;
 
 	if (!enc || !name)
 		return;
 
-	DO_FOR_ALL(e, encodings)
-		if (!strcasecmp(e->encoding, enc))
+	CLIST_WALK(e, encodings)
+		if (!strcasecmp(e->s1, enc))
 		{
-			if (e->drop_suffix)
-				chop_off_suffix(name, e->drop_suffix);
+			if (e->s2)
+				chop_off_suffix(name, e->s2);
 			return;
 		}
 }
@@ -473,7 +411,7 @@ is_ascii(void)
 			if (*k < 0x20 && !Cblank(*k))
 			{
 				if (non_ascii < 50)
-					XXTRACE("Found non-ASCII character %x at position %d", *k, pos+k-buf);
+					XXTRACE("Found non-ASCII character %x at position %d", *k, (uns)(pos+k-buf));
 				non_ascii++;
 			}
 		pos += k-buf;
@@ -519,25 +457,27 @@ evaluate_crystal_ball(struct crystal_ball *f)
 		case CBMODE_BINARY:
 			if (!dt_fb)
 				return CBRES_DONT_KNOW;
-			if (!match_binary_pattern(f->rule))
-				return CBRES_FALSE;
-			break;
+			for (uns idx=0; idx<DARY_LEN(f->rule); idx++)
+				if (match_binary_pattern(f->compiled[idx]))
+					return CBRES_TRUE;
+			return CBRES_FALSE;
 		case CBMODE_SUFFIX:
+			if (!dt_name || !plausible_name)
+				return CBRES_DONT_KNOW;
+			for (uns idx=0; idx<DARY_LEN(f->rule); idx++)
 			{
-				int start;
-				if (!dt_name || !plausible_name)
-					return CBRES_DONT_KNOW;
-				start = dt_name_len - f->rule_len;
-				if (start < 0 || strcasecmp(f->rule, dt_name + start))
-					return CBRES_FALSE;
-				break;
+				int start = dt_name_len - strlen(f->rule[idx]);
+				if (start >= 0 && !strcasecmp(f->rule[idx], dt_name + start))
+					return CBRES_TRUE;
 			}
+			return CBRES_FALSE;
 		case CBMODE_WILDCARD:
 			if (!dt_name || !plausible_name)
 				return CBRES_DONT_KNOW;
-			if(!match_pattern_nocase(f->rule, dt_name))
-				return CBRES_FALSE;
-			break;
+			for (uns idx=0; idx<DARY_LEN(f->rule); idx++)
+				if(match_pattern_nocase(f->rule[idx], dt_name))
+					return CBRES_TRUE;
+			return CBRES_FALSE;
 		case CBMODE_IS_ASCII:
 			if (!dt_fb)
 				return CBRES_DONT_KNOW;
@@ -560,12 +500,12 @@ evaluate_crystal_ball(struct crystal_ball *f)
 }
 
 static int
-validate_content_type(uns order, byte *ct, byte **type)
+validate_content_type(byte *ct, byte **type)
 {
 	struct crystal_ball *g;
 	int valid = 1, binary_res[CBRES_END];
 	bzero(binary_res, sizeof(binary_res));
-	DO_FOR_ALL(g, crystal_balls)
+	CLIST_WALK(g, crystal_balls)
 		if (!strcmp(ct, g->type))
 		{
 			if (g->mode ==  CBMODE_IS_ASCII
@@ -579,8 +519,7 @@ validate_content_type(uns order, byte *ct, byte **type)
 		}
 	if (binary_res[ CBRES_FALSE ] && !binary_res[ CBRES_TRUE ])
 		valid = 0;
-	XTRACE("Rule #%d: looks like Content-Type %s, %s", order, ct,
-		valid ? "accepted" : "rejected");
+	XTRACE("Looks like Content-Type %s, %s", ct, valid ? "accepted" : "rejected");
 	if (valid)
 	{
 		*type = canonicalize_content_type(ct);
@@ -602,33 +541,29 @@ vaticinate(byte **type)		/* Check the existing Content-Type or find the first ma
 	if (!*type)
 	{
 		struct crystal_ball *f;
-		struct extension_record *hash_rec;
 
-		if (ext_table.hash_count > 0 && dt_name_dot)
-			hash_rec = ext_find(dt_name_dot + 1);
-		else
-			hash_rec = NULL;
-		DO_FOR_ALL(f, crystal_balls)
+		CLIST_WALK(f, crystal_balls)
 		{
-			if (hash_rec && f->order > hash_rec->order)
+			if (f->mode == CBMODE_BINARY && evaluate_crystal_ball(f) == CBRES_TRUE)
 			{
-				/* The hash-table record must be processed in
-				 * right order according to other tests.  */
-				if (validate_content_type(hash_rec->order, hash_rec->content_type, type))
-					return 1;
-				hash_rec = NULL;
-			}
-			if (evaluate_crystal_ball(f) == CBRES_TRUE)
-			{
-				if (validate_content_type(f->order, f->type, type))
+				if (validate_content_type(f->type, type))
 					return 1;
 			}
 		}
-		if (hash_rec)
+		if (ext_table.hash_count > 0 && dt_name_dot)
 		{
-			if (validate_content_type(hash_rec->order, hash_rec->content_type, type))
+			struct extension_record *hash_rec = ext_find(dt_name_dot + 1);
+			if (hash_rec && validate_content_type(hash_rec->content_type, type))
 				return 1;
 		}
+		CLIST_WALK(f, crystal_balls)
+		{
+			if (f->mode != CBMODE_BINARY && evaluate_crystal_ball(f) == CBRES_TRUE)
+			{
+				if (validate_content_type(f->type, type))
+					return 1;
+			}
+		}
 		XTRACE("No rule to guess Content-Type");
 		type_m = "[unrecognized] ";
 		return 0;
@@ -636,7 +571,6 @@ vaticinate(byte **type)		/* Check the existing Content-Type or find the first ma
 	else
 	{
 		int results[CBMODE_END][CBRES_END];	/* number of results[CBMODE_*][CBRESULT_*] */
-		int ruleid[CBMODE_END];			/* dismatched ruleid[CBMODE_*] */
 		struct content_type_record *ctr;
 		int i;
 
@@ -646,8 +580,6 @@ vaticinate(byte **type)		/* Check the existing Content-Type or find the first ma
 			struct crystal_ball *f = ctr->cb;
 			int res = evaluate_crystal_ball(f);	/* CBMODE_EXTENSION rules are also included */
 			results[f->mode][res] ++;
-			if (res==CBRES_FALSE)
-				ruleid[f->mode] = f->order;
 		}
 		if (ext_table.hash_count > 0 && dt_name_dot
 		&& ext_find(dt_name_dot + 1))
@@ -656,34 +588,29 @@ vaticinate(byte **type)		/* Check the existing Content-Type or find the first ma
 			results[CBMODE_EXTENSION][CBRES_FALSE]--;
 		}
 		for (i=CBMODE_WILDCARD; i<=CBMODE_EXTENSION; i++)
-		{
-			int j;
-			if (results[i][CBRES_FALSE] > 0
-			&& !results[CBMODE_SUFFIX][CBRES_FALSE])
-				ruleid[CBMODE_SUFFIX] = ruleid[i];
-			for (j=0; j<CBRES_END; j++)
-			{
-				results[CBMODE_SUFFIX][j] += results[i][j];
-				results[i][j] = 0;
-			}
-		}
+			if (i != CBMODE_IS_ASCII)
+				for (int j=0; j<CBRES_END; j++)
+				{
+					results[CBMODE_SUFFIX][j] += results[i][j];
+					results[i][j] = 0;
+				}
 		type_m = "[confirmed] ";
 		if (results[CBMODE_SUFFIX][CBRES_FALSE]
 		&& !results[CBMODE_SUFFIX][CBRES_TRUE])
 		{
-			XTRACE("Rule #%d: unknown suffix `%s' for Content-Type %s, but it does not matter", ruleid[CBMODE_SUFFIX], dt_name, *type);
+			XTRACE("Unknown suffix `%s' for Content-Type %s, but it does not matter", dt_name, *type);
 			type_m = "[confirmed but suffix] ";
 		}
 		if (results[CBMODE_BINARY][CBRES_FALSE]
 		&& !results[CBMODE_BINARY][CBRES_TRUE])
 		{
-			XTRACE("Rule #%d: Content-Type %s is not faithful -- all binary patterns are invalid", ruleid[CBMODE_BINARY], *type);
+			XTRACE("Content-Type %s is not faithful -- all binary patterns are invalid", *type);
 			type_m = "[rejected pattern] ";
 			return 0;
 		}
 		if (results[CBMODE_IS_ASCII][CBRES_FALSE])
 		{
-			XTRACE("Rule #%d: Content-Type %s is not faithful -- not an ascii file", ruleid[CBMODE_IS_ASCII], *type);
+			XTRACE("Content-Type %s is not faithful -- not an ascii file", *type);
 			type_m = "[rejected ascii] ";
 			return 0;
 		}
@@ -722,6 +649,18 @@ set_content_type(byte *ct)
 
 /* Parsing of the content type field */
 
+static int
+check_charset_name(byte *x)
+{
+  while (*x)
+    {
+      if (*x <= ' ' || *x >= 0x7f)
+	return 0;
+      x++;
+    }
+  return 1;
+}
+
 void
 parse_content_type(byte *l, byte **charset)
 {
@@ -765,7 +704,7 @@ parse_content_type(byte *l, byte **charset)
 				*l++ = 0;
 		}
 		/* x is the key, y is the value */
-		if (!strcasecmp(x, "charset"))
+		if (!strcasecmp(x, "charset") && check_charset_name(y))
 			*charset = y;
 	}
 }
@@ -822,7 +761,7 @@ do_guess_content(void)
 			vaticinate(&type);			/* Find the best candidate */
 		}
 
-		TRACE("Contents: encoding %s%s->%s, type %s%s%s->%s%s",
+		TRACE("Contents: encoding %s%s->%s, type %s%s%s->%s%s for %s",
 			enc_m,
 			dt_enc ? : (byte*) "?",
 			enc ? : (byte*) "?",
@@ -830,7 +769,8 @@ do_guess_content(void)
 			type_m,
 			dt_type ? : (byte*) "?",
 			type ? : (byte*) "?",
-			rename_m);
+			rename_m,
+			dt_name ? : (byte*) "?");
 		if (name)
 			strcpy(dt_name, name);
 		dt_enc = enc;
