@@ -9,6 +9,7 @@
 
 #include "sherlock/sherlock.h"
 #include "lib/conf.h"
+#include "lib/getopt.h"
 #include "lib/url.h"
 #include "lib/db.h"
 #include "lib/fastbuf.h"
@@ -50,24 +51,25 @@ static uns queue_penalty_retry;
 static uns hist_num_boxes = 0;
 static uns hist_box_width = 86400;
 
-static struct cfitem expire_config[] = {
-  { "Expire",		CT_SECTION,	NULL },
-  { "Trace",		CT_INT,		&trace },
-  { "MinRevalidateAge",	CT_INT,		&min_revalidate_age },
-  { "RevalidateCycle",	CT_INT,		&revalidate_cycle },
-  { "QueueDiscardAge",	CT_INT,		&queue_discard_age },
-  { "ErrorDiscardAge",	CT_INT,		&error_discard_age },
-  { "TmpQueueFile",	CT_STRING,	&tmp_queue_name },
-  { "QKeyExpireAge",	CT_INT,		&qk_expire_age },
-  { "StampFile",	CT_STRING,	&stamp_name },
-  { "RobotExpireAge",	CT_INT,		&robot_expire_age },
-  { "QueuePostpone",	CT_INT,		&queue_postpone },
-  { "QueueBonusRefresh",CT_INT,		&queue_bonus_refresh },
-  { "QueueBonusRegather",CT_INT,	&queue_bonus_regather },
-  { "QueuePenaltyRetry",CT_INT,		&queue_penalty_retry },
-  { "HistNumBoxes",	CT_INT,		&hist_num_boxes },
-  { "HistBoxWidth",	CT_INT,		&hist_box_width },
-  { NULL,		CT_STOP,	NULL }
+static struct cf_section expire_config = {
+  CF_ITEMS {
+    CF_INT("Trace", &trace),
+    CF_INT("MinRevalidateAge", &min_revalidate_age),
+    CF_UNS("RevalidateCycle", &revalidate_cycle),
+    CF_INT("QueueDiscardAge", &queue_discard_age),
+    CF_INT("ErrorDiscardAge", &error_discard_age),
+    CF_STRING("TmpQueueFile", &tmp_queue_name),
+    CF_UNS("QKeyExpireAge", &qk_expire_age),
+    CF_STRING("StampFile", &stamp_name),
+    CF_INT("RobotExpireAge", &robot_expire_age),
+    CF_INT("QueuePostpone", &queue_postpone),
+    CF_UNS("QueueBonusRefresh", &queue_bonus_refresh),
+    CF_UNS("QueueBonusRegather", &queue_bonus_regather),
+    CF_UNS("QueuePenaltyRetry", &queue_penalty_retry),
+    CF_UNS("HistNumBoxes", &hist_num_boxes),
+    CF_UNS("HistBoxWidth", &hist_box_width),
+    CF_END
+  }
 };
 
 #define TRACE(msg) do { if (trace) log(L_DEBUG, msg); } while(0)
@@ -84,7 +86,7 @@ stamp_read(void)
       fclose(f);
     }
   if (!cycle_last)
-    { 
+    {
       log(L_INFO, "Restarting expiration cycle");
       cycle_start = cycle_last = now;
     }
@@ -128,7 +130,6 @@ judge_filter(byte *url)
   byte *msg;
 
   rfdata.url = url;
-  ref_filter_undo();
   msg = ref_filter(&rfdata);
   if (msg)
     {
@@ -996,7 +997,7 @@ main(int argc, char **argv)
   int c;
 
   log_init(argv[0]);
-  cf_register(expire_config);
+  cf_declare_section("Expire", &expire_config, 0);
   while ((c = cf_getopt(argc, argv, "fqv" CF_SHORT_OPTS, CF_NO_LONG_OPTS, NULL)) >= 0)
     switch (c)
       {
