@@ -6,7 +6,7 @@
  */
 
 #include "sherlock/sherlock.h"
-#include "lib/conf.h"
+#include "lib/getopt.h"
 #include "lib/lists.h"
 #include "lib/mempool.h"
 #include "lib/hashfunc.h"
@@ -216,13 +216,14 @@ gather_document(struct thread *t)
   if (!t->host->qkey)
     {
       setproctitle("R %.64s", t->url);
-      create_key();
+      gather_create_key();
     }
   setproctitle("D %.64s", t->url);
-  download();
+  gather_download();
   /* Since now we run with configuration altered by the filters */
   setproctitle("P %.64s", t->url);
-  parse();
+  gather_parse();
+  gather_analyse();
   uns diff = gobj_check_update();
   if (!(diff & (GOBJ_CHG_TEXT_LARGE | GOBJ_CHG_HTTP | GOBJ_CHG_BRAND_NEW)))
     gerror(4, "Not changed");
@@ -432,9 +433,6 @@ gather_finish(struct thread *t, int err)
   if (!skip_refs)
     process_refs(o);
 
-  /* Return to normal configuration */
-  ref_filter_undo();
-
   sync_counter++;
   if (sync_counter == auto_sync)
     {
