@@ -8,7 +8,7 @@
 #include "sherlock/sherlock.h"
 #include "gather/daemon/gatherd.h"
 #include "lib/url.h"
-#include "lib/conf.h"
+#include "lib/getopt.h"
 #include "lib/db.h"
 #include "lib/mempool.h"
 #include "sherlock/object.h"
@@ -92,7 +92,7 @@ static void
 list_queue(byte *u)
 {
   struct qhost *h;
- 
+
   queue_init(1);
   if (u)
     {
@@ -597,7 +597,6 @@ test_ref_filter(void)
 		   rfdata.section, rfdata.section_soft_max, rfdata.section_hard_max,
 		   rfdata.content_type, rfdata.content_encoding,
 		   rfdata.queue_bonus, rfdata.url_key);
-	  ref_filter_undo();
 	}
     }
 }
