@@ -1,7 +1,7 @@
 /*
  *	Sherlock HTML Parser
  *
- *	(c) 1997--2004 Martin Mares <mj@ucw.cz>
+ *	(c) 1997--2006 Martin Mares <mj@ucw.cz>
  */
 
 #undef LOCAL_DEBUG
@@ -10,10 +10,12 @@
 #include "lib/conf.h"
 #include "lib/chartype.h"
 #include "sherlock/index.h"
+#include "sherlock/conf-parse.h"
 #include "lib/unicode.h"
 #include "lib/ff-utf8.h"
 #include "lib/clists.h"
 #include "charset/unicat.h"
+#include "lang/lang.h"
 #include "gather/gather.h"
 #include "gather/format/html/html.h"
 
@@ -22,15 +24,18 @@
 
 /* Configuration parameters */
 
-static int trace;
-static int comment_mode;
-static int quote_hack;
-static int script_hack;
-static int log_metas;
-static int meta_charset;
-static int char_ref_hack;
-static int ignore_unknown_char_refs;
-static int robot_comments;
+static uns comment_mode;
+static uns quote_hack;
+static uns script_hack;
+static uns refresh_hack;
+static uns refresh_threshold;
+static uns log_metas;
+static uns meta_charset;
+static uns meta_language;
+static uns char_ref_hack;
+static uns ignore_unknown_char_refs;
+static uns robot_comments;
+static byte **ignored_metas;
 
 struct user_meta {
   cnode n;
@@ -39,45 +44,45 @@ struct user_meta {
 };
 static clist user_metas;
 
-static byte *
-add_user_meta(struct cfitem *c UNUSED, byte *l)
-{
-  byte *w[2];
-  if (wordsplit(l, w, 2) != 2 || strlen(w[0]) != 1)
-    return "Incorrect MetaAttr syntax";
-  struct user_meta *um = cfg_malloc(sizeof(*um));
-  um->attr = w[0][0];
-  um->meta = w[1];
-  clist_add_tail(&user_metas, &um->n);
-  return NULL;
-}
+static struct cf_section html_config_meta = {
+  CF_TYPE(struct user_meta),
+  CF_ITEMS {
+    CF_USER("Attr", PTR_TO(struct user_meta, attr), &cf_type_attr),
+    CF_STRING("Meta", PTR_TO(struct user_meta, meta)),
+    CF_END
+  }
+};
 
-static struct cfitem html_config[] = {
-  { "HTML",		CT_SECTION,	NULL },
-  { "Trace",		CT_INT,		&trace },
-  { "CommentMode",	CT_INT,		&comment_mode },
-  { "QuoteHack",	CT_INT,		&quote_hack },
-  { "ScriptHack",	CT_INT,		&script_hack },
-  { "CharRefHack",	CT_INT,		&char_ref_hack },
-  { "IgnoreUnknownCRs",	CT_INT,		&ignore_unknown_char_refs },
-  { "LogMetas",		CT_INT,		&log_metas },
-  { "MetaCharset",	CT_INT,		&meta_charset },
-  { "RobotComments",	CT_INT,		&robot_comments },
-  { "MetaAttr",		CT_FUNCTION,	add_user_meta },
-  { NULL,		CT_STOP,	NULL }
+static struct cf_section html_config = {
+  CF_ITEMS {
+    CF_UNS("CommentMode", &comment_mode),
+    CF_UNS("QuoteHack", &quote_hack),
+    CF_UNS("ScriptHack", &script_hack),
+    CF_UNS("CharRefHack", &char_ref_hack),
+    CF_UNS("RefreshHack", &refresh_hack),
+    CF_UNS("RefreshThreshold", &refresh_threshold),
+    CF_UNS("IgnoreUnknownCRs", &ignore_unknown_char_refs),
+    CF_UNS("LogMetas", &log_metas),
+    CF_UNS("MetaCharset", &meta_charset),
+    CF_UNS("MetaLanguage", &meta_language),
+    CF_UNS("RobotComments", &robot_comments),
+    CF_LIST("MetaAttr", &user_metas, &html_config_meta),
+    CF_STRING_DYN("IgnoreMetas", &ignored_metas, CF_ANY_NUM),
+    CF_END
+  }
 };
 
 static void CONSTRUCTOR html_init(void)
 {
-  cf_register(html_config);
-  clist_init(&user_metas);
+  cf_declare_section("HTML", &html_config, 0);
 }
 
 /* Global variables */
 
 static struct fastbuf *html_in, *html_out, *meta_out;
 static int head_mode, inside_head, attrs_seen;
-static byte *found_meta_charset;
+static byte *found_meta_charset, *found_meta_language;
+static byte *fake_redirect;
 
 static uns global_robot_control;	/* META ROBOTS */
 static uns local_robot_control;		/* <!-- robots:... --> */
@@ -99,7 +104,13 @@ get_char(void)
       ungot_char = -1;
       return c;
     }
-  return bget_utf8(html_in);
+  else if (head_mode)			/* We don't know the charset yet */
+    {
+      int c = bgetc(html_in);
+      return (c < 0x80) ? c : (0xe000+c);
+    }
+  else					/* Already recoded to UTF-8 */
+    return bget_utf8(html_in);
 }
 
 static inline void
@@ -192,7 +203,7 @@ static struct gobj_ref *
 html_add_ref_full(int type, byte *url, byte *ctype, struct url *base_url, uns nofollow)
 {
   struct gobj_ref *r = gobj_add_ref_full(type, url, ctype, base_url);
-  if (!(local_robot_control & RC_FOLLOW) || nofollow)
+  if (r && (!(local_robot_control & RC_FOLLOW) || nofollow))
     r->dont_follow = 1;
   return r;
 }
@@ -409,6 +420,9 @@ image_add_ref(byte *src, byte *width UNUSED, byte *height UNUSED, uns nofollow U
 
 /* Processing of META tags */
 
+#define MTRACE(x...) do { if (log_metas >= 2) log(L_DEBUG, x); } while (0)
+#define MWARN(x...) do { if (log_metas) log(L_WARN_R, x); } while (0)
+
 static uns
 chew_robot_control(byte *t, uns *maskp)
 {
@@ -470,7 +484,7 @@ chew_robot_control(byte *t, uns *maskp)
 	  mask = RC_FOLLOW | RC_INDEX | RC_ARCHIVE;
 	}
       else if (*p)
-	log(L_WARN_R, "Unknown robot control keyword: %s", p);
+	MWARN("Unknown robot control keyword: %s", p);
       *t = z;
     }
   *maskp = mask;
@@ -489,16 +503,55 @@ add_robot_note(byte *text, uns r, uns mask)
   obj_add_attr(gthis->aa, '.', comment);
 }
 
+static int
+chew_meta_refresh(byte *t)
+{
+  int after = atoi(t);
+  byte *k;
+
+  if (k = strchr(t, ';'))
+    {
+      k++;
+      while (Cspace(*k))
+	k++;
+      if (strncasecmp(k, "URL=", 4))
+	return 0;
+      k += 4;
+    }
+  else if (k = strchr(t, ','))
+    {
+      k++;
+      while (Cspace(*k))
+	k++;
+    }
+  else
+    return 1;
+  if (!*k)
+    return 0;
+
+  if (!refresh_hack || after > (int)refresh_hack)
+    {
+      html_add_ref('R', k+4, 0);
+      return 1;
+    }
+  byte url[MAX_URL_SIZE];
+  if (url_auto_canonicalize_rel(k, url, gobj_base_url()))
+    return 0;
+  if (strcmp(url, gthis->url))
+    fake_redirect = gstrdup(url);
+  return 1;
+}
+
 static void
 chew_meta(byte *name, byte *text)
 {
-  if (log_metas >= 2)
-    log(L_DEBUG, "META %s=%s", name, text);
+  MTRACE("META %s=%s", name, text);
 
   struct user_meta *um;
   CLIST_WALK(um, user_metas)
     if (!strcasecmp(name, um->meta))
       {
+	MTRACE("User-defined META -> attribute %c", um->attr);
 	ASSERT(strlen(text) < MAX_ATTR_SIZE);
 	obj_add_attr(gthis->aa, um->attr, text);
 	return;
@@ -519,50 +572,49 @@ chew_meta(byte *name, byte *text)
     add_meta(text, MT_MISC);
   else if (!strcasecmp(name, "refresh"))
     {
-      byte *k = strchr(text, ';');
-      if (k)
-	{
-	  k++;
-	  while (Cspace(*k))
-	    k++;
-	  if (strncasecmp(k, "URL=", 4))
-	    goto err;
-	  html_add_ref('R', k+4, 0);
-	}
-      else if (k = strchr(text, ','))
-	{
-	  k++;
-	  while (Cspace(*k))
-	    k++;
-	  if (*k)
-	    html_add_ref('R', k, 0);
-	}
+      if (!chew_meta_refresh(text))
+	MWARN("Cannot parse META %s=%s", name, text);
     }
-  else if (!strcasecmp(name, "content-type"))	/* Parsed in head mode */
+  else if (!strcasecmp(name, "content-type") || !strcasecmp(name, "content-language"))	/* Parsed in head mode */
     ;
-  else if (strcasecmp(name, "author")	/* Useless META's */
-	   && strcasecmp(name, "generator")
-	   && strcasecmp(name, "formatter"))
-    goto err;
-  return;
+  else
+    {
+      for (uns i=0; i<DARY_LEN(ignored_metas); i++)
+	if (!strcasecmp(name, ignored_metas[i]))
+	  return;
+      MWARN("Unknown META %s=%s", name, text);
+    }
+}
 
- err:
-  if (log_metas)
-    log(L_WARN_R, "Unknown META %s=%s", name, text);
+static void
+chew_meta_lang(byte *lang)
+{
+  byte buf[MAX_LANG_LIST_SIZE];
+  int n = lang_normalize_list(buf, lang);
+  if (n > 0)
+    {
+      found_meta_language = gstrdup(buf);
+      MTRACE("Found META language %s", found_meta_language);
+    }
+  else if (n < 0)
+    MWARN("Cannot parse META language list `%s'", lang);
 }
 
 static void
 chew_meta_head(byte *name, byte *text)
 {
-  if (strcasecmp(name, "content-type"))
-    return;
-  parse_content_type(text, &found_meta_charset);
-  if (found_meta_charset)
+  MTRACE("HEAD META %s=%s", name, text);
+  if (meta_charset && !strcasecmp(name, "content-type"))
     {
-      found_meta_charset = gstrdup(found_meta_charset);
-      if (trace)
-	log(L_DEBUG, "META charset=%s", found_meta_charset);
+      parse_content_type(text, &found_meta_charset);
+      if (found_meta_charset)
+	{
+	  found_meta_charset = gstrdup(found_meta_charset);
+	  MTRACE("Found META charset %s", found_meta_charset);
+	}
     }
+  else if (meta_language && !strcasecmp(name, "content-language"))
+    chew_meta_lang(text);
 }
 
 /* Chew an already parsed HTML tag */
@@ -590,20 +642,28 @@ skip_script(int is_style)
 
   if (script_hack)			/* Work-around for broken documents */
     {
-      byte *z = is_style ? "</STYLE>" : "</SCRIPT>"; /* Trick: no character occurs twice */
-      int state = 0;
-      while (z[state])
+      byte *end = is_style ? "</STYLE>" : "</SCRIPT>"; /* Trick: no character occurs twice */
+      byte *start = is_style ? "<STYLE>" : "<SCRIPT>";
+      int ei = 0, si = 0;
+      int nesting = 1;
+      while (nesting)
 	{
 	  r = get_char();
 	  if (r < 0)
 	    {
-	      log(L_ERROR_R, "End of file when searching for </script>");
+	      log(L_ERROR_R, "End of file at nesting level %d when searching for </script>", nesting);
 	      return 1;
 	    }
 	  if (r < 128)
 	    r = Cupcase(r);
-	  if (z[state] == r || z[state=0] == r)
-	    state++;
+	  if (start[si] == r || start[si=0] == r)
+	    si++;
+	  if (end[ei] == r || end[ei=0] == r)
+	    ei++;
+	  if (!start[si])
+	    nesting++;
+	  else if (!end[ei])
+	    nesting--;
 	}
     }
   else					/* Standard behaviour */
@@ -674,6 +734,10 @@ chew_tag(byte *name, byte **anames, byte **avals, uns acount)
 	  if ((k = getarg("HTTP-EQUIV")) && (l = getarg("CONTENT")))
 	    chew_meta_head(k, l);
 	  break;
+	case T_HTML:
+	  if ((k = getarg("LANG")) && meta_language)
+	    chew_meta_lang(k);
+	  break;
 	}
       return 0;
     }
@@ -699,6 +763,7 @@ chew_tag(byte *name, byte **anames, byte **avals, uns acount)
   switch (tag & ~(T_HEAD | T_FLOW | T_HARD | T_BOTH))
     {
     case T_IGNORE:
+    case T_HTML:		/* Processed only in the header pass */
       break;
     case T_ON:			/* Switch an attribute on */
       if (t->arg & ATTRIB_ONLY_ONCE & attrs_seen)
@@ -724,15 +789,13 @@ chew_tag(byte *name, byte **anames, byte **avals, uns acount)
     case T_A:				/* Anchor */
       if (t->arg)			/* End */
 	{
-	  if (current_a_ref)
-	    emit_ref_end(current_a_ref);
+	  emit_ref_end(current_a_ref);
 	  current_a_ref = NULL;
 	}
       else if (k = getarg("HREF"))
 	{
 	  struct gobj_ref *ref = html_add_ref_full('R', k, getarg("TYPE"), NULL, rel_nofollow_p());
-	  if (current_a_ref)
-	    emit_ref_end(current_a_ref);
+	  emit_ref_end(current_a_ref);
 	  current_a_ref = ref;
 	  emit_ref_start(current_a_ref);
 	  if (k = getarg("TITLE"))
@@ -773,15 +836,32 @@ chew_tag(byte *name, byte **anames, byte **avals, uns acount)
     case T_OBJECT:			/* An embedded object */
       if (!t->arg)
 	{
-	  k = getarg("CODEBASE");
-	  add_obj_ref(k, getarg("CLASSID"), getarg("CODETYPE"));
-	  add_obj_ref(k, getarg("DATA"), getarg("TYPE"));
-	  /* add_string(getarg("STANDBY"), WT_ALT); */
-	  attrs |= A_OBJECT;
-	  add_string(getarg("TITLE"), WT_ALT);
+	  byte *classid = getarg("CLASSID");
+	  if (classid && !strcasecmp(classid, "clsid:D27CDB6E-AE6D-11cf-96B8-444553540000")) /* Flash movie */
+	    attrs |= A_OBJECT_FLASH;
+	  else
+	    {
+	      k = getarg("CODEBASE");
+	      add_obj_ref(k, classid, getarg("CODETYPE"));
+	      add_obj_ref(k, getarg("DATA"), getarg("TYPE"));
+	      /* add_string(getarg("STANDBY"), WT_ALT); */
+	      attrs |= A_OBJECT;
+	      add_string(getarg("TITLE"), WT_ALT);
+	    }
 	}
       else
-	attrs &= ~A_OBJECT;
+	attrs &= ~(A_OBJECT | A_OBJECT_FLASH);
+      break;
+    case T_PARAM:
+      if (attrs & A_OBJECT_FLASH)
+        {
+	  byte *name = getarg("NAME");
+	  if (name && !strcasecmp(name, "MOVIE"))
+	    html_add_ref_full('A', getarg("VALUE"), "application/x-shockwave-flash", NULL, rel_nofollow_p());
+	}
+      break;
+    case T_EMBED:
+      html_add_ref_full('A', getarg("SRC"), getarg("TYPE"), NULL, rel_nofollow_p());
       break;
     case T_APPLET:			/* An applet */
       add_string(getarg("ALT"), WT_ALT);
@@ -1153,16 +1233,19 @@ chew_html(void)
 }
 
 int
-html_parse(void)
+html_parse(byte **args UNUSED)
 {
   /* Scan the document for META tags carrying charset info if asked */
   found_meta_charset = NULL;
-  if (meta_charset)
+  found_meta_language = NULL;
+  if (meta_charset || meta_language)
     {
       head_mode = 1;
       html_in = fbmem_clone_read(gthis->contents);
       chew_html();
       bclose(html_in);
+      if (found_meta_language)			/* META language is to be trusted over server language */
+	gthis->language = found_meta_language;
     }
 
   /* Guess charset and convert to UTF-8 */
@@ -1171,12 +1254,24 @@ html_parse(void)
   /* Parse the document */
   head_mode = 0;
   attrs_seen = 0;
+  fake_redirect = NULL;
   html_in = fbmem_clone_read(gthis->contents);
   html_out = gthis->text = fbmem_create(16384);
   meta_out = gthis->meta = fbmem_create(256);
   chew_html();
   bclose(html_in);
 
+  if (fake_redirect && btell(gthis->text) < refresh_threshold)
+    {
+      /* Convert the whole page to a redirect if asked for. However, we keep
+       * at least the meta-data for the indexer to use when the destination
+       * page is not gathered yet.
+       */
+      gthis->dont_save_contents = 1;
+      gobj_add_ref('Y', fake_redirect);
+      obj_add_attr(gthis->aa, '.', "META refresh turned into redirect");
+    }
+
   if (!(global_robot_control & RC_INDEX))
     gthis->dont_save_contents = 1;
   if (!(global_robot_control & RC_FOLLOW))
