@@ -2,7 +2,7 @@
  *  Sherlock PDF Parser: Lexical analysis and low-level parsing
  *
  *  (c) 2002--2003 Milan Vancura <milan@ucw.cz>
- *  (c) 2003--2004 Martin Mares <mj@ucw.cz>
+ *  (c) 2003--2005 Martin Mares <mj@ucw.cz>
  */
 
 #include "sherlock/sherlock.h"
@@ -25,9 +25,9 @@ s32 pdf_rootref, pdf_inforef;
 static OBJ_START **xreftroot;
 
 static OBJ_START
-obj_off(int i) {
+obj_off(uns i) {
     OBJ_START out={0,0};
-    int j=i/xreft_size;
+    uns j=i/xreft_size;
 
     if(j>=xreft_size || !xreftroot[j])
 	return out;
@@ -242,7 +242,7 @@ get_int_real(void) {
     if((is_intchar(c=in_get_char()) || c=='.') && (i+=(c=='.'))<2)
 	*b++=c;
     else
-	gerror(2200,"(get_int_real) Wrong int/real number: %s (start)",stringbuf);
+	gerror(2200,"(get_int_real) Wrong int/real number (start)");
     while((is_numchar(c=in_get_char()) || c=='.') && (i+=(c=='.'))<2 && b<e)
 	*b++=c;
     if(c!=EOF)
@@ -404,7 +404,14 @@ get_name(void) {
 		break;
 	    }
 	    if((b[-1]=hexbyte(c,d))==0)
-		gerror(2200,"(get_name) NULL byte not allowed in the object name: %s",stringbuf);
+		gerror(2200,"(get_name) NULL byte not allowed in object name");
+	    /*
+	     * XXX: This is not forbidden by the standard, but using control
+	     * characters in names is evil and it could break our log messages,
+	     * so we are going to disallow it for the time being.
+	     */
+	    if(b[-1] < 0x20)
+		gerror(2200,"(get_name) control character found in object name");
 	}
     }
     if(c!=EOF)
@@ -578,7 +585,7 @@ printobj(OBJECT obj) {
 	case OT_ARRAY:
 	    LOG("%*s(array)",po_shift,"");
 	    po_shift+=3;
-	    for (ap=obj.value.a;ap;ap=ap->next) 
+	    for (ap=obj.value.a;ap;ap=ap->next)
 		printobj(ap->obj);
 	    po_shift-=3;
 	    break;
@@ -597,7 +604,7 @@ printobj(OBJECT obj) {
 	    LOG("%*s(stream)",po_shift,"");
 	    break;
 	case OT_NULL:
-	    if(obj.value.n) 
+	    if(obj.value.n)
 		LOG("%*s(unknown ref to obj %d)",po_shift,"",obj.value.n);
 	    else
 		LOG("%*s(null)",po_shift,"");
@@ -691,7 +698,7 @@ parse_dict(DICT_ENTRY *dict_start,OBJ_GET *items) {
 static void
 getxrefsec(s32 start) {
     char buf[MAX_BUFF_SIZE], *ptr;
-    int i,j,o_first, o_count;
+    uns i,j,o_first, o_count;
     OBJECT trailer,root,info,encrypt,fileid,prev;
     OBJ_GET trailerdict[]={{"Root",&root},{"Prev",&prev},{"Info",&info},{"Encrypt",&encrypt},{"ID",&fileid},{NULL,NULL}};
     int prevxrefsect=0;
@@ -743,7 +750,7 @@ getxrefsec(s32 start) {
     skip_space(SP_ANYSPACE);
     pdf_read(buf,7);
     if(memcmp(buf,"trailer",7) || skip_space(SP_ANYWCOMM)==0)
-	gerror(2220,"(getxrefsec) Trailer keyword not found: %s at %d",buf,pdf_tell());
+	gerror(2220,"(getxrefsec) Trailer keyword not found at %d",pdf_tell());
     TRACE(200,"The trailer dictionary at %d",pdf_tell());
 
     saveencr=encrypted;
