@@ -48,23 +48,24 @@
 
 /* Configuration parameters */
 
-int pdf_trace;
-int xreft_size=1000;
-int respect_user_rights=1;
-int pdf_warnings=1;
+uns pdf_trace;
+uns xreft_size=1000;
+uns respect_user_rights=1;
+uns pdf_warnings=1;
 
-static struct cfitem pdf_config[] = {
-    { "PDF",		  CT_SECTION, NULL},
-    { "Trace",		  CT_INT,     &pdf_trace },
-    { "XrefTabSize",	  CT_INT,     &xreft_size },
-    { "RespectUserRights",CT_INT,     &respect_user_rights },
-    { "Warnings",	  CT_INT,     &pdf_warnings },
-    { NULL,		  CT_STOP,    NULL}
+static struct cf_section pdf_config = {
+  CF_ITEMS {
+    CF_UNS("Trace", &pdf_trace),
+    CF_UNS("XrefTabSize", &xreft_size),
+    CF_UNS("RespectUserRights", &respect_user_rights),
+    CF_UNS("Warnings", &pdf_warnings),
+    CF_END
+  }
 };
 
 static void CONSTRUCTOR
 pdf_init(void) {
-    cf_register(pdf_config);
+  cf_declare_section("PDF", &pdf_config, 0);
 }
 
 static struct fastbuf *pdf_out, *meta_out;
@@ -208,7 +209,7 @@ parse_resources(OBJECT res, uns robj) {
 	    gs=parse_gstate(gn->obj,restop->fonts);
 	    gn->gsd=gs ? gs->data : NULL;
 	    if(gn->gsd)
-		TRACE(200,"(parse_resources) GState data: %d={obj %d, ctm_sy %d, tf %d, tfs %d, tm_sy %d, trm %d}", gn->namehash, gs->obj, gn->gsd->ctm_sy,(int)gn->gsd->tf, gn->gsd->tfs, gn->gsd->tm_sy, gn->gsd->trm);
+		TRACE(200,"(parse_resources) GState data: %d={obj %d, ctm_sy %d, tf %p, tfs %d, tm_sy %d, trm %d}", gn->namehash, gs->obj, gn->gsd->ctm_sy,gn->gsd->tf, gn->gsd->tfs, gn->gsd->tm_sy, gn->gsd->trm);
 	    else
 		TRACE(200,"(parse_resources) GState data: NULL");
 	}
@@ -228,7 +229,7 @@ change_font(byte *fname, OBJECT fsize, struct fontname *fntop) {
     uns fnh;
 
     if(!fname || !fname[0] || !fntop ||!gscur || (fsize.type!=OT_INT && fsize.type!=OT_REAL)) {
-	TRACE(200,"(change_font) RETURN fname(%s), fntop(%d), fsize.type(%d)",fname,(int)fntop,fsize.type);
+	TRACE(200,"(change_font) RETURN fname(%s), fntop(%p), fsize.type(%d)",fname,fntop,fsize.type);
 	return;
     }
     fnh=hash_string(fname);
@@ -324,7 +325,7 @@ switch_class(uns class) {
     flush_word();
     if (class & (WC_META | WC_TYPE_MASK)) {
 	if ((class ^ queued_class) & WC_META) {
-	    ASSERT(class & WC_BREAK);
+	    class |= WC_BREAK;
 	    current_class = class & WC_META;
 	}
 	queued_class = (queued_class & ~(WC_META | WC_TYPE_MASK)) | (class & (WC_META | WC_TYPE_MASK));
@@ -459,7 +460,7 @@ cs_gettexts(struct resources *res) {
     TRACE(200,"Content stream out text");
     gstack[1]=gstack[0];	/* initializing gs current state */
     gscur=gstack+1;
-    
+
     while(in_check_nschar(SP_ANYWCOMM)!=EOF) {
 	if(!cur) {
 	    cur=start=mp_alloc_zero(page_pool,sizeof(ARRAY_ENTRY));
@@ -885,16 +886,16 @@ out_document(void) {
     if(bookmarkroot.type==OT_OBJREF)
 	out_bookmarks(bookmarkroot.value.n);
     mp_flush(page_pool);
-    
+
     pagescnt=0;
     if(pageroot.type==OT_OBJREF)
 	out_pages(pageroot.value.n,NULL);
 }
 
 int
-pdf_parse(void) {
+pdf_parse(byte **args UNUSED) {
     if(gthis->truncated)
-	gerror(2200, "PDF file truncated");
+	gerror(2205, "PDF file truncated");
 
     pdf_in = fbmem_clone_read(gthis->contents);
     pdf_out = gthis->text = fbmem_create(16384);
