@@ -1,7 +1,7 @@
 /*
  *	Sherlock HTTP Downloader
  *
- *	(c) 1997--2005 Martin Mares <mj@ucw.cz>
+ *	(c) 1997--2006 Martin Mares <mj@ucw.cz>
  *	(c) 2002 Robert Spalek <robert@ucw.cz>
  */
 
@@ -12,6 +12,8 @@
 #include "lib/mempool.h"
 #include "lib/base64.h"
 #include "lib/clists.h"
+#include "lib/simple-lists.h"
+#include "lang/lang.h"
 #include "sherlock/index.h"
 #include "gather/gather.h"
 
@@ -19,11 +21,12 @@
 #include <string.h>
 #include <stdlib.h>
 #include <stdarg.h>
+#include <unistd.h>
 #include <time.h>
 #include <sys/signal.h>
 #include <sys/socket.h>
 #include <netinet/in.h>
-#include <unistd.h>
+#include <netdb.h>
 
 /* Configuration parameters */
 
@@ -41,48 +44,62 @@ static byte *proxy = "";
 static uns proxy_port = 80;
 static uns trace = 0;
 static uns length_checks = 1;
+static byte *user_agent_cmt = "";
 
-struct user_hdr {
-  cnode n;
-  byte *header;
-};
+static clist list_charset, list_encoding, list_lang, list_types;
 static clist user_headers;
 
-static byte *
-add_user_hdr(struct cfitem *c UNUSED, byte *hdr)
+static void
+merge_list(byte **dest, clist *src)
 {
-  if (!hdr[0])
-    return "Empty user header";
-  struct user_hdr *uh = cfg_malloc(sizeof(*uh));
-  uh->header = hdr;
-  clist_add_tail(&user_headers, &uh->n);
+  cf_journal_block(dest, sizeof(byte*));
+  simp_node *n;
+  uns len = 0;
+  CLIST_WALK(n, *src)
+    len += strlen(n->s) + 2;
+  byte *c = *dest = cf_malloc(len + 1);
+  CLIST_WALK(n, *src)
+    c += sprintf(c, "%s, ", n->s);
+  if (c > *dest)
+    c[-2] = 0;
+}
+
+static byte *
+http_commit(void *ptr UNUSED)
+{
+  merge_list(&acc_charset, &list_charset);
+  merge_list(&acc_encoding, &list_encoding);
+  merge_list(&acc_lang, &list_lang);
+  merge_list(&acc_types, &list_types);
   return NULL;
 }
 
-static struct cfitem http_config[] = {
-  { "HTTP",		CT_SECTION,	NULL },
-  { "MaxHeaderLines",	CT_INT, 	&max_hdr_lines },
-  { "From",		CT_STRING,	&local_admin },
-  { "Referer",		CT_STRING,	&referer },
-  { "AcceptCharset",	CT_STRING,	&acc_charset },
-  { "AcceptEncoding",	CT_STRING,	&acc_encoding },
-  { "AcceptLanguage",	CT_STRING,	&acc_lang },
-  { "AcceptTypes",	CT_STRING,	&acc_types },
-  { "ConnectTimeout",	CT_INT, 	&connect_timeout },
-  { "HeaderTimeout",	CT_INT, 	&header_timeout },
-  { "BodyTimeout",	CT_INT, 	&body_timeout },
-  { "UseProxy",		CT_STRING,	&proxy },
-  { "ProxyPort",	CT_INT,		&proxy_port },
-  { "Trace",		CT_INT,		&trace },
-  { "LengthChecks",	CT_INT,		&length_checks },
-  { "Header",		CT_FUNCTION,	add_user_hdr },
-  { NULL,		CT_STOP,	NULL }
+static struct cf_section http_config = {
+  CF_COMMIT(http_commit),
+  CF_ITEMS {
+    CF_UNS("MaxHeaderLines", &max_hdr_lines),
+    CF_STRING("From", &local_admin),
+    CF_STRING("Referer", &referer),
+    CF_LIST("AcceptCharset", &list_charset, &cf_string_list_config),
+    CF_LIST("AcceptEncoding", &list_encoding, &cf_string_list_config),
+    CF_LIST("AcceptLanguage", &list_lang, &cf_string_list_config),
+    CF_LIST("AcceptTypes", &list_types, &cf_string_list_config),
+    CF_UNS("ConnectTimeout", &connect_timeout),
+    CF_UNS("HeaderTimeout", &header_timeout),
+    CF_UNS("BodyTimeout", &body_timeout),
+    CF_STRING("UseProxy", &proxy),
+    CF_UNS("ProxyPort", &proxy_port),
+    CF_UNS("Trace", &trace),
+    CF_UNS("LengthChecks", &length_checks),
+    CF_LIST("Header", &user_headers, &cf_string_list_config),
+    CF_STRING("UserAgentComment", &user_agent_cmt),
+    CF_END
+  }
 };
 
 static void CONSTRUCTOR http_init_config(void)
 {
-  cf_register(http_config);
-  clist_init(&user_headers);
+  cf_declare_section("HTTP", &http_config, 0);
 }
 
 /* Other globals */
@@ -108,11 +125,14 @@ http_connect(byte *host, uns port)
   int sock;
 
   rem.sin_family = AF_INET;
-  rem.sin_addr.s_addr = resolve_host_name(host, 1);
+  rem.sin_addr.s_addr = resolve_host_name(host);
   rem.sin_port = htons(port);
   if (proxy[0])
     {
-      rem.sin_addr.s_addr = resolve_host_name(proxy, 0);
+      struct hostent *h = gethostbyname(proxy);
+      if (!h)
+	gerror(1138, "Unable to resolve name of HTTP proxy (%s)", hstrerror(h_errno));
+      memcpy(&rem.sin_addr.s_addr, h->h_addr, 4);
       rem.sin_port = htons(proxy_port);
       host = proxy;
     }
@@ -123,7 +143,12 @@ http_connect(byte *host, uns port)
     die("No socket (%m)");
   alarm(connect_timeout);
   if (connect(sock, (struct sockaddr *) &rem, sizeof(struct sockaddr_in)) < 0)
-    gerror(1107, "Connect failed (%m)");
+    {
+      if (proxy[0])
+	gerror(1138, "Cannot connect to HTTP proxy (%m)");
+      else
+	gerror(1107, "Connect failed (%m)");
+    }
   TRACE("Connected");
 
   return sock;
@@ -131,8 +156,8 @@ http_connect(byte *host, uns port)
 
 /* Sending of single header */
 
-static void
-sendhdr(FILE *f, byte *txt, ...)
+static void FORMAT_CHECK(printf,2,3)
+sendhdr(FILE *f, char *txt, ...)
 {
   byte buf[1024];
   va_list args;
@@ -373,7 +398,13 @@ send_header(FILE *f, byte *host, uns port, byte *rest)
     }
   if (gthis->if_different_etag)
     sendhdr(f, "If-None-Match: %s", gthis->if_different_etag);
-  sendhdr(f, "User-Agent: holmes/" SHER_VER);
+
+  byte *ua = "User-Agent: holmes/" SHER_VER;
+  if (user_agent_cmt[0])
+    sendhdr(f, "%s (%s)", ua, user_agent_cmt);
+  else
+    sendhdr(f, ua);
+
   if (gthis->auth_user && gthis->auth_pass)
   {
     uns srclen = strlen(gthis->auth_user) + strlen(gthis->auth_pass) + 1;
@@ -384,9 +415,9 @@ send_header(FILE *f, byte *host, uns port, byte *rest)
     destbuf[destlen] = 0;
     sendhdr(f, "Authorization: Basic %s", destbuf);
   }
-  struct user_hdr *uh;
-  CLIST_WALK(uh, user_headers)
-    sendhdr(f, "%s", uh->header);
+
+  CLIST_FOR_EACH(simp_node *, uh, user_headers)
+    sendhdr(f, "%s", uh->s);
   fputs("\r\n" ,f);
   fflush(f);
 }
@@ -526,7 +557,10 @@ parse_hdr(void)
   switch (code)
     {
     case 304:				/* Not modified */
-      gerror(3, "Not modified");
+      if (gthis->if_modified_since_time || gthis->if_different_etag)
+	gerror(3, "Not modified");
+      else
+	gerror(2137, "Unexpected HTTP status %d: %s", code, response_text);
     case 503:				/* Temporarily Unavailable */
       if (l = findhdr("Retry-After:"))
 	obj_set_attr_num(gthis->aa, 'W', (int) atol(l));
@@ -557,14 +591,19 @@ parse_hdr(void)
   else if (l = findhdr("Content-Location:"))
     gthis->base_url = gobj_parse_url(&gthis->base_url_s, l, "base", 1);
   set_content_encoding(findhdr("Content-Encoding:"));
-  gthis->language = findhdr("Content-Language:");
+  if (l = findhdr("Content-Language:"))
+    {
+      byte buf[MAX_LANG_LIST_SIZE];
+      if (lang_normalize_list(buf, l) > 0)
+	gthis->language = gstrdup(buf);
+    }
   if (l = findhdr("Content-Type:"))
     {
       parse_content_type(l, &gthis->charset);
       set_content_type(l);
     }
   /* Let the core check whether we want the content type or not */
-  gather_filter();
+  gather_filter(0);
   if ((l = findhdr("Content-Length:")) && !head_only)
     {
       expected_length = atol(l);
