@@ -5,13 +5,13 @@
  */
 
 #include "sherlock/sherlock.h"
-#include "lib/conf.h"
+#include "lib/getopt.h"
 #include "lib/fastbuf.h"
 #include "indexer/indexer.h"
 
 #include <stdlib.h>
 
-#define SORT_KEY struct fprint
+#define SORT_KEY struct card_print
 #define SORT_PREFIX(x) fp_##x
 #define SORT_PRESORT
 #define SORT_REGULAR
@@ -19,7 +19,7 @@
 #define SORT_OUTPUT_FILE
 
 static inline int
-fp_compare(struct fprint *a, struct fprint *b)
+fp_compare(struct card_print *a, struct card_print *b)
 {
   int e = memcmp(&a->fp, &b->fp, sizeof(a->fp));
   if (e)
