@@ -1,7 +1,7 @@
 /*
  *	Sherlock Indexer -- Processing {URL,File,Catalog} keywords
  *
- *	(c) 2003 Robert Spalek <robert@ucw.cz>
+ *	(c) 2003--2006 Robert Spalek <robert@ucw.cz>
  *
  *	old code taken from centrum/indexer/oook.c by
  *
@@ -16,9 +16,10 @@
  */
 
 #include "sherlock/sherlock.h"
-#include "lib/lists.h"
+#include "lib/clists.h"
 #include "lib/url.h"
 #include "lib/conf.h"
+#include "lib/getopt.h"
 #include "lib/fastbuf.h"
 #include "lib/chartype.h"
 #include "lib/mempool.h"
@@ -28,7 +29,6 @@
 #include "charset/unicat.h"
 #include "indexer/indexer.h"
 #include "indexer/merges.h"
-#include "indexer/attrs.h"
 #include "indexer/lexicon.h"
 
 #include <stdlib.h>
@@ -39,15 +39,15 @@
 /*** Configuration ***/
 
 struct url_pattern {
-  node n;
-  byte use_hardcoded;
+  cnode n;
   regex *rx;
-  byte weight;
-  byte replacement[1];
+  byte *replacement;
+  uns weight;
+  uns use_hardcoded;
 };
 
 static uns trace_level = 0;
-static list url_patterns, file_patterns;
+static clist url_patterns, file_patterns;
 static uns name_words;
 static uns limit_count[5][3][4];
 	/* [type of limitation: 0=urlmax, 1=sitemax, 2=totalmax,
@@ -56,94 +56,65 @@ static uns limit_count[5][3][4];
 	   [weight: 0..3] */
 
 static byte *
-oconf_url_patt(struct cfitem *c, byte *b)
+compile_regexp(uns number UNUSED, byte **pars, regex **rx)
 {
-  byte *w[3];
-  struct url_pattern *h;
-  struct list *where_to_add;
-    
-  if (c->name[0] == 'H')
-  {
-    h = cfg_malloc(sizeof(struct url_pattern));
-    bzero(h, sizeof(struct url_pattern));
-    h->use_hardcoded = 1;
-    where_to_add = (c->name[9] == 'U') ?  &url_patterns : &file_patterns;
-  }
-  else
-  {
-    h = cfg_malloc(sizeof(struct url_pattern) + strlen(w[1]));
-    h->use_hardcoded = 0;
-    if (wordsplit(b, w, 3) != 3)
-      return "URL pattern syntax error";
-    h->rx = rx_compile(w[0], 0);
-    if (w[2][0] >= '0' && w[2][0] <= '3' && !w[2][1])
-      h->weight = w[2][0];
-    else
-      return "Weight out of range";
-    strcpy(h->replacement, w[1]);
-    where_to_add = (c->name[0] == 'U') ?  &url_patterns : &file_patterns;
-  }
-  add_tail(where_to_add, &h->n);
+  cf_journal_block(rx, sizeof(void*));
+  *rx = rx_compile(pars[0], 0);
   return NULL;
 }
 
 static byte *
-do_limit_count(struct cfitem *c, byte *l)
+pattern_commit(struct url_pattern *ptr)
 {
-  byte *w[4];
-  int pars = wordsplit(l, w, 4);
-  if (pars < 0)
-    return "Enter up to 4 numbers for weights 0..3";
-  uns limit, type;
-  switch (c->name[0])
-  {
-    case 'U': limit = 0; break;
-    case 'S': limit = 1; break;
-    case 'T': limit = 2; break;
-    case 'D': limit = 3; break;
-    case 'R': limit = 4; break;
-    default: ASSERT(0); return "";
-  }
-  if (strstr(c->name+1, "URL"))
-    type = 0;
-  else if (strstr(c->name, "File"))
-    type = 1;
-  else if (strstr(c->name, "Catalog"))
-    type = 2;
-  else
-    ASSERT(0);
-  int i;
-  for (i=0; i<pars; i++)
-    limit_count[limit][type][i] = atoi(w[i]);
-  for (; i<4; i++)
-    limit_count[limit][type][i] = -1;
+  if (ptr->weight > 3)
+    return "Weight out of range";
   return NULL;
 }
 
-static struct cfitem keywords_config[] = {
-  { "Keywords",			CT_SECTION,	NULL },
-  { "Trace",			CT_INT,		&trace_level },
-  { "URLWordPattern",		CT_FUNCTION,	oconf_url_patt },
-  { "FileWordPattern",		CT_FUNCTION,	oconf_url_patt },
-  { "HardcodedURLPattern",	CT_FUNCTION,	oconf_url_patt },
-  { "HardcodedFilePattern",	CT_FUNCTION,	oconf_url_patt },
-  { "NameWords",		CT_INT,		&name_words },
-  { "URLMaxURLWords",		CT_FUNCTION,	&do_limit_count },
-  { "URLMaxFileWords",		CT_FUNCTION,	&do_limit_count },
-  { "URLMaxCatalogWords",	CT_FUNCTION,	&do_limit_count },
-  { "SiteMaxURLWords",		CT_FUNCTION,	&do_limit_count },
-  { "SiteMaxFileWords",		CT_FUNCTION,	&do_limit_count },
-  { "SiteMaxCatalogWords",	CT_FUNCTION,	&do_limit_count },
-  { "TotalMaxURLWords",		CT_FUNCTION,	&do_limit_count },
-  { "TotalMaxFileWords",	CT_FUNCTION,	&do_limit_count },
-  { "TotalMaxCatalogWords",	CT_FUNCTION,	&do_limit_count },
-  { "DecreaseURLWords",		CT_FUNCTION,	&do_limit_count },
-  { "DecreaseFileWords",	CT_FUNCTION,	&do_limit_count },
-  { "DecreaseCatalogWords",	CT_FUNCTION,	&do_limit_count },
-  { "RemoveURLWords",		CT_FUNCTION,	&do_limit_count },
-  { "RemoveFileWords",		CT_FUNCTION,	&do_limit_count },
-  { "RemoveCatalogWords",	CT_FUNCTION,	&do_limit_count },
-  { NULL,			CT_STOP,	NULL }
+static struct cf_section pattern_config = {
+  CF_TYPE(struct url_pattern),
+  CF_COMMIT(pattern_commit),
+#define F(x)	PTR_TO(struct url_pattern, x)
+  CF_ITEMS {
+    CF_PARSER("Pattern", F(rx), compile_regexp, 1),
+    CF_STRING("Replace", F(replacement)),
+    CF_UNS("Weight", F(weight)),
+    CF_UNS("Hardcoded", F(use_hardcoded)),
+    CF_END
+  }
+};
+
+static byte *
+keywords_init(void *ptr UNUSED)
+{
+  memset(limit_count, -1, sizeof(limit_count));
+  return NULL;
+}
+
+static struct cf_section keywords_config = {
+  CF_INIT(keywords_init),
+  CF_ITEMS {
+    CF_UNS("Trace", &trace_level),
+    CF_LIST("URLWordPattern", &url_patterns, &pattern_config),
+    CF_LIST("FileWordPattern", &file_patterns, &pattern_config),
+    CF_UNS("NameWords", &name_words),
+    CF_UNS_ARY("URLMaxURLWords", limit_count[0][0], 4),
+    CF_UNS_ARY("URLMaxFileWords", limit_count[0][1], 4),
+    CF_UNS_ARY("URLMaxCatalogWords", limit_count[0][2], 4),
+    CF_UNS_ARY("SiteMaxURLWords", limit_count[1][0], 4),
+    CF_UNS_ARY("SiteMaxFileWords", limit_count[1][1], 4),
+    CF_UNS_ARY("SiteMaxCatalogWords", limit_count[1][2], 4),
+    CF_UNS_ARY("TotalMaxURLWords", limit_count[2][0], 4),
+    CF_UNS_ARY("TotalMaxFileWords", limit_count[2][1], 4),
+    CF_UNS_ARY("TotalMaxCatalogWords", limit_count[2][2], 4),
+    CF_UNS_ARY("DecreaseURLWords", limit_count[3][0], 4),
+    CF_UNS_ARY("DecreaseFileWords", limit_count[3][1], 4),
+    CF_UNS_ARY("DecreaseCatalogWords", limit_count[3][2], 4),
+    CF_UNS_ARY("RemoveURLWords", limit_count[4][0], 4),
+    CF_UNS_ARY("RemoveFileWords", limit_count[4][1], 4),
+    CF_UNS_ARY("RemoveCatalogWords", limit_count[4][2], 4),
+    CF_END
+  }
 };
 
 /*** Extraction of keywords from URL and filename ***/
@@ -269,7 +240,8 @@ extract_url_keywords(byte *url, byte *buf, uns source, uns target, byte type)
 
   byte *orig_buf = buf;
   buf += sprintf(buf, "%c%x %x 0 ", type, source, target);
-  WALK_LIST(p, type == 'U' ? url_patterns : file_patterns)
+  clist *list = type == 'U' ? &url_patterns : &file_patterns;
+  CLIST_WALK(p, *list)
   {
     int wt;
     if (p->use_hardcoded)
@@ -418,7 +390,7 @@ lh_dump(void)
   CMP(attr)
   /* Used with (source,target) or (source_site,target_site).
      It puts original page before redirects and pages from my site before other pages.
-     
+
      Caveat!  This only works if indexer/merges.c does NOT swap source and target vertices
      when merging trees.  */
 
@@ -540,10 +512,9 @@ filter_cheaters(struct fastbuf **F, uns consider_total_limit)
 {
   struct fastbuf *in = *F;
   struct fastbuf *out = bopen_tmp(indexer_fb_size);
-  keyword_t last, this;
+  keyword_t last = { .word = ~0U, .target = 0, .source_site = 0, .source = 0 }, this;
   uns url_cnt = 0, site_cnt = 0, total_cnt = 0;
   bzero(deleted, sizeof(deleted));
-  last.word = ~0U;
   while (bread(in, &this, sizeof(keyword_t)))
   {
     if (last.word == ~0U || last.target != this.target
@@ -702,17 +673,20 @@ dump_keywords(struct fastbuf *labels, byte *buf, keyword_t *rec, byte min_wt)
     return;
   static byte types[3] = { MT_URL_KEYWD, MT_FILE,
 #ifdef	MT_CAT_KEYWD
-    MT_CAT_KEYWD 
+    MT_CAT_KEYWD
 #endif
   };
   uns idx = type_to_index(rec->type);
-  bputl(labels, rec->source);
-  bputc(labels, 'M');
+  byte start[3], *s = start;
   if (rec->type == 'U' || rec->type == 'F')
-    bputc(labels, min_wt);	//FIXME: weight changes inside the attribute instead of min_wt
-  bputc(labels, 0x90 + types[idx]);
-  bputs0(labels, buf);
-  bputc(labels, 0);
+    *s++ = min_wt;		//FIXME: weight changes inside the attribute instead of min_wt
+  *s++ = 0x90 + types[idx];
+  *s = 0;
+
+  bputl(labels, rec->source);
+  bputc(labels, LABEL_TYPE_URL);
+  bput_attr_format(labels, 'M', "%s%s", start, buf);
+  bput_attr_separator(labels);
 }
 
 static void
@@ -787,10 +761,7 @@ main(int argc, char **argv)
   int opt;
 
   log_init(argv[0]);
-  init_list(&url_patterns);
-  init_list(&file_patterns);
-  cf_register(keywords_config);
-  memset(limit_count, -1, sizeof(limit_count));
+  cf_declare_section("Keywords", &keywords_config, 0);
   while ((opt = cf_getopt(argc, argv, short_opts, CF_NO_LONG_OPTS, NULL)) >= 0)
     switch (opt)
     {
@@ -815,18 +786,24 @@ main(int argc, char **argv)
     usage();
 
   attrs_part_map(0);
+  notes_part_map(0);
 
   if (phases & 1)
   {
     log(L_INFO, "Going to parse URLs into URL and File keywords");
     keywords = index_bopen(fn_keywords, O_RDWR | O_CREAT | (truncate&1 ? O_TRUNC : O_APPEND));
 
+    bitarray_t is_image = xmalloc(BIT_ARRAY_BYTES(card_count));
+    bit_array_zero(is_image, card_count);
+    for (uns i=0; i<card_count; i++)
+      if (bring_note(i)->flags & CARD_NOTE_IMAGE)
+	bit_array_set(is_image, i);
+
     merges_map(0);
-    bitarray_t is_image;
-    READ_ATTR_BIT(is_image, flags, CARD_FLAG_IMAGE);
     generate_url_keywords(is_image);
-    xfree(is_image);
     merges_unmap();
+
+    xfree(is_image);
     bflush(keywords);
   }
   else
@@ -872,6 +849,7 @@ main(int argc, char **argv)
     /* Dump the keywords into labels.  */
     resort(&binary, 4);
     struct fastbuf *labels = index_bopen(fn_labels_by_id, O_WRONLY | O_CREAT | (truncate&2 ? O_TRUNC : O_APPEND));
+    put_attr_set_type(BUCKET_TYPE_V33);
     dump_into_labels(labels, binary);
     bclose(labels);
     bclose(binary);
@@ -879,6 +857,7 @@ main(int argc, char **argv)
     xfree(lh_words);
   }
   bclose(keywords);
+  notes_part_unmap();
   attrs_part_unmap();
 
   return 0;
