@@ -1,7 +1,7 @@
 /*
  *	Sherlock Indexer -- Lexicon Ordering
  *
- *	(c) 2001--2003 Martin Mares <mj@ucw.cz>
+ *	(c) 2001--2005 Martin Mares <mj@ucw.cz>
  *
  *	This module reorders the lexicon to get words which are likely
  *	to be processed together (e.g. if they differ only in accents)
@@ -12,7 +12,7 @@
 #undef LOCAL_DEBUG
 
 #include "sherlock/sherlock.h"
-#include "lib/conf.h"
+#include "lib/getopt.h"
 #include "lib/fastbuf.h"
 #include "lib/hashfunc.h"
 #include "lib/math.h"
@@ -146,7 +146,7 @@ find_class_rep(uns *cls, uns n UNUSED)
 	    {
 	      struct ctxt_info *c = &ctxt_info[GET_CONTEXT(&v->context_class)];
 	      struct verbum *w = c->root;
-	      bprintf(f_classes, "%s [%d]%s\n", w->word, (struct ctxt_verbum *)v - c->contexts, r);
+	      bprintf(f_classes, "%s [%d]%s\n", w->word, (int)((struct ctxt_verbum *)v - c->contexts), r);
 	    }
 	}
     }
@@ -239,8 +239,6 @@ static void
 generate_context_words(void)
 {
   log(L_INFO, "Generating slots for %d context-dependent words", n_context_words);
-  if (lex_context_slots > CONFIG_MAX_CONTEXTS)
-    die("Lexicon.ContextSlots is larger than CONFIG_MAX_CONTEXTS.");
   if (!n_context_words)
     return;
 
@@ -347,13 +345,13 @@ generate_stems(void)
   struct stemmer *st;
   struct mempool *mp;
 
-  if (EMPTY_LIST(stemmer_list))
+  if (clist_empty(&stemmer_list))
     return b;
 
   log(L_INFO, "Searching for stems");
   mp = mp_new(4096);
   lang_init_stemmers();
-  WALK_LIST(st, stemmer_list)
+  CLIST_WALK(st, stemmer_list)
     {
       DBG("Trying stemmer %d langmask %08x", st->id, st->lang_mask);
       bputl(b, st->id);
@@ -364,11 +362,12 @@ generate_stems(void)
 	{
 	  struct verbum *v = word_array[i];
 	  struct word_node *stem;
-	  list *stems;
+	  clist *stems;
 	  PROGRESS(i, "%s: %d words of %d, created %d new stems", st->name, i, n_words, (lh_id-old_id)/8);
 	  mp_flush(mp);
-	  if (stems = lang_stem(st, v->word, mp))
-	    WALK_LIST(stem, *stems)
+	  struct word_node req = { .word_form = WORD_FORM_OTHER, .stem_form = WORD_FORM_LEMMA, .unaccented = 0, .w = v->word };
+	  if (stems = lang_stem(st, &req, mp))
+	    CLIST_WALK(stem, *stems)
 	      {
 		struct verbum *w = lh_lookup_utf8(stem->w);
 		if (v == w)
@@ -399,14 +398,14 @@ generate_syn_classes(struct fastbuf *b)
   struct mempool *mp;
   byte **cls;
 
-  if (EMPTY_LIST(syndict_list))
+  if (clist_empty(&syndict_list))
     return;
 
   mp = mp_new(4096);
   log(L_INFO, "Constructing synonymic classes");
   struct fastbuf *c = bopen(b->name, O_RDONLY, indexer_fb_size);
   bsetpos(c, btell(b));
-  WALK_LIST(s, syndict_list)
+  CLIST_WALK(s, syndict_list)
     {
       uns nclasses = 0;
       uns nwords = 0;
@@ -694,7 +693,7 @@ main(int argc, char **argv)
     if ((l->id & 7) == WC_CONTEXT)
       n_context_words++;
   n_all_words = n_words + 2*lex_context_slots*n_context_words;
-  word_array = xmalloc(sizeof(uns) * n_all_words);
+  word_array = xmalloc(sizeof(struct verbum *) * n_all_words);
   DBG("Gathering IDs for %d words", n_all_words);
   id_to_word = xmalloc(sizeof(struct verbum *) * (n_all_words+1));
   uns cnt = 0;
