@@ -5,7 +5,7 @@
  */
 
 #include "sherlock/sherlock.h"
-#include "lib/conf.h"
+#include "lib/getopt.h"
 #include "lib/fastbuf.h"
 #include "indexer/indexer.h"
 
