@@ -1,19 +1,22 @@
 /*
  *	Sherlock Indexer -- Initial Object Scanning
  *
- *	(c) 2001--2005 Martin Mares <mj@ucw.cz>
- *	(c) 2002--2004 Robert Spalek <robert@ucw.cz>
+ *	(c) 2001--2006 Martin Mares <mj@ucw.cz>
+ *	(c) 2002--2005 Robert Spalek <robert@ucw.cz>
+ *	(c) 2006 Pavel Charvat <pchar@ucw.cz>
  */
 
 #include "sherlock/sherlock.h"
-#include "lib/conf.h"
+#include "lib/getopt.h"
 #include "lib/fastbuf.h"
 #include "lib/md5.h"
 #include "lib/mempool.h"
 #include "lib/url.h"
+#include "lib/bitarray.h"
+#include "lib/math.h"
 #include "sherlock/object.h"
+#include "sherlock/attrset.h"
 #include "sherlock/tagged-text.h"
-#include "sherlock/bucket.h"
 #include "lib/unicode.h"
 #include "lib/hashfunc.h"
 #include "charset/unicat.h"
@@ -21,6 +24,8 @@
 #include "indexer/matcher.h"
 #include "indexer/params.h"
 #include "filter/filter.h"
+#include "analyser/analyser.h"
+#include "lang/lang.h"
 #include "lang/detect.h"
 
 #include <string.h>
@@ -32,11 +37,17 @@ static struct mempool *scanner_pool;
 static struct index_params parameters;
 
 struct scan_filter_data {
-  int site_id, bonus, area;
+  int bonus, card_bonus, area;
   byte *url;
   struct url url_s;
+  uns queue_key;
   byte *language;
   byte *title;
+  byte *site_name;
+  int site_level;
+  int image_size;
+  int image_aspect_ratio;
+  int image_colors;
 };
 static struct filter_args *scan_filter_args;
 static struct scan_filter_data scan_filter_data;
@@ -50,11 +61,20 @@ struct filter_binding scan_bindings[] = {
   { "path",		OFFSETOF(struct scan_filter_data, url_s.rest) },
   { "username",		OFFSETOF(struct scan_filter_data, url_s.user) },
   { "password",		OFFSETOF(struct scan_filter_data, url_s.pass) },
+  /* Gatherer attributes */
+  { "queue_key",	OFFSETOF(struct scan_filter_data, queue_key) },
   /* Attributes */
   { "bonus",		OFFSETOF(struct scan_filter_data, bonus) },
-  { "siteid",		OFFSETOF(struct scan_filter_data, site_id) },
+  { "card_bonus",	OFFSETOF(struct scan_filter_data, card_bonus) },
+  { "site",		OFFSETOF(struct scan_filter_data, site_name) },
+  { "site_level",	OFFSETOF(struct scan_filter_data, site_level) },
+#ifdef CONFIG_LANG
   { "language",		OFFSETOF(struct scan_filter_data, language) },
+#endif
   { "title",		OFFSETOF(struct scan_filter_data, title) },
+  { "image_size",	OFFSETOF(struct scan_filter_data, image_size) },
+  { "image_aspect_ratio",	OFFSETOF(struct scan_filter_data, image_aspect_ratio) },
+  { "image_colors",	OFFSETOF(struct scan_filter_data, image_colors) },
   { NULL,		0 }
 };
 
@@ -64,7 +84,17 @@ scan_filter_init(void)
   if (!indexer_filter_name || !indexer_filter_name[0])
     return;
   scan_filter_args = filter_intr_new(filter_load(indexer_filter_name, filter_builtin_vars, scan_bindings, NULL));
-  filter_intr_undo_init(scan_filter_args);
+}
+
+static uns
+get_queue_key(struct odes *obj)
+{
+  struct oattr *a = obj_find_attr(obj, 'k');;
+  if (!a)
+    return F_UNDEF_INT;
+  uns v;
+  sscanf(a->val, "%x", &v);
+  return v;
 }
 
 static byte *
@@ -86,6 +116,29 @@ find_title(struct odes *obj UNUSED, uns raw UNUSED)
   return NULL;
 }
 
+static void
+get_image_attrs(struct scan_filter_data *d, struct odes *obj)
+{
+  struct oattr *attrs = obj_find_attr(obj, 'G');
+  if (attrs)
+    {
+      uns width, height, ncolors;
+      byte colorspace[MAX_ATTR_SIZE];
+      sscanf(attrs->val, "%d%d%s%d", &width, &height, colorspace, &ncolors);
+
+      /* should be safe because of gatherer's limits */
+      d->image_size = sqrtf(width * height);
+      d->image_aspect_ratio = (width > height) ? (width << 10) / height : (height << 10) / width;
+
+      if (!strcasecmp(colorspace, "GRAY"))
+	d->image_colors = 0;
+      else
+	d->image_colors = ncolors;
+    }
+  else
+    d->image_size = d->image_aspect_ratio = d->image_colors = F_UNDEF_INT;
+}
+
 static int
 scan_filter(byte *url, struct odes *obj)
 {
@@ -93,13 +146,20 @@ scan_filter(byte *url, struct odes *obj)
   struct scan_filter_data *d = &scan_filter_data;
   byte buf1[MAX_URL_SIZE], buf2[MAX_URL_SIZE];
 
-  d->site_id = 0;
+  d->site_name = NULL;
+  d->site_level = 0;
   d->bonus = 0;
+  d->card_bonus = 0;
   d->url = url;
   if (url_canon_split(d->url, buf1, buf2, &d->url_s))
     die("scan_filter: error parsing URL");
-  d->language = obj_find_aval(obj, 'l');
+  d->queue_key = get_queue_key(obj);
+  if (!obj_find_aval(obj, 'X') && !obj_find_aval(obj, 'M'))
+    d->language = NULL;
+  else
+    d->language = an_lang_decide_language(obj);
   d->title = NULL;
+  get_image_attrs(d, obj);
   d->area = 0;
   if (!a)
     return 1;
@@ -110,75 +170,135 @@ scan_filter(byte *url, struct odes *obj)
   return filter_intr_run(a);
 }
 
-static inline void
-scan_filter_undo(void)
+static struct fastbuf *scan_fb_text, *scan_fb_metas;
+
+static void
+scan_analyse_init(void)
 {
-  if (scan_filter_args)
-    filter_intr_undo(scan_filter_args);
+  analyser_init(AN_HOOK_SCANNER, AN_NEED_TEXT | AN_NEED_METAS | AN_NEED_ALL_URLS);
+  if (an_hook_need_mask & AN_NEED_TEXT)
+    scan_fb_text = fbgrow_create(4096);
+  if (an_hook_need_mask & AN_NEED_METAS)
+    scan_fb_metas = fbgrow_create(4096);
+}
+
+static void
+scan_analyse(struct odes *o)
+{
+  struct an_iface ai = {
+    .obj = o,
+    .url_block = o,
+    .all_urls = (struct odes *[]) { o, NULL },
+  };
+
+  uns need = analyser_need(&ai);
+  if (need)
+    {
+      if (need & AN_NEED_TEXT)
+	{
+	  fbgrow_reset(scan_fb_text);
+	  for (struct oattr *a = obj_find_attr(o, 'X'); a; a=a->same)
+	    {
+	      bwrite(scan_fb_text, a->val, str_len(a->val));	// bputs(), but with str_len()
+	      if (a->same)
+		bputc(scan_fb_text, ' ');
+	    }
+	  fbgrow_rewind(scan_fb_text);
+	  ai.text = scan_fb_text;
+	}
+      if (need & AN_NEED_METAS)
+	{
+	  fbgrow_reset(scan_fb_metas);
+	  for (struct oattr *a = obj_find_attr(o, 'M'); a; a=a->same)
+	    bwrite(scan_fb_metas, a->val, str_len(a->val));
+	  fbgrow_rewind(scan_fb_metas);
+	  ai.metas = scan_fb_metas;
+	}
+      analyser_run_needed(&ai);
+    }
+}
+
+static void
+scan_analyse_end(void)
+{
+  analyser_log_stats();
+  bclose(scan_fb_text);
+  bclose(scan_fb_metas);
 }
 
 static void
 prepare_parameters(void)
 {
   parameters.ref_time = time(NULL);
+  parameters.database_version = parameters.ref_time;
 }
 
 static void
-write_parameters(void)
+gen_fingerprint(struct fastbuf *b, byte *url, uns id, struct card_attr *ca UNUSED)
 {
-  struct fastbuf *params = bopen(index_name(fn_parameters), O_CREAT | O_TRUNC | O_WRONLY, 1024);
-  bwrite(params, &parameters, sizeof(parameters));
-  bclose(params);
-}
-
-static void
-gen_fingerprint(struct fastbuf *b, byte *url, uns id)
-{
-  struct fprint fp;
+  struct card_print fp;
 
   url_fingerprint(url, &fp.fp);
   fp.cardid = id;
   bwrite(b, &fp, sizeof(fp));
+#ifdef CONFIG_MERGING_HASHES
+  if (ca)
+    memcpy(ca->merging_hash, &fp.fp, SHERLOCK_MERGING_HASH_SIZE);
+#endif
 }
 
 static void
-gen_labels_by_id(struct fastbuf *b, struct odes *o, byte *url, uns id)
+gen_labels_by_id(struct fastbuf *b, struct odes *o, byte *url, uns id, struct card_note *note UNUSED)
 {
   struct oattr *a, *v;
+  uns overrides = 0;
 
   bputl(b, id);
-  bputc(b, 'U');
-  bputs0(b, url);
+  bputc(b, LABEL_TYPE_URL | LABEL_FLAG_MERGED_ONLY);
+  bput_attr_str(b, 'U', url);
   for (a=o->attrs; v=a; a=a->next)
-    if (strchr(label_attrs, a->attr))
-      while (v)
-	{
-	  bputc(b, v->attr);
-	  bputs0(b, v->val);
-	  v = v->same;
-	}
-  bputc(b, 0);
+    {
+      if (attr_set_match(&label_attr_set, a))
+	bput_oattr(b, a);
+      if (attr_set_match(&override_label_attr_set, a))
+	overrides |= 1;
+      if (attr_set_match(&override_body_attr_set, a))
+	overrides |= 2;
+    }
+  bput_attr_separator(b);
+
+  for (uns i=0; i<2; i++)
+    if (overrides & (1 << i))
+      {
+	bputl(b, id);
+	bputc(b, (i ? LABEL_TYPE_BODY : LABEL_TYPE_URL) | LABEL_FLAG_OVERRIDE);
+	for (a=o->attrs; v=a; a=a->next)
+	  if (attr_set_match((i ? &override_body_attr_set : &override_label_attr_set), a))
+	    bput_oattr(b, a);
+	bput_attr_separator(b);
+      }
+
   if (frameset_to_redir && obj_find_attr(o, 'F'))
     {
+      bputl(b, id);
+      bputc(b, LABEL_TYPE_URL);
       byte *title = find_title(o, 1);
       if (title)
-        {
-	  bputl(b, id);
-	  bputc(b, 'M');
-	  bputs0(b, title);
-	  bputc(b, 0);
-	}
+	bput_attr_str(b, 'M', title);
+      bput_attr_str(b, '.', "frameset");
+      bput_attr_separator(b);
     }
+
 }
 
 static void
 gen_checksums(struct fastbuf *b, struct odes *o, uns id, struct card_attr *attr, struct card_note *note)
 {
+  struct csum csum = { .cardid = id };
   word buf[4096];
   uns cnt = 0;
   struct oattr *a;
   struct MD5Context ct;
-  struct csum csum;
   uns accents = 0;
   uns chars = 0;
   uns lastc = ' ';
@@ -223,7 +343,7 @@ gen_checksums(struct fastbuf *b, struct odes *o, uns id, struct card_attr *attr,
 
   if (a = obj_find_attr(o, 'N'))
     {
-      attr->flags |= CARD_FLAG_IMAGE;
+      note->flags |= CARD_NOTE_IMAGE;
       for (; a; a=a->same)
 	{
 	  byte *z = a->val;
@@ -233,6 +353,14 @@ gen_checksums(struct fastbuf *b, struct odes *o, uns id, struct card_attr *attr,
 	}
     }
 
+  for (a = obj_find_attr(o, 'A'); a; a=a->same)
+    {
+      byte *z = a->val;
+      uns len = str_len(z);
+      MD5Update(&ct, z, len + 1);
+      /* chars += len; */
+    }
+
   if (accents > chars/128)
     attr->flags |= CARD_FLAG_ACCENTED;
   if (!chars)
@@ -243,7 +371,6 @@ gen_checksums(struct fastbuf *b, struct odes *o, uns id, struct card_attr *attr,
   MD5Update(&ct, (byte *) buf, cnt * sizeof(buf[0]));
 
   MD5Final(csum.md5, &ct);
-  csum.cardid = id;
   bwrite(b, &csum, sizeof(csum));
 }
 
@@ -251,7 +378,7 @@ static void
 gen_links(struct fastbuf *b, struct odes *o, uns id, struct card_note *note)
 {
   for (struct oattr *a=o->attrs; a; a=a->next)
-    if (strchr(link_attrs, a->attr))
+    if (attr_set_match(&link_attr_set, a))
       for (struct oattr *v=a; v; v=v->same)
 	{
 	  note->flags |= CARD_NOTE_HAS_LINKS;
@@ -279,42 +406,41 @@ gen_links(struct fastbuf *b, struct odes *o, uns id, struct card_note *note)
 }
 
 #if defined(CONFIG_LANG) && defined(CONFIG_FILETYPE)
-#include "lang/lang.h"
 static void
-gen_lang(struct card_attr *ca, struct oattr *attr_X)
+gen_lang(struct card_attr *ca, struct scanner_note *incr)
 {
-  byte *lang = scan_filter_data.language;
-
   if (!FILETYPE_IS_TEXT(CA_GET_FILE_TYPE(ca)))
     return;
-
-  int id;
-  if (!lang || !lang[0])
-  {
-    if (auto_lang)
-      {
-	lang_detect_start();
-	for (struct oattr *a=attr_X; a; a=a->same)
-	  lang_detect_add_string(a->val);
-	id = lang_detect_choose_best();
-      }
-    else
-      id = 0;
-  }
-  else
-    id = lang_list_to_code(lang);
-  if (id > 0)
+  if (incr)
+    {
+      ca->type_flags = incr->type_flags;
+      return;
+    }
+  byte *lang = scan_filter_data.language;
+  int id = lang ? lang_primary_language(lang) : -1;
+  if (id >= 0)
     ca->type_flags |= id;
 }
 #else
-static void gen_lang(struct card_attr *ca UNUSED, struct oattr *attr_X UNUSED) { }
+static void gen_lang(struct card_attr *ca UNUSED, struct scanner_note *incr UNUSED) { }
 #endif
 
-static void
-gen_attrs(struct odes *o, struct card_attr *ca, struct card_note *cn)
+static inline byte
+initial_weight(struct odes *o)
 {
-  ca->weight = CLAMP(default_weight + scan_filter_data.bonus, 0, 255);
+  byte *w;
+  uns initial_weight = default_weight;
+  if (w = obj_find_aval(o, 'w'))
+    initial_weight = atoi(w);
+  return CLAMP(initial_weight + scan_filter_data.bonus, 0, 255);
+}
+
+static void
+gen_attrs(struct odes *o, struct card_attr *ca, struct card_note *cn, struct scanner_note *incr)
+{
+  ca->weight = initial_weight(o);
   cn->weight_scanner = ca->weight;
+  cn->card_bonus = CLAMP(scan_filter_data.card_bonus, -32767, 32767);
 
   if (frameset_to_redir && obj_find_attr(o, 'F'))
     ca->flags |= CARD_FLAG_FRAMESET | CARD_FLAG_EMPTY;
@@ -328,7 +454,7 @@ gen_attrs(struct odes *o, struct card_attr *ca, struct card_note *cn)
 #endif
 
   custom_create_attrs(o, ca);
-  gen_lang(ca, obj_find_attr(o, 'X'));	/* Needs type_flags set by custom_create_attrs */
+  gen_lang(ca, incr);			/* Needs type_flags set by custom_create_attrs */
 }
 
 static void
@@ -340,10 +466,10 @@ gen_signatures(struct fastbuf *signatures, struct odes *o, struct hash_permuter
 
   oa = obj_find_attr(o, 'X');
   if (!oa)
-	  return;
+    return;
   words = matcher_compute_minima(sign, permuter, oa);
   if (words < matcher_min_words)
-	  return;
+    return;
   bwrite(signatures, &id, sizeof(uns));
   bwrite(signatures, sign, sizeof(sign));
 }
@@ -384,24 +510,25 @@ gen_ref_texts(struct fastbuf *reftexts, struct odes *o, uns id)
   byte *rthis[MAX_REF_DEPTH], *rlimit[MAX_REF_DEPTH];
 
   byte *b, *c, *l;
-  struct oattr *a;
+  struct oattr *aa, *a;
   byte *wstart, *wend;
   uns nalphas;
 
   /* Find all references we're interested in */
-  for (b=ref_link_types; *b; b++)
-    for (a=obj_find_attr(o, *b); a; a=a->same)
-      {
-	byte url[MAX_URL_SIZE];
-	int rid = parse_reference(a->val, url);
-	if (rid < 0 || rid >= MAX_REFS)
-	  continue;
-	DBG("Ref <%s> id %d", url, rid);
-	while (last_ref <= (uns) rid)
-	  ref_word_types[last_ref++] = 0;
-	ref_word_types[rid] = MT_EXT;
-	url_fingerprint(url, &fp[rid]);
-      }
+  for (aa=o->attrs; aa; aa=aa->next)
+    if (attr_set_match(&ref_link_attr_set, aa))
+      for (a=aa; a; a=a->same)
+	{
+	  byte url[MAX_URL_SIZE];
+	  int rid = parse_reference(a->val, url);
+	  if (rid < 0 || rid >= MAX_REFS)
+	    continue;
+	  DBG("Ref <%s> id %d", url, rid);
+	  while (last_ref <= (uns) rid)
+	    ref_word_types[last_ref++] = 0;
+	  ref_word_types[rid] = MT_EXT;
+	  url_fingerprint(url, &fp[rid]);
+	}
 
   /* And scan the text for bracketed parts */
   for (a=obj_find_attr(o, 'X'); a; a=a->same)
@@ -527,7 +654,26 @@ xtoi(uns c)
   else if (c >= 'A' && c <= 'F')
     return c - 'A' + 10;
   else
-    ASSERT(0);
+    {
+      ASSERT(0);
+      return 0;
+    }
+}
+
+static char *short_opts = CF_SHORT_OPTS "i:";
+static char *help = "\
+Usage: scanner [<options>]\n\
+\n\
+Options:\n"
+CF_USAGE
+"-i directory\t\tPath to the previous index for incremental scanner\n\
+";
+
+static void NONRET
+usage(void)
+{
+  fputs(help, stderr);
+  exit(1);
 }
 
 int
@@ -548,19 +694,26 @@ main(int argc, char **argv)
 
   log_init(argv[0]);
   setproctitle_init(argc, argv);
-  if (cf_getopt(argc, argv, CF_SHORT_OPTS, CF_NO_LONG_OPTS, NULL) >= 0 ||
-      optind < argc)
-  {
-    fputs("This program supports only the following command-line arguments:\n" CF_USAGE, stderr);
-    exit(1);
-  }
+  int opt;
+  byte *incr_dir = NULL;
+  while ((opt = cf_getopt(argc, argv, short_opts, CF_NO_LONG_OPTS, NULL)) >= 0)
+    switch (opt)
+      {
+	case 'i':
+	  incr_dir = optarg;
+	  break;
+	default:
+	  usage();
+      }
+  if (optind < argc)
+    usage();
 
-#ifdef CONFIG_LANG
-  lang_detect_build_automaton();
-#endif
+  matcher_init();
+  scan_analyse_init();
   prepare_parameters();
   fingerprints = index_maybe_bopen(fn_fingerprints, O_CREAT | O_TRUNC | O_WRONLY);
   labels_by_id = index_maybe_bopen(fn_labels_by_id, O_CREAT | O_TRUNC | O_WRONLY);
+  put_attr_set_type(BUCKET_TYPE_V33);
   attributes = index_bopen(fn_attributes, O_CREAT | O_TRUNC | O_WRONLY);
   checksums = index_maybe_bopen(fn_checksums, O_CREAT | O_TRUNC | O_WRONLY);
   links = index_maybe_bopen(fn_links, O_CREAT | O_TRUNC | O_WRONLY);
@@ -568,11 +721,10 @@ main(int argc, char **argv)
   merges = index_maybe_bopen(fn_merges, O_CREAT | O_TRUNC | O_WRONLY);
   if (matcher_signatures && index_name_defined(fn_signatures))
   {
-    if (matcher_signatures % 2)
-      die("Matcher.Signatures must be even");
     signatures = index_bopen(fn_signatures, O_CREAT | O_TRUNC | O_WRONLY);
     permuter = permuter_new();
-    srand(time(NULL));
+    parameters.srand = time(NULL);
+    parameters.last_download = 0;
   }
   else
     signatures = NULL, permuter = NULL;
@@ -580,10 +732,13 @@ main(int argc, char **argv)
   notes = index_bopen(fn_notes, O_CREAT | O_TRUNC | O_WRONLY);
   notes_new = index_maybe_bopen(fn_notes_new, O_CREAT | O_TRUNC | O_WRONLY);
 
+  struct scanner_note *incr = NULL;
+
   scanner_pool = mp_new(16384);
   struct bucket_source *src = get_buck_init();
   scan_filter_init();
   url_key_init();
+  srand(parameters.srand);
   log(L_INFO, "Scanning objects");
 
   id = count_in = count_bots = count_ok = count_new = count_err = 0;
@@ -614,6 +769,7 @@ main(int argc, char **argv)
 	for (uns i=0; i<16; i++)
 	  note.footprint[i] = xtoi(footprint[2*i]) << 4 | xtoi(footprint[2*i+1]);
       }
+      scan_analyse(o);
       if (scan_filter(url, o))
 	{
 	  int ok = 1;
@@ -638,8 +794,8 @@ main(int argc, char **argv)
 	      if (notes_new)
 		{
 		  if (fingerprints)
-		    gen_fingerprint(fingerprints, url, id_new);
-		  note.weight_scanner = CLAMP(default_weight + scan_filter_data.bonus, 0, 255);
+		    gen_fingerprint(fingerprints, url, id_new, NULL);
+		  note.weight_scanner = initial_weight(o);
 		  bwrite(notes_new, &note, sizeof(note));
 		  id_new++;
 		}
@@ -648,15 +804,15 @@ main(int argc, char **argv)
 	    {
 	      count_ok++;
 	      if (fingerprints)
-		gen_fingerprint(fingerprints, url, id);
+		gen_fingerprint(fingerprints, url, id, &attr);
 	      if (labels_by_id)
-		gen_labels_by_id(labels_by_id, o, url, id);
-	      if (checksums)
+		gen_labels_by_id(labels_by_id, o, url, id, &note);
+	      if (checksums && !incr)
 		gen_checksums(checksums, o, id, &attr, &note);
 	      if (links)
 		gen_links(links, o, id, &note);
-	      gen_attrs(o, &attr, &note);
-	      if (signatures)
+	      gen_attrs(o, &attr, &note, incr);
+	      if (signatures && !incr)
 	        gen_signatures(signatures, o, permuter, id);
 	      if (reftexts)
 		gen_ref_texts(reftexts, o, id);
@@ -669,30 +825,27 @@ main(int argc, char **argv)
 	      id++;
 	    }
 	}
-      scan_filter_undo();
     }
 
   src->cleanup(src);
   parameters.objects_in = count_ok + count_err + count_bots;
-  write_parameters();
+  params_save(&parameters);
   bclose(fingerprints);
   bclose(labels_by_id);
   bclose(attributes);
-  bclose(checksums);
   bclose(links);
   bclose(urls);
   bclose(merges);
-  if (signatures)
-  {
-    bclose(signatures);
-    xfree(permuter);
-  }
   bclose(reftexts);
   bclose(notes);
   bclose(notes_new);
   mp_delete(scanner_pool);
 
+  bclose(checksums);
+  bclose(signatures);
+
   log(L_INFO, "Scanned %d objects (%d ok, %d err, %d robots, %d new)", count_in, count_ok, count_err, count_bots, count_new);
   log(L_INFO, "Created %d cards and %d notes on new URL's", id, id_new - FIRST_ID_NEW);
+  scan_analyse_end();
   return 0;
 }
