@@ -5,7 +5,7 @@
  */
 
 #include "sherlock/sherlock.h"
-#include "lib/conf.h"
+#include "lib/getopt.h"
 #include "lib/fastbuf.h"
 #include "indexer/indexer.h"
 #include "indexer/params.h"
@@ -24,14 +24,10 @@ main(int argc, char **argv)
     exit(1);
   }
 
-  struct fastbuf *fi = bopen(index_name(fn_parameters), O_RDWR, sizeof(struct index_params));
   struct index_params params;
-  int f = breadb(fi, &params, sizeof(params));
-  ASSERT(f);
+  params_load(&params);
   params.version = INDEX_VERSION;
-  brewind(fi);
-  bwrite(fi, &params, sizeof(params));
-  bclose(fi);
+  params_save(&params);
 
   log(L_INFO, "Index sealed");
   return 0;
