@@ -2,10 +2,12 @@
  *	Language detector --- Computing detection tables
  *
  * 	(c) 2003, Robert Spalek <robert@ucw.cz>
+ *	(c) 2006, Martin Mares <mj@ucw.cz>
  */
 
 #include "sherlock/sherlock.h"
 #include "lib/conf.h"
+#include "lib/getopt.h"
 #include "lib/mempool.h"
 #include "lib/fastbuf.h"
 #include "sherlock/object.h"
@@ -13,7 +15,6 @@
 #include "sherlock/tagged-text.h"
 #include "lib/unicode.h"
 #include "lang/lang.h"
-#include "lang/kmp.h"
 #include "lang/detect.h"
 #include "charset/unicat.h"
 #include "indexer/indexer.h"
@@ -42,48 +43,30 @@ static byte *bucket_state_file, *freq_file, *coef_file, *threshold_file;
 static uns training_ratio = 500;
 static uns seq_length = 2;
 static uns nr_best = 20;
-static enum lang_flag lang_flag[MAX_LANGUAGES];
+static uns accent_lang_set, noaccent_lang_set;
 
+static enum lang_flag lang_flag[MAX_LANGUAGES];
 static uns detect_lang_count[3];		/* number of languages */
 static int lang2pos[MAX_LANGUAGES][2];		/* position of the language in the frequency array */
 static int pos2lang[MAX_LANGUAGES*2];		/* language at the given position */
 static enum seq_flag pos2flag[MAX_LANGUAGES*2];	/* and its flag */
 
-static byte *
-add_lang(struct cfitem *c, byte *arg)
-{
-	byte *w[MAX_LANGUAGES];
-	int n;
-	if ((n = wordsplit(arg, w, ARRAY_SIZE(w))) < 0)
-		return "<= 32 fields expected";
-	for (int i=0; i<n; i++)
-	{
-		int code = lang_name_to_code(w[i]);
-		if (code < 0)
-			return "Unknown language";
-		ASSERT(code < MAX_LANGUAGES);
-		if (lang_flag[code])
-			return "Language already registered";
-		lang_flag[code] = c->name[0]=='A' ? LF_ACCENT : LF_NOACCENT;
+static struct cf_section lang_tables_config = {
+	CF_ITEMS {
+		CF_UNS("Trace", &trace),
+		CF_UNS("Progress", &my_progress),
+		CF_STRING("Filter", &my_filter_name),
+		CF_STRING("BucketStateFile", &bucket_state_file),
+		CF_STRING("FrequencyFile", &freq_file),
+		CF_STRING("CoefficientFile", &coef_file),
+		CF_STRING("ThresholdFile", &threshold_file),
+		CF_UNS("TrainingRatio", &training_ratio),
+		CF_UNS("MaxSequenceLength", &seq_length),
+		CF_UNS("NumberOfBestSeq", &nr_best),
+		CF_PARSER("AccentLanguages", &accent_lang_set, lang_cf_parse_set, CF_ANY_NUM),
+		CF_PARSER("NoAccentLanguages", &noaccent_lang_set, lang_cf_parse_set, CF_ANY_NUM),
+		CF_END
 	}
-	return NULL;
-}
-
-static struct cfitem prepare_lang_config[] = {
-	{ "LangTables",		CT_SECTION,	NULL },
-	{ "Trace",		CT_INT,		&trace },
-	{ "Progress",		CT_INT,		&my_progress },
-	{ "Filter",		CT_STRING,	&my_filter_name },
-	{ "BucketStateFile",	CT_STRING,	&bucket_state_file },
-	{ "FrequencyFile",	CT_STRING,	&freq_file },
-	{ "CoefficientFile",	CT_STRING,	&coef_file },
-	{ "ThresholdFile",	CT_STRING,	&threshold_file },
-	{ "TrainingRatio",	CT_INT,		&training_ratio },
-	{ "MaxSequenceLength",	CT_INT,		&seq_length },
-	{ "NumberOfBestSeq",	CT_INT,		&nr_best },
-	{ "AccentLanguages",	CT_FUNCTION,	add_lang },
-	{ "NoAccentLanguages",	CT_FUNCTION,	add_lang },
-	{ NULL,			CT_STOP,	NULL }
 };
 
 static void
@@ -92,7 +75,14 @@ compute_translation_tables(void)
 	bzero(detect_lang_count, sizeof(detect_lang_count));
 	for (uns i=0; i<MAX_LANGUAGES; i++)
 	{
-		ASSERT(lang_flag[i] < 3);
+		int af = (accent_lang_set & (1 << i));
+		int nf = (noaccent_lang_set & (1 << i));
+		if (af && nf)
+			die("Language `%s' declared as both accent and noaccent", lang_code_to_name(i));
+		if (af)
+			lang_flag[i] = LF_ACCENT;
+		else if (nf)
+			lang_flag[i] = LF_NOACCENT;
 		detect_lang_count[ lang_flag[i] ]++;
 	}
 	memset(lang2pos, -1, sizeof(lang2pos));
@@ -229,11 +219,11 @@ add_character(uns c)
 	if (!Ualpha(c))
 	{
 		/* Translate sequences of control characters into one
-		 * CONTROL_CHAR.  */
+		 * KMP_CONTROL_CHAR.  */
 		if (non_letter)
 			return;
 		non_letter = 1;
-		c = CONTROL_CHAR;
+		c = ':';
 	}
 	else
 	{
@@ -274,17 +264,17 @@ static void
 doc_line(byte *line)
 {
 	byte *line_end = line + strlen(line);
-	add_character(CONTROL_CHAR);
+	add_character(':');
 	for (byte *pos=line; pos<line_end; )
 	{
 		uns c;
 		GET_TAGGED_CHAR(pos, c);
 		if (c >= 0x80000000)
-			add_character(CONTROL_CHAR);
+			add_character(':');
 		else
 			add_character(c);
 	}
-	add_character(CONTROL_CHAR);
+	add_character(':');
 }
 
 static void
@@ -319,11 +309,9 @@ my_filter_init(void)
 {
 	if (!my_filter_name || !my_filter_name[0])
 		return NULL;
-	struct filter_args *a = filter_intr_new(
+	return filter_intr_new(
 		filter_load(my_filter_name, filter_builtin_vars, my_bindings, filter_builtin_func)
 	);
-	filter_intr_undo_init(a);
-	return a;
 }
 
 static void
@@ -357,13 +345,6 @@ my_filter_get_language(struct filter_args *a, byte *url, struct odes *obj, struc
 		return NULL;
 }
 
-static void
-my_filter_undo(struct filter_args *a)
-{
-	if (a)
-		filter_intr_undo(a);
-}
-
 /***** Computing the sequence frequencies *****/
 
 static void
@@ -395,11 +376,10 @@ compute_frequencies(struct filter_args *fa)
 		/* Parse the language of the object.  */
 		byte *url = obj_find_aval(obj, 'U');
 		byte *lang = my_filter_get_language(fa, url, obj, obj_pool);
-		int lang_code = lang ? lang_list_to_code(lang) : -1;
+		int lang_code = lang ? lang_primary_language(lang) : -1;
 		if (lang_code >= 0 && !lang_flag[lang_code])
 			lang_code = -1;
 		lang = lang_code >= 0 ? lang_code_to_name(lang_code) : (byte*) "??";
-		my_filter_undo(fa);
 
 		byte train = 0;
 		bprintf(b_buck, "%2s ", lang);
@@ -561,7 +541,7 @@ compute_coefficients(void)
 	time(&t);
 	struct tm *tm = localtime(&t);
 	strftime(buf, MAXBUF, "%Y-%m-%d %H:%M:%S", tm);
-	bprintf(b_coef, "[LangDetect]\n\
+	bprintf(b_coef, "LangDetect {\n\
 #\tGenerated at %s by lang-tables\n\n\
 Reset\t\t\t%d\n\
 MinDocumentLength\t%d\n\n", buf, seq_total_count, lang_detect_min_doc_length);
@@ -585,6 +565,7 @@ MinDocumentLength\t%d\n\n", buf, seq_total_count, lang_detect_min_doc_length);
 		bputc(b_coef, '\n');
 	}
 	HASH_END_FOR;
+	bputsn(b_coef, "}");
 	bclose(b_coef);
 	printf("Exported %d best sequences\n\n", seq_total_count);
 
@@ -877,7 +858,7 @@ main(int argc, char **argv)
 	int opt;
 	uns seed = 1;
 
-	cf_register(prepare_lang_config);
+	cf_declare_section("LangTables", &lang_tables_config, 0);
 	log_init(argv[0]);
 	while ((opt = cf_getopt(argc, argv, short_opts, CF_NO_LONG_OPTS, NULL)) >= 0)
 		switch (opt)
