@@ -4,7 +4,7 @@
  *	(c) 2003 Martin Mares <mj@ucw.cz>
  */
 
-#include "sherlock/sherlock.h"
+#include "lib/lib.h"
 #include "lib/fastbuf.h"
 #include "lib/mempool.h"
 #include "lang/lang.h"
