@@ -1,7 +1,7 @@
 /*
  *	UCW Library -- Fast Buffered I/O on Static Buffers
  *
- *	(c) 2003--2004 Martin Mares <mj@ucw.cz>
+ *	(c) 2003--2006 Martin Mares <mj@ucw.cz>
  *
  *	This software may be freely distributed and used according to the terms
  *	of the GNU Lesser General Public License.
@@ -10,12 +10,27 @@
 #include "lib/lib.h"
 #include "lib/fastbuf.h"
 
+#include <stdlib.h>
+
 static int
 fbbuf_refill(struct fastbuf *f UNUSED)
 {
   return 0;
 }
 
+static void
+fbbuf_seek(struct fastbuf *f, sh_off_t pos, int whence)
+{
+  /* Somebody might want to seek to the end of buffer, try to be nice to him. */
+  sh_off_t len = f->bufend - f->buffer;
+  if (whence == SEEK_END)
+    pos += len;
+  ASSERT(pos >= 0 && pos <= len);
+  f->bptr = f->buffer + pos;
+  f->bstop = f->bufend;
+  f->pos = len;
+}
+
 void
 fbbuf_init_read(struct fastbuf *f, byte *buf, uns size, uns can_overwrite)
 {
@@ -25,7 +40,7 @@ fbbuf_init_read(struct fastbuf *f, byte *buf, uns size, uns can_overwrite)
   f->pos = size;
   f->refill = fbbuf_refill;
   f->spout = NULL;
-  f->seek = NULL;
+  f->seek = fbbuf_seek;
   f->close = NULL;
   f->config = NULL;
   f->can_overwrite_buffer = can_overwrite;
