@@ -13,25 +13,22 @@
 
 #include "lib/lib.h"
 
-/* An equivalent of the Intel's rol instruction.  */
-#define	ROL(x, bits)	(((x) << (bits)) | ((x) >> (sizeof(uns)*8 - (bits))))
-
 /* The following functions need str to be aligned to uns.  */
-uns str_len_aligned(const byte *str) CONST;
-uns hash_string_aligned(const byte *str) CONST;
-uns hash_block_aligned(const byte *str, uns len) CONST;
+uns str_len_aligned(const byte *str) PURE;
+uns hash_string_aligned(const byte *str) PURE;
+uns hash_block_aligned(const byte *str, uns len) PURE;
 
 #ifdef	CPU_ALLOW_UNALIGNED
 #define	str_len(str)		str_len_aligned(str)
 #define	hash_string(str)	hash_string_aligned(str)
 #define	hash_block(str, len)	hash_block_aligned(str, len)
 #else
-uns str_len(const byte *str) CONST;
-uns hash_string(const byte *str) CONST;
-uns hash_block(const byte *str, uns len) CONST;
+uns str_len(const byte *str) PURE;
+uns hash_string(const byte *str) PURE;
+uns hash_block(const byte *str, uns len) PURE;
 #endif
 
-uns hash_string_nocase(const byte *str) CONST;
+uns hash_string_nocase(const byte *str) PURE;
 
 /*
  *  We hash integers by multiplying by a reasonably large prime with
@@ -39,7 +36,8 @@ uns hash_string_nocase(const byte *str) CONST;
  *  of using shifts and adds on architectures where multiplication
  *  instructions are slow).
  */
-static inline uns CONST hash_int(uns x) { return 0x01008041*x; }
-static inline uns CONST hash_pointer(void *x) { return hash_int((uns)x); }
+static inline uns CONST hash_u32(uns x) { return 0x01008041*x; }
+static inline uns CONST hash_u64(u64 x) { return hash_u32((uns)x ^ (uns)(x >> 32)); }
+static inline uns CONST hash_pointer(void *x) { return ((sizeof(x) <= 4) ? hash_u32((uns)(addr_int_t)x) : hash_u64((u64)(addr_int_t)x)); }
 
 #endif
