@@ -23,6 +23,8 @@
 #define sh_mmap(a,l,p,f,d,o) mmap64(a,l,p,f,d,o)
 #define sh_pread pread64
 #define sh_pwrite pwrite64
+#define sh_stat stat64
+typedef struct stat64 sh_stat_t;
 
 #else	/* !CONFIG_LFS */
 
@@ -32,6 +34,8 @@
 #define sh_mmap(a,l,p,f,d,o) mmap(a,l,p,f,d,o)
 #define sh_pread pread
 #define sh_pwrite pwrite
+#define sh_stat stat
+typedef struct stat sh_stat_t;
 
 #endif	/* !CONFIG_LFS */
 
