@@ -6,6 +6,7 @@
 
 #include "sherlock/sherlock.h"
 #include "lib/mempool.h"
+#include "lib/stkstring.h"
 #include "sherlock/index.h"
 #include "lib/unicode.h"
 #include "indexer/params.h"
@@ -15,15 +16,16 @@
 #include "lang/lang.h"
 #endif
 
-#include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
 #include <time.h>
 #include <setjmp.h>
+#include <unistd.h>
 
 struct query *current_query;
 struct database *current_dbase;
 static jmp_buf query_err_jmp;
+static u64 incarnation;
 
 /*** Profiling counters ***/
 
@@ -68,14 +70,12 @@ profiler_show(struct query *q)
     x += sprintf(x, " wrds:%d phrs:%d near:%d chns:%d chKB:%u",
 		 q->nwords, q->nphrases, q->nnears,
 		 q->stat_num_chains, q->stat_len_chains / 1024);
-  uns l = x - stats + 1;
-  q->profile_stats = mp_alloc(q->pool, l);
-  memcpy(q->profile_stats, stats, l);
-  add_qr("T%s", stats);
+  q->profile_stats = mp_memdup(q->pool, stats, x-stats+1);
+  add_footer("T%s", stats);
 #endif
 
   q->time_total = get_timer();
-  add_qr("t%d", q->time_total);
+  add_footer("t%d", q->time_total);
 }
 
 prof_t *
@@ -112,9 +112,9 @@ format_expr(struct query *q, struct expr *e, byte *buf, byte *bend, uns lastop,
     case EX_MATCH:
       if (buf + 32 + strlen(e->u.match.word) > bend)
 	return NULL;
-      buf += sprintf(buf, "\"%s\"[C%08x T%d S%d W%d A%d M%d P%d Y%d X%Ld]",
+      buf += sprintf(buf, "\"%s\"[T%08x t%d S%d W%d A%d M%d P%d Y%d X%Ld]",
 		     e->u.match.word,
-		     e->u.match.classmap,
+		     e->u.match.type_mask,
 		     e->u.match.is_string,
 		     e->u.match.sense,
 		     e->u.match.o.weight,
@@ -130,13 +130,14 @@ format_expr(struct query *q, struct expr *e, byte *buf, byte *bend, uns lastop,
 	}
       break;
     case EX_OPTIONS:
-      buf += sprintf(buf, "OPTIONS(W=%d A=%d M=%d P=%d Y=%d X=%Ld ",
+      buf += sprintf(buf, "OPTIONS(W=%d A=%d M=%d P=%d Y=%d X=%Ld T=%08x ",
 		     e->u.options.o.weight,
 		     e->u.options.o.accent_mode,
 		     e->u.options.o.morphing,
 		     e->u.options.o.spelling,
 		     e->u.options.o.synonyming,
-		     e->u.options.o.syn_expand);
+		     e->u.options.o.syn_expand,
+		     e->u.options.o.default_word_types);
       buf = format_expr(q, e->u.options.inside, buf, bend, EX_OPTIONS, mode);
       if (!buf || buf + 2 >= bend)
 	buf = NULL;
@@ -172,11 +173,11 @@ format_expr(struct query *q, struct expr *e, byte *buf, byte *bend, uns lastop,
 	  *buf = 0;
 	}
       break;
-    case EX_REF_WORD:
-      buf += sprintf(buf, "REFW(%d)", e->u.ref.index);
+    case EX_WORD:
+      buf += sprintf(buf, "W(%d)", e->u.word.w->index);
       break;
-    case EX_REF_PHRASE:
-      buf += sprintf(buf, "REFP(%d)", e->u.ref.index);
+    case EX_PHRASE:
+      buf += sprintf(buf, "P(%d)", e->u.phrase.p->index);
       break;
     default:
       ASSERT(0);
@@ -193,7 +194,7 @@ debug_expr(struct query *q, struct expr *e, uns attr)
     return;
   if (!format_expr(q, e, buf, buf+sizeof(buf)-1, EX_RESERVED, 1))
     strcpy(buf, "<too long>");
-  add_qr(".%c: %s", attr, buf);
+  add_reply(".%c: %s", attr, buf);
 }
 
 static void
@@ -227,7 +228,7 @@ debug_simple(struct query *q, clist *l, uns attr)
     *b = 0;
   else
     strcpy(buf, "<too long>");
-  add_qr(".%c: %s", attr, buf);
+  add_reply(".%c: %s", attr, buf);
 }
 
 /*** Propagation of query options ***/
@@ -241,6 +242,10 @@ propagate_opts(struct expr *e, struct options *o)
     {
     case EX_MATCH:
       merge_options(&e->u.match.o, o, &e->u.match.o);
+      if (e->u.match.type_mask == ~0U)
+	e->u.match.type_mask = e->u.match.o.default_word_types;
+      e->u.match.is_string = !!(e->u.match.type_mask & 0x8000);
+      e->u.match.type_mask &= 0xffff00ff;
       e->u.match.next_simple = propagate_opts(e->u.match.next_simple, o);
       return e;
     case EX_OPTIONS:
@@ -273,9 +278,9 @@ format_norm_query(struct query *q, byte *buf, uns size)
 {
   byte *bend = buf + size - 1;
 
-  buf += sprintf(buf, "[D%08x S%x M%d A%d P%d T%d O%d a%d G%d,%d X%x] ",
-		 q->db_mask,
-		 q->site_only,
+  buf += sprintf(buf, "[D%d S%llx M%d A%d P%d T%d O%d a%d G%d,%d X%x] ",
+		 q->debug,
+		 q->site_hash,
 		 q->site_max,
 		 q->allow_approx,
 		 q->partial_answers,
@@ -296,7 +301,16 @@ format_norm_query(struct query *q, byte *buf, uns size)
 #undef SMALL_SET_ATTR
 #undef LATE_INT_ATTR
 #undef LATE_SMALL_SET_ATTR
-  buf += CUSTOM_MATCH_CACHE_KEY(q, buf);
+
+  byte *custom_key = CUSTOM_MATCH_CACHE_KEY(q, q->pool);
+  if (custom_key)
+    {
+      uns len = strlen(custom_key);
+      if (buf + len >= bend)
+	return NULL;
+      memcpy(buf, custom_key, len + 1);
+      buf += len;
+    }
   return format_expr(q, q->expr, buf, bend, EX_RESERVED, 1);
 }
 
@@ -329,50 +343,73 @@ cache_hash_fn(byte *x)
 static struct results *
 lookup_cache(struct query *q)
 {
-  byte norm[IOBUF_SIZE];
-  uns hash;
+  byte norm[4096];
   struct results *r;
+  struct results *recycle = NULL;
   time_t now = time(NULL);
   struct mempool *pool;
 
   if (!format_norm_query(q, norm, sizeof(norm)))
     {
-      add_qerr("-101 Normal form of query too long");
+      add_err("-101 Normal form of query too long");
       return NULL;
     }
-  add_qr(".N: %s", norm);
+  if (q->debug & DEBUG_ANALYSE)
+    add_reply(".N: %s (db_mask=%x, need %d results)", norm, q->db_mask, q->needed_results);
 
-  hash = cache_hash_fn(norm);
+  uns hash = cache_hash_fn(norm);
   CLIST_WALK(r, cache_hash[hash])
-    if (!strcmp(r->request, norm) && !(q->debug & DEBUG_NOCACHE))
+    if (!strcmp(r->request, norm) &&
+	(q->needed_results ? (r->db_mask == q->db_mask)
+			   : ((r->db_mask & q->db_mask) == q->db_mask)))
       {
-	clist_remove(&r->n);
-	clist_add_tail(&cache_lru, &r->n);
-	r->access_time = now;
-	return r;
+	if ((q->debug & DEBUG_NOCACHE) ||
+	    r->max_matches < q->needed_results && r->num_matches == r->max_matches)
+	  {
+	    /*
+	     * Currently, we are not smart enough to be able to re-use query
+	     * analysis and just produce more results, because it would involve
+	     * modifying already generated database headers in the reply.
+	     */
+	    recycle = r;
+	  }
+	else
+	  {
+	    clist_remove(&r->n);
+	    clist_add_tail(&cache_lru, &r->n);
+	    r->access_time = now;
+	    return r;
+	  }
       }
 
-  if (cache_count < cache_size)
+  if (!recycle && cache_count >= cache_size)
+    recycle = SKIP_BACK(struct results, n, clist_head(&cache_lru));
+
+  if (recycle)
+    {
+      clist_remove(&recycle->h);
+      clist_remove(&recycle->n);
+      pool = recycle->pool;
+      mp_flush(pool);
+    }
+  else
     {
       pool = mp_new(4096);
       cache_count++;
     }
-  else
-    {
-      r = SKIP_BACK(struct results, n, clist_head(&cache_lru));
-      clist_remove(&r->h);
-      clist_remove(&r->n);
-      pool = r->pool;
-      mp_flush(pool);
-    }
+
   r = mp_alloc_zero(pool, sizeof(struct results));
   clist_add_tail(&cache_hash[hash], &r->h);
   clist_add_tail(&cache_lru, &r->n);
   r->pool = pool;
   r->access_time = r->create_time = now;
-  r->request = mp_alloc(pool, strlen(norm)+1);
-  strcpy(r->request, norm);
-  init_reply_buf(&r->rbuf, pool);
+  r->request = mp_strdup(pool, norm);
+  r->db_mask = q->db_mask;
+  if (q->needed_results > num_matches)
+    r->max_matches = max_matches;
+  else if (q->needed_results)
+    r->max_matches = num_matches;
+  init_reply_buf(&r->reply_header, pool);
   r->status = -1;
   return r;
 }
@@ -425,7 +462,7 @@ simple_fold(clist *l)
 
   if (!maybes && !sharps)
     {
-      add_cerr("-119 Simple search expression contains only non-indexed words");
+      add_err("-119 Simple search expression contains only non-indexed words");
       eval_err(319);
     }
   if (!maybes || !sharps)
@@ -477,7 +514,7 @@ analyse_query(struct query *q, struct expr *e)
 
 /*** Taking care of words ***/
 
-int
+struct word *
 lookup_word(struct query *q, struct expr *e, byte *wd)
 {
   uns i;
@@ -485,8 +522,8 @@ lookup_word(struct query *q, struct expr *e, byte *wd)
 
   for (i=0; i<q->nwords; i++)
     {
-      w = &q->words[i];
-      if (w->type_mask == e->u.match.classmap &&
+      w = q->words[i];
+      if (w->type_mask == e->u.match.type_mask &&
 	  w->is_string == e->u.match.is_string &&
 	  w->options.accent_mode == e->u.match.o.accent_mode &&
 	  w->options.morphing == e->u.match.o.morphing &&
@@ -496,65 +533,59 @@ lookup_word(struct query *q, struct expr *e, byte *wd)
 	  !strcmp(w->word, wd))
 	{
 	  w->use_count++;
-	  return i;
+	  return w;
 	}
     }
   if (q->nwords >= max_words)
     {
-      add_cerr("-103 Too many words");
+      add_err("-103 Too many words");
       eval_err(103);
     }
-  w = &q->words[q->nwords++];
-  bzero(w, sizeof(*w));
-  w->type_mask = e->u.match.classmap;
+  w = mp_alloc_zero(q->results->pool, sizeof(*w));
+  q->words[q->nwords] = w;
+  w->index = q->nwords++;
+  w->type_mask = e->u.match.type_mask;
   w->is_string = e->u.match.is_string;
   memcpy(&w->options, &e->u.match.o, sizeof(struct options));
-  w->word = wd;
+  w->word = mp_strdup(q->results->pool, wd);
   w->use_count = 1;
-  clist_init(&w->variants);
-  return i;
-}
-
-void
-add_hilited_word(struct query *q, byte *w)
-{
-  struct results *r = q->results;
-  struct hilite_word *h = mp_alloc(r->pool, sizeof(*h) + strlen(w));
-  h->next = r->first_hilite;
-  r->first_hilite = h;
-  strcpy(h->w, w);
+  w->boolean_id = -1;
+  return w;
 }
 
 static void
 debug_words(struct query *q)
 {
+  if (!(q->debug & DEBUG_ANALYSE))
+    return;
+
   for (uns i=0; i<q->nwords; i++)
     {
-      struct word *w = &q->words[i];
-      add_qr(".W%d: <%s> class=%08x is_string=%d outer=%d wei=%d acc=%d mor=%d spl=%d syn=%d sex=%Lx stat=%d docs=%d refs=%d vars=%d wcl=%d uc=%d hc=%d wild=%d",
-	     i,
-	     w->word,
-	     w->type_mask,
-	     w->is_string,
-	     w->is_outer,
-	     w->weight,
-	     w->options.accent_mode,
-	     w->options.morphing,
-	     w->options.spelling,
-	     w->options.synonyming,
-	     w->options.syn_expand,
-	     w->status,
-	     w->doc_count,
-	     w->ref_count,
-	     w->var_count,
-	     w->word_class,
-	     w->use_count,
-	     w->hide_count,
-	     w->is_wild);
+      struct word *w = q->words[i];
+      add_reply(".W%d: <%s> class=%08x is_string=%d outer=%d wei=%d acc=%d mor=%d spl=%d syn=%d sex=%Lx stat=%d docs=%d vars=%d wcl=%d uc=%d hc=%d wild=%d",
+		i,
+		w->word,
+		w->type_mask,
+		w->is_string,
+		w->is_outer,
+		w->weight,
+		w->options.accent_mode,
+		w->options.morphing,
+		w->options.spelling,
+		w->options.synonyming,
+		w->options.syn_expand,
+		w->status,
+		w->doc_count,
+		w->var_count,
+		w->word_class,
+		w->use_count,
+		w->hide_count,
+		w->is_wild);
+      word_dump_variants(w);
     }
   for (uns i=0; i<q->nphrases; i++)
     {
-      struct phrase *p = &q->phrases[i];
+      struct phrase *p = q->phrases[i];
       byte buf[1024], *z = buf;
       for (uns j=0; j<p->length; j++)
 	z += sprintf(z, "(%d,%d%s)",
@@ -562,81 +593,120 @@ debug_words(struct query *q)
 		     p->relpos[j],
 		     (p->prox_map & (1 << j)) ? "+" : "");
       *z = 0;
-      add_qr(".P: W%d [%s]", p->weight, buf);
+      add_reply(".P: W%d [%s]", p->weight, buf);
     }
   for (uns i=0; i<q->nnears; i++)
     {
-      struct phrase *p = &q->nears[i];
+      struct phrase *p = q->nears[i];
       byte buf[1024], *z = buf;
       for (uns j=0; j<p->length; j++)
 	z += sprintf(z, "(%d,%d)",
 		     p->word[j],
 		     p->relpos[j]);
       *z = 0;
-      add_qr(".n: [%s]", buf);
+      add_reply(".n: [%s]", buf);
     }
 }
 
 /*** Check words for errors and assign word ids ***/
 
+static void
+fixup_words(struct query *q)
+{
+  /* XXX: Make sure that all words have at least one variant (this is needed
+   * at several places, most notably in the fulltext matcher). For normal words,
+   * word_expand() takes care of it, but ignored words which don't have their
+   * ph_word at all are missed. If it's the case, synthesize the dummy variant here.
+   * Hopefully this will disappear soon with a rewrite of words.c.
+   */
+  for (uns i=0; i<q->nwords; i++)
+    {
+      struct word *w = q->words[i];
+      if (!w->is_string)
+	word_ensure_variants(w);
+    }
+}
+
 static void
 check_words(struct query *q)
 {
-  uns i, j, id;
+  uns i, j;
+  uns id = 0;
+  uns tracing = (q->debug & DEBUG_ANALYSE);
 
-  id = 0;
+  q->query_word_count = 0;
   for (i=0; i<q->nwords; i++)
     {
-      struct word *w = &q->words[i];
+      struct word *w = q->words[i];
       if (w->use_count && w->status && !q->allow_approx)
 	{
 	  switch (w->status)
 	    {
 	    case 105:
-	      add_cerr("-105 Too many word matches for <%s>", w->word);
+	      add_err("-105 Too many word matches for <%s>", w->word);
 	      break;
 	    case 111:
-	      add_cerr("-111 Wildcard pattern too long");
+	      add_err("-111 Wildcard pattern too long");
 	      break;
 	    case 113:
-	      add_cerr("-113 Maximum wildcard zone exceeded");
+	      add_err("-113 Maximum wildcard zone exceeded");
 	      break;
 	    case 114:
-	      add_cerr("-114 Wildcard prefix <%s> too short", w->word);
+	      add_err("-114 Wildcard prefix <%s> too short", w->word);
 	      break;
 	    case 116:
-	      add_cerr("-116 Word <%s> not indexed", w->word);
+	      add_err("-116 Word <%s> not indexed", w->word);
 	      break;
 	    default:
-	      add_cerr("-%d Unknown error for word <%s>", w->status, w->word);
+	      add_err("-%d Unknown error for word <%s>", w->status, w->word);
 	    }
 	  eval_err(w->status + 200);
 	}
-      word_add_hilites(q, w);
-      if (w->ref_count)
-	w->boolean_id = id++;
+
+      /* Assign the boolean ID only when there are variants which can be matched */
+      uns var_count = 0;
+      SLIST_FOR_EACH(struct variant *, v, w->variants)
+	if (v->refchain_len)
+	  var_count++;
+      w->boolean_id = (var_count ? (int)id++ : -1);
+      if (w->use_count > w->hide_count)
+	q->query_word_count++;
+      if (tracing)
+	add_reply(".B Word <%s>: bool ID %d", w->word, w->boolean_id);
     }
   for (i=0; i<q->nphrases; i++)
     {
-      struct phrase *p = &q->phrases[i];
+      struct phrase *p = q->phrases[i];
       p->word_mask = 0;
       for (j=0; j<p->length; j++)
-	p->word_mask |= 1 << q->words[p->word[j]].boolean_id;
-      p->boolean_id = id++;
+	{
+	  struct word *w = q->words[p->word[j]];
+	  p->word_mask |= 1 << ((w->boolean_id < 0) ? 31 : w->boolean_id);
+	}
+      p->boolean_id = ((p->word_mask & 0x80000000) ? -1 : (int)id++);
+      if (tracing)
+	add_reply(".B Phrase %d: bool ID %d, mask %08x", i, p->boolean_id, p->word_mask);
     }
   for (i=0; i<q->nnears; i++)
     {
-      struct phrase *p = &q->nears[i];
+      struct phrase *p = q->nears[i];
       p->word_mask = 0;
       for (j=0; j<p->length; j++)
-	p->word_mask |= 1 << q->words[p->word[j]].boolean_id;
+	{
+	  struct word *w = q->words[p->word[j]];
+	  p->word_mask |= 1 << ((w->boolean_id < 0) ? 31 : w->boolean_id);
+	}
+      if (tracing)
+	add_reply(".B Near %d: mask %08x", i, p->word_mask);
     }
   if (id >= max_bools)
     {
-      add_cerr("-118 Boolean expression too complex");
+      add_err("-118 Boolean expression too complex");
       eval_err(118);
     }
   q->n_bool_ids = id;
+  if (tracing)
+    add_reply(". Word check: %d total words, %d query words, %d boolean ID's", q->nwords, q->query_word_count, q->n_bool_ids);
 }
 
 /*** Shortcut checks ***/
@@ -649,8 +719,8 @@ shortcut(struct query *q, struct expr *e)
   /* Genuine tri-state logic: 1=yes, -1=no, 0=maybe */
   switch (e->type)
     {
-    case EX_REF_WORD:
-      return q->words[e->u.ref.index].ref_count ? 0 : -1;
+    case EX_WORD:
+      return ((e->u.word.w->boolean_id < 0) ? -1 : 0);
     case EX_AND:
       i = shortcut(q, e->u.op.l);
       j = shortcut(q, e->u.op.r);
@@ -665,11 +735,10 @@ shortcut(struct query *q, struct expr *e)
       return 1;
     case EX_NONE:
       return -1;
-    case EX_REF_PHRASE:
-      return 0;
+    case EX_PHRASE:
+      return ((e->u.phrase.p->boolean_id < 0) ? -1 :0);
     default:
       ASSERT(0);
-      return -1;
     }
 }
 
@@ -680,14 +749,16 @@ check_shortcut(struct query *q, struct expr *e)
 
   if (err < 0)
     {
-      add_cr(". Shortcut: no solutions");
+      add_reply(". Shortcut: no solutions");
       eval_err(1);
     }
+#ifndef CONFIG_ALLOW_ANY
   if (err > 0)
     {
-      add_cerr("-104 All documents match");
+      add_err("-104 All documents match");
       eval_err(304);
     }
+#endif
 }
 
 /*** Boolean expression evaluation ***/
@@ -699,7 +770,8 @@ static void
 map_bool(struct query *q, struct expr *x, u32 *Z)
 {
   u32 *X, *Y, *X0, *Y0;
-  uns c, i;
+  uns c;
+  int i;
 
   switch (x->type)
     {
@@ -725,19 +797,16 @@ map_bool(struct query *q, struct expr *x, u32 *Z)
 	*Z++ = ~*X++;
       xfree(X0);
       break;
-    case EX_REF_PHRASE:
-      i = q->phrases[x->u.ref.index].boolean_id;
+    case EX_PHRASE:
+      i = x->u.phrase.p->boolean_id;
       goto got_id;
-    case EX_REF_WORD:
-      ASSERT(q->words[x->u.ref.index].use_count);
-      if (!q->words[x->u.ref.index].ref_count)
-	{
-	  memset(Z, 0, bool_bytes);
-	  break;
-	}
-      i = q->words[x->u.ref.index].boolean_id;
+    case EX_WORD:
+      ASSERT(x->u.word.w->use_count);
+      i = x->u.word.w->boolean_id;
     got_id:
-      if (i < 5)			/* Fits in u32 */
+      if (i < 0)
+	memset(Z, 0, bool_bytes);
+      else if (i < 5)			/* Fits in u32 */
 	{
 	  uns k = 1 << i;
 	  uns t;
@@ -750,7 +819,7 @@ map_bool(struct query *q, struct expr *x, u32 *Z)
 	}
       else
 	{
-	  uns per = 1 << (i - 5);
+	  int per = 1 << (i - 5);
 	  uns t = 2*per*sizeof(u32);
 	  for(c=0; c<bool_bytes; c += t)
 	    {
@@ -775,28 +844,56 @@ map_bool(struct query *q, struct expr *x, u32 *Z)
 static void
 construct_bool(struct query *q, struct expr *x)
 {
-  uns c;
-
   if (q->n_bool_ids <= 5)
     bool_bytes = sizeof(u32);
   else
     bool_bytes = 1 << (q->n_bool_ids - 3);
   DBG("Constructing boolean expression map of size %d", bool_bytes);
   map_bool(q, x, bool_map_buf);
+  q->bool_map = bool_map_buf;
 
+  if (q->n_bool_ids < 5)
+    {
+      uns mask = (1 << (1 << q->n_bool_ids)) - 1;
+      if (!(bool_map_buf[0] & mask))
+	goto no_solution;
+      if ((bool_map_buf[0] & mask) == mask)
+	goto all_matched;
+    }
+  else
+    {
+      uns or = 0, and = 0xffffffff;
+      for (uns i = 0; i < bool_bytes / 4; i++)
+        {
+	  or |= bool_map_buf[i];
+	  and &= bool_map_buf[i];
+	  if (unlikely(or && and != 0xffffffff))
+	    break;
+	}
+      if (!or)
+	goto no_solution;
+      else if (and == 0xffffffff)
+	goto all_matched;
+    }
+
+#ifndef CONFIG_ALLOW_ANY
   if (bool_map_buf[0] & 1)
     {
-      add_cerr("-104 All documents match");
-      eval_err(304);
+      add_err("-123 Negative queries not allowed");
+      eval_err(323);
     }
-  for (c=0; c<bool_bytes/4; c++)
-    if (bool_map_buf[c])
-      {
-	q->bool_map = bool_map_buf;
-	return;
-      }
-  add_cr(". Boolean check: no solutions");
+#endif
+  return;
+
+no_solution:
+  add_reply(". Boolean check: no solutions");
   eval_err(1);
+
+all_matched: ;
+#ifndef CONFIG_ALLOW_ANY
+  add_err("-104 All documents match");
+  eval_err(304);
+#endif
 }
 
 /*** Constructing boolean expression for optimistic matching ***/
@@ -810,9 +907,9 @@ optimistic_expr(struct expr *x)
 
   switch (x->type)
     {
-    case EX_REF_WORD:
+    case EX_WORD:
       return x;
-    case EX_REF_PHRASE:
+    case EX_PHRASE:
     case EX_NOT:
       return NULL;
     case EX_ANY:
@@ -843,41 +940,81 @@ construct_optimistic_bool(struct query *q, struct expr *x)
   q->optimistic_bool_map = optimistic_bool_map_buf;
 }
 
+/*** Partial evaluation of expressions for the fulltext matcher ***/
+
+int
+simplified_eval_expr(struct expr *x, uns bool_mask)
+{
+  int l, r;
+
+  /* Returns 1 for true, -1 for false, 0 for "only string types found" */
+  switch (x->type)
+    {
+    case EX_WORD:
+      if (x->u.word.w->is_string)
+	return 0;
+      else if (x->u.word.w->boolean_id < 0)
+	return -1;
+      else
+	return ((bool_mask & (1 << x->u.word.w->boolean_id)) ? 1 : -1);
+    case EX_PHRASE:
+      return ((bool_mask & (1 << x->u.phrase.p->boolean_id)) ? 1 : -1);
+    case EX_ANY:
+      return 1;
+    case EX_NONE:
+      return -1;
+    case EX_AND:
+      l = simplified_eval_expr(x->u.op.l, bool_mask);
+      r = simplified_eval_expr(x->u.op.r, bool_mask);
+      if (l < 0 || r < 0)
+	return -1;
+      else if (l > 0 || r > 0)
+	return 1;
+      else
+	return 0;
+    case EX_OR:
+      l = simplified_eval_expr(x->u.op.l, bool_mask);
+      r = simplified_eval_expr(x->u.op.r, bool_mask);
+      if (l < 0 && r < 0)
+	return -1;
+      else if (l > 0 || r > 0)
+	return 1;
+      else
+	return -1;
+    case EX_NOT:
+      return -simplified_eval_expr(x->u.op.l, bool_mask);
+    default:
+      ASSERT(0);
+    }
+}
+
 /*** Display query statistics ***/
 
 static void
-format_phrase_stat(struct query *q, byte *x, struct phrase *p)
+show_phrase_stat(struct query *q, struct phrase *p, uns attr)
 {
+  char *words[2*p->length];
   for (uns j=0; j<p->length; j++)
     {
-      if (j)
-	*x++ = ' ';
-      ASSERT(p->relpos[j] <= 4);
-      for (uns k=2; k<=p->relpos[j]; k++)
-	{
-	  *x++ = '*';
-	  *x++ = ' ';
-	}
-      struct word *w = &q->words[p->word[j]];
+      int rp = (p->relpos[j] > 1) || (p->prox_map & (1 << j));
+      words[2*j] = j ? (" * " + 2 - 2*rp) : "";
+
+      struct word *w = q->words[p->word[j]];
       if (w->root)
 	w = w->root;
-      for (byte *c=w->word; *c;)
-	*x++ = *c++;
+      words[2*j+1] = w->word;
     }
-  *x = 0;
+  add_reply("%c\"%s\" %d", attr, stk_strarraycat(words, 2*p->length), p->matches);
 }
 
 #ifdef CONFIG_FILETYPE
 static void
-ext_ft_show(struct query *q, void (*add)(char *fmt, ...))
+ext_ft_show(struct stats *s, void (*add)(char *fmt, ...))
 {
-  uns bufsize = 8;
+  char *t[MAX_FILE_TYPES];
   for (uns i=0; i<MAX_FILE_TYPES; i++)
-    bufsize += strlen(custom_file_type_names[i]) + 16;
-  byte ftbuf[bufsize], *z = ftbuf;
-  for (uns i=0; i<MAX_FILE_TYPES; i++)
-    z += sprintf(z, " %s=%d", custom_file_type_names[i], q->matching_per_type[i]);
-  add("t%s", ftbuf+1);
+    t[i] = stk_printf(" %s=%d", custom_file_type_names[i], s->matching_per_type[i]);
+  add("t%s", stk_strarraycat(t, MAX_FILE_TYPES)+1);
 }
 #endif
 
@@ -886,7 +1023,7 @@ show_query_stats(struct query *q)
 {
   for (uns i=0; i<q->nwords; i++)
     {
-      struct word *w = &q->words[i];
+      struct word *w = q->words[i];
       if (w->use_count && (w->use_count != w->hide_count || w->doc_count))
 	{
 	  /* We need to avoid spaces in complexes, so convert them to non-breakable spaces */
@@ -899,39 +1036,99 @@ show_query_stats(struct query *q)
 		qq--;
 		PUT_UTF8(qq, 0xa0);
 	      }
-	  add_cr("W%s %d %d %d %d %d%d%08x%d%d%d%Lx", disp, w->ref_count, w->ref_total_len / 1024,
-		 (q->results->status ? (w->ref_count ? -1 : 0) : (int)w->doc_count),
-		 w->status, w->is_string, w->options.accent_mode, w->type_mask,
-		 w->options.morphing, w->options.spelling, w->options.synonyming, w->options.syn_expand);
-	  q->stat_num_chains += w->ref_count;
-	  q->stat_len_chains += w->ref_total_len;
+	  add_reply("W%s %d %d %d %d %d%d%08x%d%d%d%Lx", disp, w->var_count, w->ref_total_len / 1024,
+		    (q->results->status ? ((w->boolean_id < 0) ? -1 : 0) : (int)w->doc_count),
+		    w->status, w->is_string, w->options.accent_mode, w->type_mask,
+		    w->options.morphing, w->options.spelling, w->options.synonyming, w->options.syn_expand);
 	}
     }
   for (uns i=0; i<q->nphrases; i++)
-    {
-      struct phrase *p = &q->phrases[i];
-      byte pbuf[MAX_PHRASE_LEN*(MAX_WORD_LEN+10)];
-      format_phrase_stat(q, pbuf, p);
-      add_cr("P\"%s\" %d", pbuf, p->matches);
-    }
+    show_phrase_stat(q, q->phrases[i], 'P');
   for (uns i=0; i<q->nnears; i++)
+    show_phrase_stat(q, q->nears[i], 'n');
+
+  struct stats *st = &q->stats;
+  add_reply("T%d", st->matching_docs);
+  EXTENDED_SHOW_STATS(q, st, add_reply);
+}
+
+/*** Caching of interim results ***/
+
+#define INTERIMS(x) x(words) x(nwords) x(phrases) x(nphrases) x(nears) x(nnears)
+
+static struct expr *
+copy_expr(struct mempool *pool, struct expr *e)
+{
+  if (!e)
+    return e;
+
+  struct expr *f = mp_alloc(pool, sizeof(*f));
+  *f = *e;
+  switch (e->type)
     {
-      struct phrase *p = &q->nears[i];
-      byte pbuf[MAX_PHRASE_LEN*(MAX_WORD_LEN+10)];
-      format_phrase_stat(q, pbuf, p);
-      add_cr("n\"%s\" %d", pbuf, p->matches);
+    case EX_AND:
+    case EX_OR:
+    case EX_NOT:
+      f->u.op.l = copy_expr(pool, e->u.op.l);
+      f->u.op.r = copy_expr(pool, e->u.op.r);
+      break;
+    case EX_ANY:
+    case EX_NONE:
+    case EX_WORD:
+    case EX_PHRASE:
+      /* The literal copy done above is enough */
+      break;
+    default:
+      ASSERT(0);	/* The other types cannot occur in preprocessed query */
     }
-  add_cr("T%d", q->matching_docs);
-  q->matching_docs = 0;
-  EXTENDED_SHOW_STATS(q, add_cr);
+  return f;
+}
+
+static void
+save_interims(struct query *q, struct results *r, struct expr *e)
+{
+  struct cached_interims *c = mp_alloc_zero(r->pool, sizeof(*c));
+
+  c->db = q->dbase;
+  c->expr = copy_expr(r->pool, e);
+#define SAVE_INTERIM(item) c->item = q->item;
+  INTERIMS(SAVE_INTERIM);
+#undef SAVE_INTERIM
+  clist_add_tail(&r->interims, &c->n);
+}
+
+int
+restore_interims(struct query *q, struct results *r, struct database *db)
+{
+  current_dbase = db;
+  CLIST_FOR_EACH(struct cached_interims *, c, r->interims)
+    if (c->db == db)
+      {
+	q->dbase = db;
+	q->prep_expr = c->expr;
+#define RESTORE_INTERIM(item) q->item = c->item;
+	INTERIMS(RESTORE_INTERIM);
+#undef RESTORE_INTERIM
+	return 1;
+      }
+  return 0;
 }
 
 /*** Top-level query processing ***/
 
-static struct word *word_buffer;
-static struct phrase *phrase_buffer;
-static struct phrase *near_buffer;
-static struct ref_chain *ref_buffer;
+void
+init_stats(struct query *q UNUSED, struct stats *st)
+{
+  st->matching_docs = 0;
+  EXTENDED_INIT_STATS(q, st);
+}
+
+void
+merge_stats(struct query *q UNUSED, struct stats *f, struct stats *t)
+{
+  t->matching_docs += f->matching_docs;
+  EXTENDED_MERGE_STATS(q, f, t);
+}
 
 static int
 eval_query_db(struct query *q)
@@ -944,27 +1141,33 @@ eval_query_db(struct query *q)
     return status;
 
   db_switch_config(q->dbase);
-  q->words = word_buffer;
+  q->words = mp_alloc_zero(r->pool, sizeof(struct word *) * max_words);
   q->nwords = 0;
-  q->first_ref = q->last_ref = ref_buffer;
-  q->phrases = phrase_buffer;
+  q->phrases = mp_alloc_zero(r->pool, sizeof(struct phrase *) * max_phrases);
   q->nphrases = 0;
-  q->nears = near_buffer;
+  q->nears = mp_alloc_zero(r->pool, sizeof(struct phrase *) * max_nears);
   q->nnears = 0;
-  EXTENDED_INIT_STATS(q);
+  init_stats(q, &q->stats);
+
   q->age_min = convert_age(r->create_time - MIN(r->create_time,q->age_raw_min), q->dbase->params->ref_time);
   q->age_max = convert_age(r->create_time - MIN(r->create_time,q->age_raw_max), q->dbase->params->ref_time);
   e = analyse_query(q, q->expr);
   if (e->type == EX_IGNORE)
     e->type = EX_ANY;
   debug_expr(q, e, 'A');
+  fixup_words(q);
   debug_words(q);
-  spell_check(q);
+  if (r->max_matches)
+    spell_check(q);
   check_words(q);
-  check_shortcut(q, e);
-  construct_bool(q, e);
-  construct_optimistic_bool(q, e);
-  process_refs(q);
+  save_interims(q, r, e);
+  if (r->max_matches)
+    {
+      check_shortcut(q, e);
+      construct_bool(q, e);
+      construct_optimistic_bool(q, e);
+      process_refs(q);
+    }
   return 0;
 }
 
@@ -983,53 +1186,66 @@ eval_query(struct query *q, struct results *r)
   uns first_err_code = 0;
 
   r->status = 0;
-  cache_rbuf = &r->rbuf;
+  q->current_reply_buf = &r->reply_header;
+  clist_init(&r->interims);
 
   query_init_refs(q);
   debug_expr(q, q->expr, 'S');
-  for (q->dbase=databases, i=0; q->dbase && !r->status; q->dbase=q->dbase->next, i++)
-    if (q->db_mask & (1 << i))
-      {
-	current_dbase = q->dbase;
-	if (!current_dbase->params)
-	  continue;
-	add_cr("(D");
-	add_cr("D%s", current_dbase->name);
-	add_cr("N%d", current_dbase->num_ids);
-	add_cr("I%d", current_dbase->params->objects_in);
-	add_cr("L%d", (int) current_dbase->params->ref_time);
-	status = eval_query_db(q);
-	if (status < 100)	/* OK or no solution */
-	  successful_dbs++;
-	else if (status >= 300 && q->partial_answers)
-	  {
-	    /* Partial answer: Move the status message at the end */
-	    struct reply *err = first_reply_last(&r->rbuf);
-	    if (!first_error)
-	      {
-		first_error = err;
-		first_err_code = status - 200;
-	      }
-	  }
-	else
-	  r->status = (status >= 300) ? (status - 200) : status;
-	show_query_stats(q);
-	memory_flush(q);
-	add_cr(")");
-      }
+  if (q->debug & DEBUG_ANALYSE)
+    add_reply(".r%d", r->max_matches);
+
+  i = 0;
+  CLIST_FOR_EACH(struct database *, db, databases)
+    {
+      q->dbase = db;
+      if (r->status)
+	break;
+      if (q->db_mask & (1 << i))
+        {
+	  current_dbase = q->dbase;
+	  if (!current_dbase->params)
+	    continue;
+	  add_reply("(D");
+	  add_reply("D%s", current_dbase->name);
+	  add_reply("N%d", current_dbase->num_ids);
+	  add_reply("I%d", current_dbase->params->objects_in);
+	  add_reply("L%d", (int) current_dbase->params->ref_time);
+	  add_reply("V%08x", (int) current_dbase->params->ref_time);
+	  status = eval_query_db(q);
+	  if (status < 100)	/* OK or no solution */
+	    successful_dbs++;
+	  else if (status >= 300 && q->partial_answers)
+	    {
+	      /* Partial answer: Move the status message at the end */
+	      struct reply *err = first_reply_last(&r->reply_header);
+	      if (!first_error)
+	        {
+		  first_error = err;
+		  first_err_code = status - 200;
+	        }
+	    }
+	  else
+	    r->status = (status >= 300) ? (status - 200) : status;
+	  show_query_stats(q);
+	  memory_flush(q);
+	  add_reply(")");
+        }
+      i++;
+    }
   if (!successful_dbs && first_error)
     {
       char buf[first_error->len];
       memcpy(buf, first_error->text, first_error->len);
       buf[first_error->len - 1] = 0;
-      add_cerr(buf);
+      add_err(buf);
       r->status = first_err_code;
     }
   if (r->status < 100)
     {
       query_finish_refs(q);
-      add_cerr("+000 OK");
+      add_err("+000 OK");
     }
+  q->current_reply_buf = &q->reply_header;
 }
 
 void
@@ -1037,14 +1253,10 @@ query_init(void)
 {
   if (max_words > HARD_MAX_WORDS)
     die("MaxWords (%d) > HARD_MAX_WORDS (%d)", max_words, HARD_MAX_WORDS);
-  word_buffer = xmalloc(sizeof(struct word) * max_words);
-  phrase_buffer = xmalloc(sizeof(struct phrase) * max_phrases);
-  if (max_nears)
-    near_buffer = xmalloc(sizeof(struct phrase) * max_nears);
-  ref_buffer = xmalloc(sizeof(struct ref_chain) * max_words * max_word_matches);
   bool_bytes = 1 << (MAX(5, max_bools) - 3);
   bool_map_buf = xmalloc(bool_bytes);
   optimistic_bool_map_buf = xmalloc(bool_bytes);
+  incarnation = ((u64)time(NULL) << 32) + getpid();
 }
 
 static void
@@ -1059,8 +1271,9 @@ init_query(struct query *q)
   o->spelling = global_spelling;
   o->synonyming = global_synonyming;
   o->syn_expand = (global_syn_expand >= 0x80000000) ? ~0ULL : global_syn_expand;
+  o->default_word_types = default_word_types;
   q->context_chars = global_context_chars;
-  q->title_chars = global_title_chars;
+  memcpy(q->meta_chars, global_meta_chars, sizeof(global_meta_chars));
   q->intervals = global_intervals;
   q->site_max = global_site_max;
   q->url_max = global_url_max;
@@ -1096,25 +1309,21 @@ process_query(struct query *q)
   struct results *r;
 
   current_query = q;
-  query_rbuf = &q->rbuf;
-  cache_rbuf = NULL;
   init_query(q);
+  add_reply("V" SHER_VER);
+  add_reply("I%Lx", (long long) incarnation);
 
   profiler_init();
   profiler_switch(&prof_analyse);
-  add_qr("V" SHER_VER);
   if (!q->iobuf[0])
     {
-      add_qerr("-102 Empty request");
+      add_err("-102 Empty request");
       return;
     }
-  q->contains_accents = contains_accents(q->iobuf);
+  q->contains_accents = word_contains_accents(q->iobuf);
   if (err = parse_query(q->iobuf))
     {
-      if (err[0] == '-')
-	add_qerr("%s", err);
-      else
-	add_qerr("-102 %s", err);
+      add_err("%s%s", (err[0] == '-' ? "" : "-102 "), err);
       return;
     }
   if (!q->expr)
@@ -1122,9 +1331,11 @@ process_query(struct query *q)
       do_command(q);
       return;
     }
+
+  tidy_query(q);
+
   if (check_result_set(q))
     return;
-  tidy_query(q);
 
   debug_expr(q, q->expr, 'I');
   process_options(q);
@@ -1135,21 +1346,22 @@ process_query(struct query *q)
       if (r->status >= 0)
 	{
 	  q->cache_age = r->access_time - r->create_time;
-	  add_qr("C%d", q->cache_age);
+	  add_reply("C%d", q->cache_age);
 	}
       else
 	{
 	  q->cache_age = -1;
 	  eval_query(q, r);
 	}
-      ship_reply_buf(q, &r->rbuf);
+      ship_reply_buf(q, &r->reply_header);
       q->q_status = r->status;
     }
-  flush_reply_buf(q, &q->rbuf);
-  reply_f("%s", "");	/* Avoid format string warning */
+  flush_reply_buf(q, &q->reply_header);
+  send_reply_string("\n");
+  q->current_reply_buf = &q->reply_footer;
+  add_footer("+");
   if (r && !r->status)
     show_results(q);
-  reply_f("+");
   profiler_switch(NULL);
   profiler_show(q);
 }
