@@ -1,11 +1,13 @@
 /*
  *	Sherlock Search Engine
  *
- *	(c) 1997--2005 Martin Mares <mj@ucw.cz>
+ *	(c) 1997--2006 Martin Mares <mj@ucw.cz>
  */
 
 #include "lib/clists.h"
+#include "lib/slists.h"
 #include "sherlock/index.h"
+#include "indexer/sites.h"
 
 #define PROFILE_TOD
 #include "lib/profile.h"
@@ -14,8 +16,10 @@
 
 #define MAX_PHRASE_LEN 8	/* Must be a power of two */
 #define HARD_MAX_WORDS 32	/* Limited to 32 by refs.c */
-#define HARD_MAX_NOTES 6
 #define HARD_MAX_SYNONYMA 63	/* Limited to 63 by bits in syn_expand */
+#define HARD_MAX_DATABASES 32	/* Limited to 32 by db_mask bits */
+#define NUM_BESTS 8		/* How many best matches are kept for displaying of match context */
+#define BESTS_PER_WORD 2	/* No more than this number of matches can be recorded for a single word */
 
 /* config.c */
 
@@ -23,73 +27,82 @@ extern byte *log_name, *status_name;
 extern uns log_incoming, log_rejected, log_requests, log_replies;
 extern uns port, listen_queue, connection_timeout, hydra_processes;
 extern byte *control_password;
-extern struct database *databases;
-extern struct spell_pair *spell_common_pairs;
-extern uns num_matches, cache_size, max_output_matches;
+extern clist databases;
+extern clist spell_common_pairs;
+extern uns num_matches, max_matches, cache_size, max_output_matches;
 extern uns max_words, max_word_matches, max_phrases, max_nears, max_bools;
-extern uns global_accent_mode, max_wildcard_zone, min_wildcard_prefix_len;
-extern uns global_context_chars, global_title_chars, global_intervals, global_site_max;
-extern uns global_url_max, global_redirect_url_max, global_morphing, global_spelling, global_synonyming;
+extern uns global_accent_mode, wildcard_asterisks, wildcard_qmarks, max_wildcard_zone, min_wildcard_prefix_len;
+extern uns global_context_chars, global_intervals, highlight_substring, global_site_max;
+extern uns global_meta_chars[16];
+extern byte *url_attributes;
+extern uns global_url_max, global_max_redir_brack, global_max_cat_brack, global_morphing, global_spelling, global_synonyming;
 extern uns doc_weight_scale, word_weight_scale, word_bonus;
-extern uns global_allow_approx, prox_limit, prox_penalty;
+extern uns global_allow_approx, prox_penalty;
+extern int prox_limit;
 extern uns global_partial_answers, default_word_types, global_debug, global_sorting, global_sort_reverse;
 extern uns mem_map_zone_size, mem_map_elide_gaps, mem_map_prefetch;
 extern uns query_watchdog, second_best_reduce;
-extern uns magic_complexes, magic_merge_words, magic_merge_classes, magic_near, magic_merge_bonus;
-extern uns near_bonus_word, near_penalty_gap, near_bonus_connect;
+extern uns magic_complexes, magic_merge_words, magic_merge_classes;
+extern uns magic_keyphrases, magic_keyphrases_classes, magic_keyphrases_string_types, magic_keyphrases_bonus;
+extern uns magic_near, magic_merge_bonus;
+extern uns near_bonus_word, near_penalty_gap, near_bonus_connect, near_min_weight, near_max_weight;
 extern uns blind_match_penalty, misaccent_penalty, stem_penalty, morph_penalty, synonymum_penalty;
 extern uns spell_good_freq, spell_min_len, spell_margin, spell_dwarf, spell_dwarf_margin, global_syn_expand, spell_common_penalty;
 extern uns spell_add_penalty, spell_del_penalty, spell_mod_penalty, spell_xpos_penalty, spell_accent_penalty;
-extern struct ipaccess_list *access_list;
+extern clist access_list;
 
 struct database {
-  struct database *next;
+  cnode node;
   byte *name;
-  int is_optional;
+  byte *directory;
+  enum {
+    DB_PART_WORDS = 1,
+    DB_PART_STRINGS = 2,
+    DB_PART_PRINTS = 4,
+    DB_PART_BLACKLIST = 8
+  } parts;
+  uns is_optional;
+  struct mempool *pool;			/* Contains all information local to this database */
 
   /* Parameters */
-  byte *fn_params;
   struct index_params *params;
 
   /* Cards */
   oid_t num_ids;
-  byte *fn_cards;
-  byte *fn_card_attrs;
-  byte *fn_references;
   int fd_cards, fd_refs;
   struct card_attr *card_attrs, *card_attrs_end;
   sh_off_t card_file_size, ref_file_size;
 
   /* Words */
-  uns word_weights[8], meta_weights[16];
-  byte *fn_lexicon, *fn_stems;
+  uns word_weights[8];
+  uns meta_weights[16][4];
   uns lexicon_words, lexicon_complexes, lexicon_file_size;
   struct lex_entry **lex_array;
   uns lex_by_len[MAX_WORD_LEN+2];
   struct lex_entry ***cplx_array;
   uns stems_file_size;
   clist stem_block_list, syn_block_list;
+  slist wexc_list;
+  struct vocabolario *wexc_vocabolario;
 
   /* Strings */
   uns string_weights[8];
-  byte *fn_string_map;
-  byte *fn_string_hash;
   u32 *string_hash;
   uns string_buckets, string_hash_order, string_count;
   int fd_string_map;
   uns string_hash_file_size, string_map_file_size;
 
   /* Fingerprints */
-  byte *fn_card_prints;
   struct fastbuf *fb_card_prints;
+
 };
 
 struct spell_pair {
-  struct spell_pair *next;
+  cnode n;
   uns x, y;
 };
 
-/* reply.c */
+/*** Reply buffers: we keep fragments of the reply in them (reply.c) ***/
 
 struct reply {
   struct reply *next;
@@ -97,33 +110,28 @@ struct reply {
   byte text[1];
 };
 
-struct replybuf {
+struct reply_buf {
   struct reply *first, **last;
   struct mempool *pool;
 };
 
 struct query;
 
-void init_reply_buf(struct replybuf *, struct mempool *);
-void add_reply(struct replybuf *, char *, ...) __attribute__((format(printf,2,3)));
-void ship_reply_buf(struct query *, struct replybuf *);
-void flush_reply_buf(struct query *, struct replybuf *);
-struct reply *first_reply_last(struct replybuf *);
+void init_reply_buf(struct reply_buf *, struct mempool *);
+void add_reply_to(struct reply_buf *, char *, ...) FORMAT_CHECK(printf,2,3);
+void ship_reply_buf(struct query *, struct reply_buf *);
+void flush_reply_buf(struct query *, struct reply_buf *);
+struct reply *first_reply_last(struct reply_buf *);
 
-extern struct replybuf *query_rbuf, *cache_rbuf;
-void add_qr(char *, ...) __attribute__((format(printf,1,2)));
-void add_qerr(char *, ...) __attribute__((format(printf,1,2)));
-void add_cr(char *, ...) __attribute__((format(printf,1,2)));
-void add_cerr(char *, ...) __attribute__((format(printf,1,2)));
-void reply_f(char *, ...) __attribute__((format(printf,1,2)));
+void add_reply(char *fmt, ...) FORMAT_CHECK(printf,1,2);
+void add_err(char *fmt, ...) FORMAT_CHECK(printf,1,2);
+void add_footer(char *fmt, ...) FORMAT_CHECK(printf,1,2);
 
-/* lex.c */
+void send_reply(char *fmt, ...) FORMAT_CHECK(printf,1,2);
+void send_reply_string(char *msg);
+void send_reply_block(char *start, uns len);
 
-void lex_init(byte *);
-int yylex(void);
-int lookup_custom_attr(byte *);
-
-/* parse.y */
+/*** Syntax tree of the query ***/
 
 enum expr_type {
   /*
@@ -142,8 +150,8 @@ enum expr_type {
   EX_ANY,				/* PIO  Logical 1 */
   EX_NONE,				/* PIO  Logical 0 */
   EX_IGNORE,				/* .A.  Non-indexed word */
-  EX_REF_WORD,				/* ..O  Match word with specified index */
-  EX_REF_PHRASE				/* ..O  Match phrase with specified index */
+  EX_WORD,				/* ..O  Match a word */
+  EX_PHRASE				/* ..O  Match a phrase */
 };
 
 struct options {
@@ -153,6 +161,7 @@ struct options {
   sbyte spelling;
   sbyte synonyming;
   u64 syn_expand;
+  uns default_word_types;		/* ~0U = unset */
 };
 
 #define OPT_DEFAULT -1
@@ -167,8 +176,8 @@ struct expr {
     struct {
       struct options o;
       byte *word;
-      uns classmap;
-      byte is_string;
+      uns type_mask;			/* Can be ~0U (unset) before propagation of options */
+      byte is_string;			/* Set after option propagation */
       sbyte sense;			/* Matching sense: 1=YES, -1=NOT, 0=MAYBE */
       struct expr *next_simple;		/* Next in simple search chain */
     } match;
@@ -177,40 +186,80 @@ struct expr {
       struct options o;
     } options;
     struct {
-      uns index;
-    } ref;
+      struct word *w;
+    } word;
+    struct {
+      struct phrase *p;
+    } phrase;
   } u;
 };
 
-struct val_set {			/* Syntactic representation of integer sets */
-  struct val_set *next;
-  u32 min, max;
-  byte *text;
-};
-
 #define ACCENT_AUTO 0			/* magic auto mode */
 #define ACCENT_STRIP 1			/* strip all accents before comparing */
 #define ACCENT_STRICT 2			/* compare with accents */
 #define ACCENT_AUTO_LOCAL 3		/* magic auto mode done per word */
 
-void err(byte *) NONRET;
+/* lex.c */
+
+void lex_init(byte *);
+int yylex(void);
+int lookup_custom_attr(byte *);
+
+/* parse.y */
+
+void err(char *, ...) NONRET FORMAT_CHECK(printf,1,2);
 byte *parse_query(byte *);
 struct expr *new_node(enum expr_type t);
 struct expr *new_op(enum expr_type t, struct expr *l, struct expr *r);
 void merge_options(struct options *dest, struct options *old, struct options *new);
 
+struct set_node {			/* Syntactic representation of integer or string sets */
+  snode n;
+  uns min, max;
+  byte *text;
+};
+
+struct db_node {			/* Syntactic representation of database lists */
+  snode n;
+  struct database *db;
+  int id;
+};
+
+struct dump_node {			/* ... and of sets of cards to dump */
+  snode n;
+  oid_t id;
+  int db_id;
+  struct database *db;
+};
+
+enum out_mode {				/* Output modes */
+  OUT_STATS,
+  OUT_LIST,
+  OUT_SHOW,
+  OUT_DUMP
+};
+
 /* sherlockd.c */
 
-#define IOBUF_SIZE 4096
+void write_reply(struct query *, byte *, uns);
 
-struct query {
+/*** Internal representation of queries ***/
+
+struct stats {				/* Statistics */
+  uns matching_docs;			/* Matching documents for this dbase */
+  EXTENDED_STAT_VARS			/* Custom statistics */
+};
+
+struct query {				/* Keep the most frequently used parts near the top */
   cnode n;
   struct mempool *pool;			/* Memory pool for _query_ data (use results->pool for results to be cached) */
 
   /* Query parameters */
   u32 db_mask;				/* Database mask */
+  slist *db_list;			/* A list of databases requested */
+  uns debug;				/* Debug flags */
   struct options default_options;	/* Default word options */
-  uns site_only;			/* Only this site */
+  u64 site_hash;			/* Only this site */
   uns site_max;				/* Number of documents per site */
   uns allow_approx;			/* Allow approximation */
   uns partial_answers;			/* Allow partial answers */
@@ -228,6 +277,9 @@ struct query {
 #undef LATE_SMALL_SET_ATTR
 
   CUSTOM_MATCH_VARS			/* Include what custom matchers need */
+#define CUSTOM_MATCH_KWD(id,keywd,parse) byte id##_placeholder;
+  CUSTOM_MATCH_PARSE
+#undef CUSTOM_MATCH_KWD
 
   int custom_sorting;			/* Sort on custom attribute */
   u32 custom_sort_reverse;		/* ~0 if sorting reverse, else 0 */
@@ -235,48 +287,43 @@ struct query {
   uns age_raw_min, age_raw_max;		/* Limits on document age */
   u32 explain_id;			/* Which object we'd like to explain for [CONFIG_EXPLAIN] */
 
+  /* Query processing status */
+  struct results *results;		/* Result structure for this query */
+  int cache_age;			/* Age of cached reply, -1 if not cached */
+  uns contains_accents;			/* There is an accent anywhere in the query */
+  uns needed_results;			/* How many results do we have to calculate */
+  uns results_to_show;			/* How many results are requested to be shown */
+  struct global_match_heap *match_heap;	/* refs.c: Temporary heap used for keeping the best matches */
+  slist ft_contexts;			/* fulltext.c: Fulltext contexts for all databases */
+
+  /* Local processing status for the current database */
+  struct database *dbase;		/* Database we're currently examining */
+  struct expr *prep_expr;		/* Preprocessed expression */
+  struct word **words;			/* Words found in the query */
+  uns nwords;
+  struct phrase **phrases, **nears;	/* Phrase matchers and near matchers */
+  uns nphrases, nnears;
+  u32 *bool_map;			/* Truth tables of boolean expressions */
+  u32 *optimistic_bool_map;
+  uns n_bool_ids;			/* Number of words/phrases involved in boolean expression */
+  int age_min, age_max;			/* Document age relative to ref_time of current database */
+  struct stats stats;			/* Statistics */
+  uns stat_num_chains;			/* Statistics: total number of reference chains seen */
+  uns stat_len_chains;			/* Statistics: total length of reference chains seen */
+  uns query_word_count;			/* The number of query words */
+
   /* Display parameters */
-  struct val_set *range;		/* Range of results to display */
-  uns list_only;			/* Show only a list of results, don't show full cards */
-  uns debug;				/* Debug flags */
+  enum out_mode out_mode;		/* Type of output requested */
+  slist *out_range;			/* List of results to display */
   uns context_chars;			/* Number of context chars to print */
-  uns title_chars;			/* Number of title chars to print */
+  uns meta_chars[16];			/* Number of meta chars to print for each meta type */
   uns intervals;			/* Number of intervals to print */
   uns url_max;				/* Maximum # of URL's to print */
 
   /* Reply */
-  struct replybuf rbuf;			/* Reply header */
-
-  /* Connection */
-  sh_time_t established;		/* Time this connection was established */
-  int fd;				/* Socket file descriptor */
-  int fd_err;				/* Set in case an error has occured */
-  int q_status;				/* Error code returned */
-  byte ipaddr[16];			/* IP address of the other end */
-  byte iobuf[IOBUF_SIZE];		/* I/O buffering */
-  byte *ibptr, *ibend;
-  byte *obptr, *obend;
-
-  /* Query processing status */
-  struct results *results;		/* Result structure for this query */
-  struct database *dbase;		/* Database we're currently examining */
-  struct word *words;			/* Words matched in the query */
-  uns nwords;
-  struct ref_chain *first_ref, *last_ref;
-  struct phrase *phrases;
-  uns nphrases;
-  struct phrase *nears;
-  uns nnears;
-  u32 *bool_map;
-  u32 *optimistic_bool_map;
-  uns n_bool_ids;			/* Number of words/phrases involved in boolean expression */
-  uns matching_docs;			/* Matching documents for this dbase */
-  EXTENDED_STAT_VARS			/* Custom statistics */
-  int age_min, age_max;			/* Document age relative to ref_time of current database */
-  uns contains_accents;			/* There is an accent anywhere in the query */
-  int cache_age;			/* Age of cached reply, -1 if not cached */
-  uns stat_num_chains;			/* Statistics: total number of reference chains seen */
-  uns stat_len_chains;			/* Statistics: total length of reference chains seen */
+  struct reply_buf reply_header;	/* Reply header */
+  struct reply_buf reply_footer;	/* Reply footer */
+  struct reply_buf *current_reply_buf;	/* Either query header or cached results header */
 
   /* Timing statistics */
   uns time_total;
@@ -284,12 +331,20 @@ struct query {
 
   /* Memory mappings */
   addr_int_t last_mapping;
+
+  /* Connection */
+  sh_time_t established;		/* Time this connection was established */
+  int fd;				/* Socket file descriptor */
+  int fd_err;				/* Set in case an error has occured */
+  int q_status;				/* Error code returned */
+  byte ipaddr[16];			/* IP address of the other end */
+  byte *iobuf;				/* I/O buffering */
+  byte *ibptr, *ibend;
+  byte *obptr, *obend;
 };
 
 #define CONTEXT_FULL 1000000000		/* context_chars when CONTEXT FULL is asked for */
 
-void reply_string(struct query *, byte *, uns);
-
 /* Internal parameters masking themselves as extended attributes */
 /* Beware, they should not collide with extended attr offsets in struct query */
 #define PARAM_SITE		0
@@ -302,12 +357,10 @@ void reply_string(struct query *, byte *, uns);
 #define DEBUG_ANALYSE		2	/* Debug query analyser */
 #define DEBUG_DUMPING		4	/* Dumping the context */
 #define DEBUG_WORDS		8	/* Debug processing of words */
-#define DEBUG_CARD_INFO		16	/* Show result notes and card attributes */
+#define DEBUG_CARD_INFO		16	/* Show result notes and card attributes (used by mux, too) */
+#define DEBUG_FETCH		32	/* Show which parts of files are fetched */
 
-/* query.c */
-
-extern struct query *current_query;
-extern struct database *current_dbase;
+/*** Words and phrases ***/
 
 /* When processing a simple search query, we represent it by a list of these structures */
 
@@ -328,31 +381,50 @@ struct simple {
 
 struct word {
   /* stuff used by refs.c goes first to improve caching */
-  uns boolean_id;			/* ID used in boolean expression */
+  int boolean_id;			/* ID used in boolean expression, <0 if known to be unmatchable */
   uns type_mask;			/* [K] Bitmap of allowed word/string types, upper 16 bits are meta types */
   uns doc_count;			/* Number of documents matched */
   int weight;				/* Word weight */
-  int q, q2, q2strict;			/* refs.c: Best matches recorded for the current OID */
-  int pos, pos2;
   uns is_string;			/* [K] Match strings, not words */
   uns is_outer;				/* At least once outside a phrase */
   struct options options;		/* [K] Local word matching options; translate_accent_mode()'d */
-  uns ref_count;			/* Number of reference chains matched */
-  uns ref_total_len;			/* Total length of all reference chains */
   uns status;				/* 0 for OK or error code */
   byte *word;				/* [K] The word itself */
   uns expanded;				/* Already expanded (0=not, 1=partially, 2=with refs) */
   uns word_class;			/* words.c: Word class */
-  clist variants;			/* words.c: The list of word variants */
-  uns var_count;			/* words.c: The number of variants */
+  slist variants;			/* The list of word variants (struct variant) */
+  uns var_count;			/* The number of variants */
+  uns ref_total_len;			/* refs.c: Total length of all reference chains read */
   uns cover_count;			/* words.c: Number of occurences inside a complex */
   uns use_count;			/* Number of times this word was looked up */
   uns hide_count;			/* If hide_count == use_count, don't report the word unless it was matched */
   uns is_wild;				/* Contains wildcards */
   struct word *root;			/* If it's a complex, then its root word, else NULL */
-#ifdef CONFIG_EXPLAIN
-  uns explain_ref, explain_ref2s;	/* Ref entries remembered for EXPLAINing */
-#endif
+  uns index;				/* Index of this word in word array */
+};
+
+struct variant {
+  snode n;
+  uns lex_id;				/* words.c: Word ID in the lexicon */
+  u32 lang_mask;			/* Language mask */
+  byte noaccent_only;			/* Use in accentless documents only */
+  byte penalty;				/* Extra negative weight */
+  byte flags;				/* VF_xxx */
+  byte rfu;
+  sh_off_t refchain_start;		/* Refchain corresponding to this variant */
+  uns refchain_len;			/* (can be empty) */
+};
+
+enum var_flags {
+  VF_SOURCE_MASK = 0x1f,		/* These bits indicate source of the variant */
+  VF_QUERY = 1,				/* Query word and its accent/wildcard expansions */
+  VF_SYNTHETIC = 2,			/* Query word which isn't present in the lexicon, so we've synthesized a lex_entry for it */
+  VF_MORPH = 4,				/* Generated by morphological expansion */
+  VF_ACCENTS = 8,			/* Generated by accent expansion */
+  VF_SYNONYMUM = 16,			/* Generated by synonymic expansion */
+  VF_LEMMA = 32,			/* This variant is a lemma */
+  VF_ACCENTIFIED = 64,			/* Searching for accent variants already applied to this word */
+  VF_MORPHED = 128,			/* Morphological expansion already applied */
 };
 
 struct phrase {				/* Used for both phrases and near-matchers */
@@ -364,51 +436,62 @@ struct phrase {				/* Used for both phrases and near-matchers */
   uns length;				/* Number of words */
   int weight;
   uns matches;
-  uns boolean_id;
+  int boolean_id;
   u32 word_mask;
+  uns index;				/* Index of this phrase in the phrase array */
 };
 
-struct ref_chain {
-  union {
-    struct {
-      sh_off_t start;
-      u32 size;
-    } file;
-    struct {
-      u16 *pos;
-    } mem;
-  } u;
-  u32 lang_mask;			/* Language mask */
-  byte word_index;			/* Word this ref belongs to */
-  byte noaccent_only;			/* Search accentless documents only */
-  byte penalty;				/* Extra negative weight applied to references from this chain */
+/*** Cached query results ***/
+
+struct result_note {
+  struct card_attr *attr;		/* Attributes of this card */
+  int q;
+  u32 sec_sort_key;
 };
 
 struct results {			/* Query results we cache */
+  /* Cache control */
   cnode h;				/* In hash queue (_must_ be first!) */
   cnode n;				/* In LRU queue */
   int status;				/* -1=uninitialized, 0=OK, else error code */
   byte *request;			/* Normalized request */
+  u32 db_mask;				/* A set of databases */
+  uns max_matches;			/* Result limit */
   sh_time_t create_time, access_time;	/* Time created/last used */
-  struct mempool *pool;			/* Pool holding results */
-  struct replybuf rbuf;			/* Reply strings */
-  struct result_note **result_heap;	/* Matching documents */
-  uns nresults;
-  struct result_note *first_note, *free_note;
-  struct hilite_word *first_hilite;	/* A list of words to highlight in output */
+
+  /* The results */
+  struct mempool *pool;			/* Pool holding this structure and everything referenced by it */
+  struct reply_buf reply_header;	/* Reply header */
+  struct result_note *matches;		/* Matching documents */
+  uns num_matches;
+  clist interims;			/* List of cached_interims for all databases involved in the query */
 };
 
-struct hilite_word {
-  struct hilite_word *next;
-  byte w[1];
+struct cached_interims {		/* Cached preprocessed parts of query used when dumping results (per database) */
+  cnode n;
+  struct database *db;			/* Which database is this related to */
+  struct expr *expr;			/* Preprocessed expression */
+  struct word **words;			/* Words, phrases and nears */
+  struct phrase **phrases, **nears;
+  uns nwords, nphrases, nnears;
 };
 
+/* query.c */
+
+extern struct query *current_query;
+extern struct database *current_dbase;
+
 void cache_init(void);
 void query_init(void);
 void process_query(struct query *q);
-int lookup_word(struct query *q, struct expr *e, byte *w);
-void add_hilited_word(struct query *q, byte *w);
-void eval_err(int code);
+struct word *lookup_word(struct query *q, struct expr *e, byte *w);
+void eval_err(int code) NONRET;
+
+void init_stats(struct query *q, struct stats *st);
+void merge_stats(struct query *q, struct stats *f, struct stats *t);
+int simplified_eval_expr(struct expr *x, uns bool_mask);
+
+int restore_interims(struct query *q, struct results *r, struct database *db);
 
 /* Profiling counters */
 #define PROFILERS(sep) P(analyse) sep P(reff) sep P(refs) sep P(resf) sep P(results)
@@ -421,8 +504,9 @@ prof_t *profiler_switch(prof_t *p);
 
 /* dbase.c */
 
-void db_init(void);
+void db_init(int merge_only);
 void db_switch_config(struct database *db);
+byte *db_file_name(struct database *db, byte *fn);
 struct database *attr_to_db(struct card_attr *attr, oid_t *ooid);
 
 /* cards.c */
@@ -438,16 +522,21 @@ extern byte *st_names[];
 
 void words_init(struct database *db);
 void word_analyse_simple(struct query *q, clist *l);
-int word_classify(struct database *db, byte *word);
-int contains_accents(byte *s);
-void word_add_hilites(struct query *q, struct word *w);
-void spell_check(struct query *q);
+int word_contains_accents(byte *s);
+void word_dump_variants(struct word *w);
+void word_ensure_variants(struct word *w);
+void word_extract_variant(struct word *w, struct variant *v, byte *buf);
 
 /* strings.c */
 
 void strings_init(struct database *db);
+struct expr *string_analyse(struct query *q, struct expr *e);
 void string_analyse_simple(struct query *q, clist *l);
 
+/* spell.c */
+
+void spell_check(struct query *q);
+
 /* memory.c */
 
 struct mmap_request {
@@ -473,14 +562,6 @@ extern struct lizard_buffer *liz_buf;
 
 /* refs.c */
 
-struct result_note {
-  uns heap;				/* Back-link to the heap */
-  struct card_attr *attr;		/* Attributes of this card */
-  int q;
-  u32 sec_sort_key;
-  u16 best[HARD_MAX_NOTES];		/* position, 0xffff if undefined */
-};
-
 void refs_init(void);
 void process_refs(struct query *q);
 void query_init_refs(struct query *q);
