@@ -17,7 +17,6 @@
 #include "lib/conf.h"
 
 #include <string.h>
-#include <stdlib.h>
 #include <fcntl.h>
 #include <unistd.h>
 #include <sys/file.h>
@@ -36,19 +35,20 @@ static int obuck_shake_buflen = 1048576;
 static uns obuck_shake_security;
 static uns obuck_slurp_buflen = 65536;
 
-static struct cfitem obuck_config[] = {
-  { "Buckets",		CT_SECTION,	NULL },
-  { "BucketFile",	CT_STRING,	&obuck_name },
-  { "BufSize",		CT_INT,		&obuck_io_buflen },
-  { "ShakeBufSize",	CT_INT,		&obuck_shake_buflen },
-  { "ShakeSecurity",	CT_INT,		&obuck_shake_security },
-  { "SlurpBufSize",	CT_INT,		&obuck_slurp_buflen },
-  { NULL,		CT_STOP,	NULL }
+static struct cf_section obuck_config = {
+  CF_ITEMS {
+    CF_STRING("BucketFile", &obuck_name),
+    CF_UNS("BufSize", &obuck_io_buflen),
+    CF_INT("ShakeBufSize", &obuck_shake_buflen),
+    CF_UNS("ShakeSecurity", &obuck_shake_security),
+    CF_UNS("SlurpBufSize", &obuck_slurp_buflen),
+    CF_END
+  }
 };
 
 static void CONSTRUCTOR obuck_init_config(void)
 {
-  cf_register(obuck_config);
+  cf_declare_section("Buckets", &obuck_config, 0);
 }
 
 /*** Internal operations ***/
@@ -352,6 +352,8 @@ obuck_create(void)
   sh_off_t start = sh_seek(obuck_fd, 0, SEEK_END);
   if (start & (OBUCK_ALIGN - 1))
     obuck_broken("Misaligned file", start);
+  if (start >= ((sh_off_t) OBUCK_OID_FIRST_SPECIAL << OBUCK_SHIFT))
+    obuck_broken("Maximum bucket file size exceeded", start);
   obuck_create_hdr.magic = OBUCK_INCOMPLETE_MAGIC;
   obuck_create_hdr.oid = start >> OBUCK_SHIFT;
   obuck_create_hdr.length = 0;
