@@ -39,3 +39,27 @@ fingerprint(byte *string, struct fingerprint *fp)
   MD5Final(digest, &c);
   memcpy(fp->hash, digest, 12);
 }
+
+#ifdef TEST
+
+#include "lib/fastbuf.h"
+
+int main(void)
+{
+  struct fastbuf *in = bfdopen_shared(0, 4096);
+  struct fastbuf *out = bfdopen_shared(1, 4096);
+  byte buf[4096];
+  while (bgets(in, buf, sizeof(buf)))
+    {
+      struct fingerprint f;
+      fingerprint(buf, &f);
+      for (uns i=0; i<12; i++)
+	bprintf(out, "%02x", f.hash[i]);
+      bputc(out, '\n');
+    }
+  bclose(in);
+  bclose(out);
+  return 0;
+}
+
+#endif
