@@ -49,7 +49,7 @@ Descriptors:
 	--url URL
 	--bucket BUCKETNR
 	--id ID		... stage1 object ID, intervals are also allowed
-	  --withlinks	... documents that point TO this document
+	  --withlinks
 	  --withsimilar
 	  --withmerges
 	  --withcards
@@ -276,17 +276,14 @@ sub id2card
 sub get_links
 {
 	my ($bid) = @_;
-	my $renum = `./bin/idxdump -f $idxdir/link-graph-goes -m $bid`;
-	$renum =~ /^0*$bid -> ([0-9a-f]*)$/ or return ($bid);
-	my $graph_idx = `./bin/idxdump -f $idxdir/link-graph-index -G $1`;
+	my $graph_idx = `./bin/idxdump -f $idxdir/link-graph-index -G $bid`;
 	$graph_idx =~ /.* -> ([0-9a-f]*)/ or return ($bid);
-	if ($1 eq "ffffffffff") { return ($bid); }
 	my $target = `./bin/idxdump -f $idxdir/link-graph -g $1`;
 	$target =~ /Vertex ([0-9a-f]*) /
 	and hex $1 == hex $bid
 		or return ($bid);
 	return $bid, map {
-		if (/<- ([0-9a-f]*)($| \[[fr])/)
+		if (/-> ([0-9a-f]*)($| \[[fr])/)
 		{ $1 }
 		else { () }
 	} split /\n/, $target;
