@@ -10,15 +10,16 @@
 byte *terminal_charset = "utf-8";
 uns line_len = 78;
 
-static struct cfitem dumperconfig[] = {
-	{ "Dumper",		CT_SECTION,	NULL },
-	{ "TerminalCharset",	CT_STRING,	&terminal_charset },
-	{ "TerminalWidth",	CT_INT,		&line_len },
-	{ NULL,			CT_STOP,	NULL }
+static struct cf_section dumper_config = {
+  CF_ITEMS {
+    CF_STRING("TerminalCharset", &terminal_charset),
+    CF_UNS("TerminalWidth", &line_len),
+    CF_END
+  }
 };
 
 static void CONSTRUCTOR dumperconfig_init(void)
 {
-	cf_register(dumperconfig);
+  cf_declare_section("Dumper", &dumper_config, 0);
 }
 
