@@ -1,14 +1,13 @@
 /*
  *	Sherlock Utilities -- Index Dumper
  *
- *	(c) 2001--2004 Robert Spalek <robert@ucw.cz>
- *	(c) 2002--2004 Martin Mares <mj@ucw.cz>
+ *	(c) 2001--2005 Robert Spalek <robert@ucw.cz>
+ *	(c) 2002--2006 Martin Mares <mj@ucw.cz>
  */
 
 #include "sherlock/sherlock.h"
-#include "lib/conf.h"
+#include "lib/getopt.h"
 #include "lib/fastbuf.h"
-#include "sherlock/bucket.h"
 #include "lib/chartype.h"
 #include "sherlock/object.h"
 #include "lib/url.h"
@@ -35,29 +34,60 @@ static struct fastbuf *output;
 static int verbose;				/* Expand object attribute names?  */
 static int bare;				/* Avoid headings */
 static int raw;					/* Do not parse carse */
+static int annotate;				/* Annotate with URL's */
+static byte *zoek;				/* Only dump records matching this string */
 
 /* Maximal input and output line length.  */
 #define	BUFSIZE		2048
 #define	LINE_LEN	512
+/* Special reserved value */
+#define	PASS_POSITION	0xc001cafe
+
+static void
+format_flags(byte *dest, const byte *fmt, uns flags)
+{
+	for (uns i=0; fmt[i]; i++) {
+		if (flags & (1 << i))
+			*dest++ = fmt[i];
+		else
+			*dest++ = '-';
+	}
+	*dest = 0;
+}
+
+static byte *
+get_annot(int id)
+{
+	static struct fastbuf *annot_fb;
+	static int annot_last_id = -1;
+	static byte annot_buf[256];
+	if (!annotate)
+		return "";
+	if (!annot_fb)
+		annot_fb = index_bopen("url-list", O_RDONLY);
+	ASSERT(annot_last_id <= id);
+	while (annot_last_id < id) {
+		if (!bgets(annot_fb, annot_buf, sizeof(annot_buf)))
+			die("Unable to annotate oid 0x%x: out of url-list", id);
+		annot_last_id++;
+	}
+	return annot_buf;
+}
 
 static void
 dump_card_attr(u64 id, void *tmp)
 {
 	struct card_attr *a = tmp;
-	byte *attrs = "EADMIFO*";
-	byte at[9];
-	byte tf[32];
-	uns i;
+	byte *attrs = "EADMF123";
+	byte at[9], tf[32], mh[9];
 
 	if (!a)
 	{
 		if (!verbose)
-			bprintf(output, "ID       Card     SiteID     Area Wgt %s Age Type\n", attrs);
+			bprintf(output, "ID       Card     SiteID   MrgHash    Area Wgt %s Age Type\n", attrs);
 		return;
 	}
-	for (i=0; i<8; i++)
-	  at[i] = (a->flags & (1<<i)) ? attrs[i] : '-';
-	at[i] = 0;
+	format_flags(at, attrs, a->flags);
 #ifdef CONFIG_FILETYPE
 	byte *x = tf + sprintf(tf, "%02x %s", a->type_flags, custom_file_type_names[CA_GET_FILE_TYPE(a)]);
 #ifdef CONFIG_LANG
@@ -69,10 +99,16 @@ dump_card_attr(u64 id, void *tmp)
 #else
 	tf[0] = 0;
 #endif
+	for (int i=0; i<SHERLOCK_MERGING_HASH_SIZE; i++)
+		sprintf(mh+2*i, "%02x", a->merging_hash[i]);
+	mh[2*SHERLOCK_MERGING_HASH_SIZE] = 0;
 	if (verbose)
 	{
 		bprintf(output, "Attribute %x:\n", (u32) id);
 		bprintf(output, "Card:\t%x\n", a->card);
+#ifdef CONFIG_MERGING_HASHES
+		bprintf(output, "MrgHash:\t%s\n", mh);
+#endif
 		bprintf(output, "Weight:\t%d\n", a->weight);
 		bprintf(output, "Flags:\t%s\n", at);
 #ifdef CONFIG_LASTMOD
@@ -90,13 +126,16 @@ dump_card_attr(u64 id, void *tmp)
 #undef SMALL_SET_ATTR
 #undef LATE_INT_ATTR
 #undef LATE_SMALL_SET_ATTR
+		if (annotate)
+			bprintf(output, "URL:\t%s\n", get_annot(id));
 		bputc(output, '\n');
 	}
 	else
 	{
-		bprintf(output, "%8x %8x %08x %6d %3d %s %3d %s\n",
+		bprintf(output, "%8x %8x %8x %-8s %6d %3d %s %3d %-20s %s\n",
 			(u32) id, a->card,
 		       0,
+		       mh,
 		       0,
 		       a->weight, at,
 #ifdef CONFIG_LASTMOD
@@ -104,11 +143,30 @@ dump_card_attr(u64 id, void *tmp)
 #else
 		       0,
 #endif
-		       tf
+		       tf,
+		       get_annot(id)
 		       );
 	}
 }
 
+static void
+dump_footprint(struct fastbuf *output, byte *fp)
+{
+	for (uns i=0; i<16; i++)
+	{
+		if (i == 8)
+			bputc(output, ':');
+		bprintf(output, "%02x", fp[i]);
+	}
+}
+
+static void
+dump_md5(struct fastbuf *output, byte *md5)
+{
+	for (uns i=0; i<16; i++)
+		bprintf(output, "%02x", md5[i]);
+}
+
 static void
 dump_note(u64 id, void *tmp)
 {
@@ -118,33 +176,33 @@ dump_note(u64 id, void *tmp)
 
 	if (!n)
 	{
-		bprintf(output, "ID         Area USize  Scn Ook Dyn Mrg %s Footprint\n", attrs);
+		byte header[SITE_HASH_SIZE*2+1], format[10];
+		sprintf(format, "%%-%d.%ds", SITE_HASH_SIZE*2, SITE_HASH_SIZE*2);
+		sprintf(header, format, "SiteHash");
+		bprintf(output, "ID         Area CBonus USize  Scn Ook Dyn Mrg %s %s Footprint\n", attrs, header);
 		return;
 	}
-	for (uns i=0; i<8; i++)
-	  at[i] = (n->flags & (1<<i)) ? attrs[i] : '-';
-	at[8] = 0;
-	bprintf(output, "%8x %6d %6d %3d %3d %3d %3d %s ", (u32) id,
+	format_flags(at, attrs, n->flags);
+	bprintf(output, "%8x %6d %6d %6d %3d %3d %3d %3d %s ", (u32) id,
+	       0,
+	       n->card_bonus,
 	       n->useful_size,
 	       n->weight_scanner,
 	       0,
 	       0,
 	       n->weight_merged,
 	       at);
-	for (uns i=0; i<16; i++)
-	{
-		if (i == 8)
-			bputc(output, ':');
-		bprintf(output, "%02x", n->footprint[i]);
-	}
-	bputc(output, '\n');
+	for (int i=0; i<SITE_HASH_SIZE; i++)
+		bprintf(output, "%02x", n->site_hash[i]);
+	bputc(output, ' ');
+	dump_footprint(output, n->footprint);
+	bprintf(output, " %s\n", get_annot(id));
 }
 
 static void
 dump_checksum(u64 id, void *tmp)
 {
 	struct csum *c = tmp;
-	int i;
 	if (!tmp)
 	{
 		if (!verbose)
@@ -155,16 +213,14 @@ dump_checksum(u64 id, void *tmp)
 	{
 		bprintf(output, "Checksum at %x:\n", (u32) id);
 		bprintf(output, "MD5:\t");
-		for (i=0; i<16; i++)
-			bprintf(output, "%02x", c->md5[i]);
+		dump_md5(output, c->md5);
 		bprintf(output, "\n");
 		bprintf(output, "Card:\t%08x\n\n", c->cardid);
 	}
 	else
 	{
 		bprintf(output, "%08x ", (u32) id);
-		for (i=0; i<16; i++)
-			bprintf(output, "%02x", c->md5[i]);
+		dump_md5(output, c->md5);
 		bprintf(output, " %08x\n", c->cardid);
 	}
 }
@@ -172,7 +228,7 @@ dump_checksum(u64 id, void *tmp)
 static void
 dump_fingerprint(u64 id, void *tmp)
 {
-	struct fprint *c = tmp;
+	struct card_print *c = tmp;
 	int i;
 	if (!c)
 	{
@@ -232,26 +288,31 @@ static void
 dump_card(u64 start, void *tmp)
 {
 	struct fastbuf *f = tmp;
-	u32 id = start >> CARD_POS_SHIFT;
+	u32 id;
 	byte *buf;
 	int buf_len;
 	uns buf_type;
 
 	if (!f)
 		return;
+	id = start >> CARD_POS_SHIFT;
 	ASSERT(!(start & ((1 << CARD_POS_SHIFT) - 1)));
 	if (raw > 1)
 	{
-		buf_type = bgetl(f);
+		buf_len = bgetl(f);
+		buf_type = bgetc(f) + BUCKET_TYPE_PLAIN;
 		if (buf_type < BUCKET_TYPE_PLAIN || buf_type > BUCKET_TYPE_V33_LIZARD)
 			die("Cannot parse card %08x: bucket_type=%08x", id, buf_type);
-		buf_len = bgetl(f);
+		uns sep = bgetc(f);
+		ASSERT(!sep);
+		buf_len -= LIZARD_COMPRESS_HEADER;
 		bput_attr_format(output, '#', "## %08x %06d %08x", id, buf_len, buf_type);
 		if (buf_type == BUCKET_TYPE_V33_LIZARD)
 		{
 			uns orig_len = bgetl(f);
 			uns adler = bgetl(f);
 			bput_attr_format(output, '#', "## orig_len=%d adler=%08x", orig_len, adler);
+			buf_len -= 8;
 		}
 	}
 	else
@@ -306,13 +367,6 @@ dump_card(u64 start, void *tmp)
 		bgetc(f);
 }
 
-struct lab {
-	u32 merged_id;
-	u32 url_id;
-	u32 redir_id;
-	u32 count;
-} PACKED;
-
 static byte *wt_names[8] = { WORD_TYPE_USER_NAMES };
 static byte *mt_names[16] = { META_TYPE_USER_NAMES };
 
@@ -336,11 +390,33 @@ dump_type_name(struct fastbuf *f, struct fastbuf *output, uns meta)
 	return shift;
 }
 
+static void
+dump_single_label(struct parsed_attr *pa)
+{
+	byte *p = pa->val, *end = pa->val + pa->len;
+	bprintf(output, "\t%c", pa->attr);
+	while (p < end)
+	{
+		uns c;
+		GET_TAGGED_CHAR(p, c);
+		if (c < 0x80000000)
+			bput_utf8(output, c);
+		else if (c < 0x80001000)
+			bprintf(output, " [%s] ", mt_names[c & 0xf]);
+		else
+			bprintf(output, " [???] ");
+	}
+	bputc(output, '\n');
+}
+
+static const char label_flags[] = "BUMO";
+
 static void
 dump_labels(u64 start, void *tmp)
 {
 	struct fastbuf *f = tmp;
 	struct lab label;
+	byte fl[9];
 
 	if (!f)
 		return;
@@ -350,19 +426,17 @@ dump_labels(u64 start, void *tmp)
 	bprintf(output, "URL ID:\t\t%08x\n", label.url_id);
 	bprintf(output, "Redirect ID:\t%08x\n", label.redir_id);
 	bprintf(output, "Count:\t%d\n", label.count);
-	bprintf(output, "Labels:\n\t");
+	format_flags(fl, label_flags, label.flags);
+	bprintf(output, "Flags:\t%s\n", fl);
+	bprintf(output, "Labels:\n");
 	sh_off_t stop = btell(f) + label.count;
+	get_attr_set_type(BUCKET_TYPE_V33);
 	while (btell(f) < stop)
 	{
-		uns c = bget_tagged_char(f);
-		if (!c)
-			bprintf(output, "\n\t");
-		else if (c < 0x80000000)
-			bput_utf8(output, c);
-		else if (c < 0x80001000)
-			bprintf(output, " [%s] ", mt_names[c & 0xf]);
-		else
-			bprintf(output, " [???] ");
+		struct parsed_attr pa;
+		if (bget_attr(f, &pa) <= 0)
+			die("Broken label file");
+		dump_single_label(&pa);
 	}
 	bprintf(output, "\n");
 }
@@ -376,48 +450,25 @@ dump_labels_id(u64 start, void *tmp)
 		return;
 
 	u32 orig_id = bgetl(f);
-	bprintf(output, "Label block at %08qx for URL %08x\n\t", start, orig_id);
+	byte fl[9];
+	format_flags(fl, label_flags, bgetc(f));
+	bprintf(output, "Label block at %08qx for URL %08x (%s)\n", start, orig_id, fl);
+	get_attr_set_type(BUCKET_TYPE_V33);
 	while (1)
 	{
-		int c;
-		c = bgetc(f);
-		if (c == EOF)
-		{
-			bprintf(output, "\n");
+		struct parsed_attr pa;
+		int c = bget_attr(f, &pa);
+		if (c < 0)
+			die("Broken label file");
+		if (!c)
 			break;
-		}
-		bputc(output, c);
-		uns x;
-		while ((x = bget_tagged_char(f)) != 0 && x != ~0U)
-		{
-			if (x < 0x80000000)
-				bput_utf8(output, x);
-			else if (x < 0x80001000)
-				bprintf(output, " [%s] ", mt_names[x & 0xf]);
-			else
-				bprintf(output, " [???] ");
-		}
-		bprintf(output, "\n\t");
-		c = bgetc(f);
-		if (!c || c == EOF)
-			break;
-		bungetc(f);
+		dump_single_label(&pa);
 	}
 	bprintf(output, "\n");
 }
 
 static char *word_classes[] = { "cplx", "ignr", "word", "garb", "ctxt", "brek", "????", "????" };
 
-static void
-dump_lex_word(struct fastbuf *f, uns len)
-{
-  byte word[MAX_WORD_LEN+1];
-
-  ASSERT(len <= MAX_WORD_LEN);
-  breadb(f, word, len);
-  bwrite(output, word, len);
-}
-
 static void
 dump_lex_entry(struct lex_entry *l, uns id, struct fastbuf *f)
 {
@@ -434,6 +485,11 @@ dump_lex_entry(struct lex_entry *l, uns id, struct fastbuf *f)
 #else
 	uns freq = 0;
 #endif
+	byte word[l->length+1];
+	bread(f, word, l->length);
+	word[l->length] = 0;
+	if (zoek && strcmp(zoek, word))
+		return;
 
 	if (verbose)
 	{
@@ -450,7 +506,7 @@ dump_lex_entry(struct lex_entry *l, uns id, struct fastbuf *f)
 		bprintf(output, "%8x %8qx+%07x %s %3d %04x %3d ", id, (long long) ref_pos, chlen,
 		       (l->class > 7 ? "????" : word_classes[l->class]), freq, ctxt, l->length);
 	}
-	dump_lex_word(f, l->length);
+	bwrite(output, word, l->length);
 	bprintf(output, verbose ? "\n\n" : "\n");
 }
 
@@ -513,44 +569,72 @@ dump_lex_words(u64 start, void *tmp)
 }
 
 static void
-dump_lex_temp(u64 start, void *tmp)
+dump_lex_temp(u64 pos, void *tmp)
 {
 	struct fastbuf *f = tmp;
+	static uns wid;
 
+	if (pos == PASS_POSITION)		// start of a new block
+	{
+		wid = ~0U;
+		return;
+	}
 	if (!f)
 	{
 		if (!verbose)
-			bprintf(output, "ID       Count      Flgs Ctxt Len Word\n");
+			bprintf(output, "Pos      ID       Count      Flgs Ctxt Len Word\n");
 		return;
 	}
-	if (!start)
+	if (pos < wid)				// requested former word
+	{
+		wid = 0;
+		bsetpos(f, 0);
+	}
+	if (!wid)				// beginning of the file
 	{
 		u32 wc = bgetl(f);
 		bprintf(output, "# Word count:\t%d\n", wc);
 	}
+
+	u32 id, count;
+	uns class, context, length;
+	while (1)
+	{
+		id = bgetl(f);
+		if (id == ~0U)			// eof
+			return;
+		count = bgetl(f);
+		class = id & 7;
+		context = bget_context(f);
+		length = bgetc(f);
+		if (wid++ < pos)
+			bskip(f, length);
+		else
+			break;
+	}
+
+	byte word[length+1];
+	bread(f, word, length);
+	word[length] = 0;
+	if (zoek && strcmp(zoek, word))
+		return;
+
+	if (verbose)
+	{
+		bprintf(output, "Record %x:\n", (uns) pos);
+		bprintf(output, "Word ID %x:\n", id);
+		bprintf(output, "Count:\t\t%d\n", count);
+		bprintf(output, "Word class:\t\t%s\n", word_classes[class]);
+		bprintf(output, "Ctxt class:\t\t%d\n", context);
+		bprintf(output, "Length:\t\t%d\n", length);
+		bprintf(output, "Word:\t\t");
+	}
 	else
 	{
-		u32 id = bgetl(f);
-		u32 count = bgetl(f);
-		uns class = id & 7;
-		uns context = bget_context(f);
-		uns length = bgetc(f);
-		if (verbose)
-		{
-			bprintf(output, "Word ID %x:\n", id);
-			bprintf(output, "Count:\t\t%d\n", count);
-			bprintf(output, "Word class:\t\t%s\n", word_classes[class]);
-			bprintf(output, "Ctxt class:\t\t%d\n", context);
-			bprintf(output, "Length:\t\t%d\n", length);
-			bprintf(output, "Word:\t\t");
-		}
-		else
-		{
-			bprintf(output, "%8x %10d %s %04x %3d ", id, count, word_classes[class], context, length);
-		}
-		dump_lex_word(f, length);
-		bprintf(output, verbose ? "\n\n" : "\n");
+		bprintf(output, "%8x %8x %10d %s %04x %3d ", (uns) pos, id, count, word_classes[class], context, length);
 	}
+	bwrite(output, word, length);
+	bprintf(output, verbose ? "\n\n" : "\n");
 }
 
 static void
@@ -671,53 +755,104 @@ dump_u32(u64 id, void *tmp)
 	bprintf(output, "%08x -> %08x\n", (u32) id, *u);
 }
 
+static inline uns
+Bgetc(struct fastbuf *f)
+{
+  uns c = bgetc(f);
+  //bprintf(output, "#%02x ", c);
+  return c;
+}
+
+static inline uns
+Bgetw(struct fastbuf *f)
+{
+  uns w = bgetw(f);
+  //bprintf(output, "#%04x ", w);
+  return w;
+}
+
+static uns
+dump_refchain(struct fastbuf *f)
+{
+	uns rlen = 4;
+	u64 pos = btell(f);
+	u32 oid = bgetl(f);
+	if (!oid)
+		return rlen;
+	uns len = oid >> 28;
+	oid &= (1<<28) - 1;
+	if (!len)
+		len = bget_utf8_32(f), rlen += utf8_space(len);
+	rlen += len;
+	if (verbose)
+		bprintf(output, "OID:\t%08x\nLength:\t%d\n\t", oid, len);
+	else
+		bprintf(output, "%8qx %8x %8d ", pos, oid, len);
+	uns last_pos = 0;
+	while (len)
+	{
+		uns pos, type, wt = 0, len1;
+		uns c = Bgetc(f);
+		if (c < 0x80)
+			len1 = 1, type = c >> 6, pos = c & 0x3f;
+		else if (c < 0xc0)
+			len1 = 2, type = c & 7, pos = ((c & 0x38) << 5) | Bgetc(f);
+		else if (c < 0xe0)
+			len1 = 3, type = c & 7, pos = ((c & 0x18) << 13) | Bgetw(f);
+		else if (c < 0xf0)
+			len1 = 2, type = 0x80 | c & 0xf,
+			c = Bgetc(f), wt = c >> 6, pos = c & 0x3f;
+		else if (c < 0xf8)
+			len1 = 3, wt = c & 3, type = 0x80 | (c & 4) << 1,
+			pos = Bgetw(f), type |= pos & 7, pos >>= 3;
+		else if (c < 0xfc)
+			len1 = 4, type = c & 3,
+			c = Bgetc(f), type |= (c >> 5) & 4,
+			pos = ((c & 0x7f) << 16) | Bgetw(f);
+		else if (c < 0xfe)
+			len1 = 4, wt = c & 1,
+			c = Bgetc(f), wt |= (c >> 6) & 2, type = 0x80 | c & 0xf,
+			pos = ((c & 0x70) << 12) | Bgetw(f);
+		else
+		{
+			if (c >= 0xfe)
+				bprintf(output, "RFU ");
+			len--;
+			continue;
+		}
+		if (!(type & 0x80))
+			pos += last_pos;
+		if (verbose)
+			if (type & 0x80)
+				bprintf(output, "(%03x:%s:%d)_%d ", pos, mt_names[type & 0x7f], wt, len1);
+			else
+				bprintf(output, "(+%02x=%03x:%s)_%d ", pos-last_pos, pos, wt_names[type], len1);
+		else
+			if (type & 0x80)
+				bprintf(output, "(%03x:%x:%d) ", pos, type & 0x7f, wt);
+			else
+				bprintf(output, "(%03x:%x) ", pos, type);
+		last_pos = pos;
+		len -= len1;
+	}
+	bprintf(output, "\n");
+	return rlen;
+}
+
 static void
 dump_refs(u64 start, void *tmp)
 {
 	struct fastbuf *f = tmp;
-	u16 hi, lo;
 
 	if (!f)
 	{
 		if (!verbose)
-			bprintf(output, "Pos      OID      Count ref\n");
+			bprintf(output, "Pos      OID      Length   ref\n");
 		return;
 	}
 	if (verbose)
 		bprintf(output, "Reference block at %08qx:\n", start);
-	hi = bgetw(f);
-	while (1)
-	{
-		lo = bgetw(f);
-		if (!hi && !lo)
-			break;
-		uns cnt = bgetw(f);
-		if (verbose)
-			bprintf(output, "OID:\t%08x\nCount:\t%d\n\t", hi<<16 | lo, cnt);
-		else
-			bprintf(output, "%8qx %8x %5d ", (long long)btell(f)-6, hi<<16 | lo, cnt);
-		while (1)
-		{
-			word ref = bgetw(f);
-			if (!(ref >> 12))
-			{
-				hi = ref;
-				ASSERT(!cnt);
-				break;
-			}
-			if (verbose)
-			{
-				if (ref < 0x8000)
-					bprintf(output, "%04x(%03x:%s) ", ref, (ref & 0xfff), wt_names[ ref >> 12 ]);
-				else
-					bprintf(output, "%04x(%03x:%s:%d) ", ref, ((ref >> 2) & 0x1ff), mt_names[ ((ref >> 11) & 0xf) ], ref & 3);
-			}
-			else
-				bprintf(output, "%04x ", ref);
-			cnt--;
-		}
-		bprintf(output, "\n");
-	}
+	while (dump_refchain(f) > 4);
 	bprintf(output, "\n");
 }
 
@@ -748,31 +883,62 @@ dump_ref_texts(u64 start UNUSED, void *tmp)
 }
 
 static void
-dump_ascii(u64 start UNUSED, void *tmp)
+dump_sites(u64 id, void *tmp)
 {
-	struct fastbuf *f = tmp;
-	byte line[BUFSIZE];
-
-	if (!f)
+	struct site_mapping *m = tmp;
+	int i;
+	if (!m)
+	{
+		if (!verbose)
+			bprintf(output, "SiteID   SiteHash\n");
 		return;
-	bgets(f, line, BUFSIZE);
-	bputs(output, line);
-	bputc(output, '\n');
+	}
+	if (verbose)
+	{
+		bprintf(output, "ID:\t%x\n", (u32) id);
+		bprintf(output, "Hash:\t");
+		for (i=0; i<SITE_HASH_SIZE; i++)
+			bprintf(output, "%02x", m->site_hash[i]);
+		bprintf(output, "\n");
+	}
+	else
+	{
+		bprintf(output, "%08x ", (u32) id);
+		for (i=0; i<SITE_HASH_SIZE; i++)
+			bprintf(output, "%02x", m->site_hash[i]);
+		bprintf(output, "\n");
+	}
 }
 
 static void
-dump_urls(u64 start UNUSED, void *tmp)
+dump_urls(u64 pos, void *tmp)
 {
 	struct fastbuf *f = tmp;
 	byte line[BUFSIZE];
 	static uns oid;
 
-	if (!f) {
-		oid = 0;
+	if (pos == PASS_POSITION)		// startup of a new block
+	{
+		oid = ~0U;
 		return;
 	}
-	bgets(f, line, BUFSIZE);
-	bprintf(output, "%08x %s\n", oid, line);
+	if (!f)
+		return;
+	if (pos < oid)				// requested former line
+	{
+		oid = 0;
+		bsetpos(f, 0);
+	}
+	while (oid < pos)			// skip lines until the requested
+	{
+		if (!bgets(f, line, BUFSIZE))
+			return;
+		oid++;
+	}
+	if (!bgets(f, line, BUFSIZE))
+		return;
+	if (!zoek || strstr(line, zoek))
+		bprintf(output, "%08x %s\n", oid, line);
 	oid++;
 }
 
@@ -805,33 +971,7 @@ dump_string_index(u64 start, void *tmp)
 		bprintf(output, " %8d\n", size);
 	}
 	while (size > 0)
-	{
-		u32 oid;
-		u16 count;
-		u64 estart = btell(f);
-		oid = bgetl(f);
-		count = bgetw(f);
-		size -= sizeof(u32) + (count+1)*sizeof(u16);
-		if (verbose)
-		{
-			bprintf(output, "OID:\t%08x\n", oid);
-			bprintf(output, "Count:\t%d\n", count);
-		}
-		else
-		{
-			bprintf(output, "O%8qx %8x %5d  ", estart, oid, count);
-		}
-		for (i=0; i<count; i++)
-		{
-			u16 ref = bgetw(f);
-			if (verbose)
-				bprintf(output, "Ref:\t%04x\n", ref);
-			else
-				bprintf(output, "%04x ", ref);
-		}
-		if (!verbose)
-			bprintf(output, "\n");
-	}
+		size -= dump_refchain(f);
 	bprintf(output, "\n");
 }
 
@@ -874,7 +1014,6 @@ dump_word_index(u64 start, void *tmp)
 {
 	struct fastbuf *f = tmp;
 	u32 wordid, size;
-	uns i;
 
 	if (!f)
 		return;
@@ -891,33 +1030,7 @@ dump_word_index(u64 start, void *tmp)
 		bprintf(output, "S%8qx %8x %8d\n", start, wordid, size);
 	}
 	while (size > 0)
-	{
-		u32 oid;
-		u16 count;
-		u64 estart = btell(f);
-		oid = bgetl(f);
-		count = bgetw(f);
-		size -= sizeof(u32) + (count+1)*sizeof(u16);
-		if (verbose)
-		{
-			bprintf(output, "OID:\t%08x\n", oid);
-			bprintf(output, "Count:\t%d\n", count);
-		}
-		else
-		{
-			bprintf(output, "O%8qx %8x %5d  ", estart, oid, count);
-		}
-		for (i=0; i<count; i++)
-		{
-			u16 ref = bgetw(f);
-			if (verbose)
-				bprintf(output, "Ref:\t%04x\n", ref);
-			else
-				bprintf(output, "%04x ", ref);
-		}
-		if (!verbose)
-			bprintf(output, "\n");
-	}
+		size -= dump_refchain(f);
 	bprintf(output, "\n");
 }
 
@@ -934,11 +1047,18 @@ dump_params(u64 start UNUSED, void *tmp)
 	bprintf(output, "\tmin_len_ign\t%d\n", par->lex_config.min_len_ign);
 	bprintf(output, "\tmin_len\t\t%d\n", par->lex_config.min_len);
 	bprintf(output, "\tmax_len\t\t%d\n", par->lex_config.max_len);
-	bprintf(output, "\tmax_hex_len\t%d\n", par->lex_config.max_hex_len);
+	bprintf(output, "\tmax_num_len\t%d\n", par->lex_config.max_num_len);
+	bprintf(output, "\tmax_mixed_len\t%d\n", par->lex_config.max_mixed_len);
 	bprintf(output, "\tmax_ctrl_len\t%d\n", par->lex_config.max_ctrl_len);
 	bprintf(output, "\tmax_gap\t\t%d\n", par->lex_config.max_gap);
 	bprintf(output, "\tcontext_slots\t%d\n", par->lex_config.context_slots);
 	bprintf(output, "Input objects:\t\t%d\n", par->objects_in);
+	bprintf(output, "Number of sites:\t%d\n", par->sites);
+	bprintf(output, "Srand:\t\t\t%d\n", par->srand);
+	bprintf(output, "Last download:\t\t%d\n", par->last_download);
+	bprintf(output, "DB version:\t\t%08x\n", par->database_version);
+	bprintf(output, "Type mask:\t\t%04x\n", par->type_mask);
+	bprintf(output, "ID mask:\t\t%04x\n", par->id_mask);
 }
 
 static void
@@ -991,19 +1111,18 @@ dump_feedback_gath(u64 id, void *tmp)
 		bprintf(output, "ID       Footprint                         CardID   Flags    Dyn\n");
 		return;
 	}
-	for (uns i=0; i<8; i++)
-		at[i] = (f->flags & (1<<i)) ? attrs[i] : '-';
-	at[8] = 0;
-	bprintf(output, "%08x ", id);
-	for (uns i=0; i<16; i++)
-	{
-		if (i == 8)
-			bputc(output, ':');
-		bprintf(output, "%02x", f->footprint[i]);
-	}
+	format_flags(at, attrs, f->flags);
+	bprintf(output, "%08x ", (uns)id);
+	dump_footprint(output, f->footprint);
 	bprintf(output, " %08x %s %d\n", f->cardid, at, f->weight);
 }
 
+static void
+dump_incremental(u64 id UNUSED, void *tmp UNUSED)
+{
+  die("Disabled CONFIG_INCREMENTAL");
+}
+
 struct ule {
 	uns pos;
 	char x[1];
@@ -1069,7 +1188,7 @@ struct index_file {
 	int option;
 	byte **default_filename;
 	int record_size;
-		/* 
+		/*
 		 * record_size > 0:	uniform sequence with records of size record_size
 		 * record_size == 0:	non-uniform sequence
 		 * record_size < 0:	non-uniform sequence with records aligned to -record_size
@@ -1082,16 +1201,17 @@ static struct index_file index_files[] = {
 	{ 'c', &fn_cards,		-(1 << CARD_POS_SHIFT),		dump_card	},
 	{ 'd', &fn_card_attrs,		sizeof(struct card_attr),	dump_card_attr	},
 	{ 'h', &fn_checksums,		sizeof(struct csum),		dump_checksum	},
-	{ 'F', &fn_fingerprints,	sizeof(struct fprint),		dump_fingerprint},
+	{ 'F', &fn_fingerprints,	sizeof(struct card_print),	dump_fingerprint},
 	{ 'l', &fn_labels,		0,				dump_labels	},
 	{ 'L', &fn_labels_by_id,	0,				dump_labels_id	},
 	{ 'x', &fn_lexicon,		0,				dump_lexicon	},
-	{ 'Q', &fn_lex_by_freq,		0,				dump_lex_temp	},
-	{ 'R', &fn_lex_raw,		0,				dump_lex_temp	},
+	{ 'Q', &fn_lex_by_freq,		PASS_POSITION,			dump_lex_temp	},
+	{ 'R', &fn_lex_raw,		PASS_POSITION,			dump_lex_temp	},
 	{ 'T', &fn_lex_raw,		0,				dump_lex_stats	},
-	{ 'O', &fn_lex_ordered,		0,				dump_lex_temp	},
+	{ 'O', &fn_lex_ordered,		PASS_POSITION,			dump_lex_temp	},
 	{ 'W', &fn_lex_words,		0,				dump_lex_words	},
-	{ 'k', &fn_links,		sizeof(struct fprint),		dump_fingerprint},
+	{ 'k', &fn_links,		sizeof(struct card_print),	dump_fingerprint},
+	{ 'K', &fn_blacklist,		sizeof(u32),			dump_u32	},
 	{ 'g', &fn_link_graph,		0,				dump_graph	},
 	{ 'G', &fn_link_graph_index,	-BYTES_PER_O,			dump_graph_index},
 	{ 'm', &fn_merges,		sizeof(u32),			dump_u32	},
@@ -1101,29 +1221,34 @@ static struct index_file index_files[] = {
 	{ 'P', &fn_card_prints,		sizeof(struct card_print),	dump_fingerprint},
 	{ 'r', &fn_references,		0,				dump_refs	},
 	{ 't', &fn_ref_texts,		0,				dump_ref_texts	},
-	{ 's', &fn_sites,		0,				dump_ascii	},
+	{ 's', &fn_sites,		sizeof(struct site_mapping),	dump_sites	},
 	{ 'U', &fn_signatures,		0, /* computed run-time */	dump_signatures	},
 	{ 'y', &fn_stems,		0,				dump_stems	},
 	{ 'Y', &fn_stems_ordered,	0,				dump_stems_temp	},
 	{ 'H', &fn_string_hash,		sizeof(u32),			dump_u32	},
-	{ 'I', &fn_string_index,	0,				dump_string_index },
+	{ 'X', &fn_string_index,	0,				dump_string_index },
 	{ 'M', &fn_string_map,		0,				dump_string_map },
-	{ 'u', &fn_urls,		0,				dump_urls	},
+	{ 'u', &fn_urls,		PASS_POSITION,			dump_urls	},
 	{ 'q', &fn_urls,		0,				NULL	 	},	/* kludge */
 	{ 'w', &fn_word_index,		0,				dump_word_index },
 	{ 'B', &fn_feedback_gath,	sizeof(struct feedback_gatherer),	dump_feedback_gath },
+	{ 'I', &fn_incremental,		0, /* computed run-time */	dump_incremental},
 	{ 0,   NULL,			0,				NULL		}
 };
 
-static char *shortopts = CF_SHORT_OPTS "0abcdf:ghklmnpqrstuvwxyBFGHILMNOQPRTUWY";
+static char *shortopts = "0abcdf:ghi:klmnpqrstuvwxyz:ABFGHIKLMNOQPRTUWXY" CF_SHORT_OPTS;
 static struct option longopts[] =
 {
 	CF_LONG_OPTS
+	{ "zoek",		0, 0, 'z' },
 	{ "raw",		0, 0, '0' },
 	{ "bare",		0, 0, 'b' },
 	{ "filename",		0, 0, 'f' },
 	{ "verbose",		0, 0, 'v' },
+	{ "index",		0, 0, 'i' },
+	{ "annotate",		0, 0, 'A' },
 	{ "attr",		0, 0, 'a' },
+	{ "blacklist",		0, 0, 'K' },
 	{ "card",		0, 0, 'c' },
 	{ "card-attr",		0, 0, 'd' },
 	{ "checksum",		0, 0, 'h' },
@@ -1145,18 +1270,19 @@ static struct option longopts[] =
 	{ "parameters",		0, 0, 'p' },
 	{ "prints",		0, 0, 'P' },
 	{ "reference",		0, 0, 'r' },
-	{ "reference-texts",	0, 0, 't' },
+	{ "reftexts",		0, 0, 't' },
 	{ "site",		0, 0, 's' },
 	{ "signature",		0, 0, 'U' },
 	{ "stems",		0, 0, 'y' },
 	{ "stems-ordered",	0, 0, 'Y' },
 	{ "string-hash",	0, 0, 'H' },
-	{ "string-index",	0, 0, 'I' },
+	{ "string-index",	0, 0, 'X' },
 	{ "string-map",		0, 0, 'M' },
 	{ "url",		0, 0, 'u' },
 	{ "url-lookup",		0, 0, 'q' },
 	{ "word",		0, 0, 'w' },
 	{ "feedback-gath",	0, 0, 'B' },
+	{ "incremental",	0, 0, 'I' },
 	{ NULL,			0, 0, 0 }
 };
 
@@ -1165,17 +1291,23 @@ Usage: idxdump [<options>] <index-file> [<id> | [<first-id>]-[<last-id>]]\n\
 \n\
 Options:\n"
 CF_USAGE
-"-b, --bare\t\tDon't print table heading\n\
+"\
+-A, --annotate\t\tAnnotate the output with URL's (currently only with -a and -n)\n\
+-b, --bare\t\tDon't print table heading\n\
 -f, --filename\t\tOverride default filename for given index file\n\
+-i, --index\t\tOverride default directory containing the index\n\
 -v, --verbose\t\tSet verbose mode\n\
--0, --raw\t\tDump raw card after decompression\n\n\
+-0, --raw\t\tDump raw card after decompression\n\
+-z, --zoek\t\tOnly dump records matching given string (lexicon*, url)\n\n\
 Index files:\n\
 -a, --attr\t\tAttributes\n\
+-K, --blacklist\t\tBlacklist\n\
 -c, --card\t\tCards\n\
 -d, --card-attr\t\tCard attributes\n\
 -h, --checksum\t\tChecksums\n\
 -B, --feedback-gath\tFeedback to the gatherer\n\
 -F, --finger\t\tFingerprints\n\
+-I, --incremental\tData of incremental scanner\n\
 -l, --label\t\tLabels\n\
 -L, --labels-id\t\tLabels by id\n\
 -x, --lexicon\t\tFinal lexicon\n\
@@ -1199,7 +1331,7 @@ Index files:\n\
 -y, --stems\t\tStem mappings\n\
 -Y, --stems-ordered\tTemporary stem mappings from lexorder\n\
 -H, --string-hash\tString hash\n\
--I, --string-index\tString index\n\
+-X, --string-index\tString index\n\
 -M, --string-map\tString map\n\
 -u, --url\t\tUrl list\n\
 -q, --url-lookup\tFind given URL's in the URL list\n\
@@ -1239,7 +1371,18 @@ xtol64(byte *c)
 static void
 dump_index_interval(struct index_file *f, struct fastbuf *b, u64 start, u64 stop)
 {
-	if (f->record_size > 0)
+	if (f->record_size == (int) PASS_POSITION)
+	{
+		sh_off_t size = bfilesize(b);
+		f->dump_single(PASS_POSITION, b);		// startup
+		for (u64 pos=start; pos<=stop; pos++)
+		{
+			f->dump_single(pos, b);
+			if (btell(b) >= size)
+				break;
+		}
+	}
+	else if (f->record_size > 0)
 	{
 		byte *buf = alloca(f->record_size);
 		bsetpos(b, f->record_size * start);
@@ -1330,6 +1473,9 @@ main(int argc, char **argv)
 	while ((opt = cf_getopt(argc, argv, shortopts, longopts, NULL)) >= 0)
 		switch (opt)
 		{
+			case 'A':
+				annotate++;
+				break;
 			case 'f':
 				force_filename = optarg;
 				break;
@@ -1342,6 +1488,12 @@ main(int argc, char **argv)
 			case '0':
 				raw++;
 				break;
+			case 'i':
+				fn_directory = optarg;
+				break;
+			case 'z':
+				zoek = optarg;
+				break;
 			default:
 				for (i=0; ; i++)
 					if (opt == index_files[i].option)
