@@ -3,7 +3,6 @@
 #
 # (c) 2003-2004 Tomas Valla <tom@ucw.cz>
 # (c) 2006 Vladimir Jelen <vladimir.jelen@netcentrum.cz>
-# (c) 2007 Pavel Charvat <pchar@ucw.cz>
 #
 
 package Sherlock::Watsonlib;
@@ -17,7 +16,7 @@ analyze_init analyze_finish
 compute_stat logprint info warning stat_usage stat_options
 analyze_options analyze_usage
 $stat_prefix $stat_picname $stat_begintime $stat_endtime $stat_expanddays
-read_config read_config_value read_config_array server_config_value $config_file %config
+read_config read_config_value $config_file %config
 );
 
 use strict;
@@ -47,54 +46,22 @@ our %config = ();
 
 sub read_config {
 	-f $config_file or die "Cannot open config file $config_file";
-	my $cmd = ". $config_file >/dev/null 2>&1";
+	my $cmd = ". $config_file";
 	foreach my $var (@_) {
-		# A little complex import of bash variables from the configuration script ;-)
-		# ... should work for any string values
-		if ($var =~ /^[a-zA-Z_][a-zA-Z0-9_]*$/) { # A single varible
-			$cmd.=' && echo -n "\$config{\"'.$var.'\"}='."'".'" && ' .
-			'(echo -n "${'.$var.'}" | sed "s'."/'/'.\\\"'\\\".'/".'g") && ' .
-			'echo "'."'".';"';
-		}
-		elsif ($var =~ /^([a-zA-Z_][a-zA-Z_0-9]*)\*$/) { # All variables with a given prefix
-			$cmd.=' && (for VAR in ${!'.$1.'*} ; do ' . 
-			'echo -n "\$config{\"$VAR\"}='."'".'" && ' .
-			'(eval echo -n "\${$VAR}" | sed "s'."/'/'.\\\"'\\\".'/".'g") && ' .
-			'echo "'."'".';"; done)';
-		}
-		else { die "Invalid read_config parameter"; }
+		$cmd .= " && echo $var:\$$var";
 	}
-	eval `$cmd`;
+	%config = map { split /:/, $_, 2 } split(/\n/, `$cmd`);
 }
 
 sub read_config_value {
 	my ($var, $default) = @_;
-	defined $config{"$var"} or return $default;
+	defined $config{"$var"} or die "Variable $var was not requested by function read_config";
 	if ($config{"$var"} eq '') {
 		defined $default or die "Missing required value $var in config file $config_file";
 		return $default;
 	}
 	return $config{"$var"};
 }
-
-sub read_config_array {
-	my ($var) = @_;
-	return split(/\s+/, read_config_value($var, ""));
-}
-
-sub server_config_value {
-	my ($server, $var_prefix, $var_default, $default) = @_;
-	my $v = $default;
-	if (defined($config{$var_default ? $var_default : $var_prefix})) {
-		$v = $config{$var_default ? $var_default : $var_prefix};
-	}
-	my $x = $var_prefix;
-	foreach my $y (split(/_/, $server)) {
-		$x = $x."_".$y;
-		if (defined($config{$x})) { $v = $config{$x}; }
-        }
-	return $v;
-}
 			    
 ########################################################
 # Misc. funtions
@@ -416,7 +383,6 @@ sub new {
 "set grid
 set data style lines
 set bmargin 3
-set border 0
 set xdata time
 set timefmt \"%Y:%j:%H:%M\"
 set terminal png
