@@ -4,7 +4,6 @@
 #
 # (c) 2003-2005 Tomas Valla <tom@ucw.cz>
 # (c) 2006 Vladimir Jelen <vladimir.jelen@netcentrum.cz>
-# (c) 2007 Pavel Charvat <pchar@ucw.cz>
 #
 
 use lib 'lib/perl5';
@@ -18,6 +17,8 @@ analyze_options() or die
 # Recompute whole interfile, because we don't remember which index we are processing
 $force = 1;
 
+read_config("INDEXER_INDICES");
+my @indices = split /\s+/, read_config_value('INDEXER_INDICES');
 my $last_index = '';
 my $sending = 0;
 
@@ -56,7 +57,6 @@ sub chew_indexerlog {
 		lang_fr
 		lang_es
 		lang_it
-		lang_ru
 		indexing_start
 		indexing_end
 		idxcopy_start
@@ -81,9 +81,12 @@ sub chew_indexerlog {
 		size_refs
 		size_string
 		size_total );
-		# chewer_card_*
-		# size_total_*
-		# size_lexicon_*
+
+	foreach my $index (@indices) {
+		push @keys, "chewer_card_$index";
+		push @keys, "size_total_$index";
+		push @keys, "size_lexicon_$index";
+	}
 
 	my $hdr_string="#@\n";
 
@@ -127,7 +130,7 @@ sub chew_indexerlog {
 			$d{'scan_obj'} = $1;
 			$d{'scan_card'} = $2;
 		}
-		elsif (/\[scanner] Scanned (\d+) objects \((\d+) ok, (\d+) err, (\d+) robots, (\d+) (new|skeletons)\)/) {
+		elsif (/\[scanner] Scanned (\d+) objects \((\d+) ok, (\d+) err, (\d+) robots, (\d+) new\)/) {
 			$d{'scan_obj'} = $1;
 			$d{'scan_doc'} = $2 + $3 + $4;
 		}
@@ -182,20 +185,18 @@ sub chew_indexerlog {
 			$d{'ft_out_png'} = $8;
 			$d{'ft_out_gif'} = $9;
 		}
-		elsif (/\[ireport] Languages: (.+)$/) {
-			my %lang = map { split /=/, $_, 2 } split(/\s+/, $1);
-			$d{'lang_unknown'} = $lang{'??'};
-			$d{'lang_en'} = $lang{'en'};
-			$d{'lang_cs'} = $lang{'cs'};
-			$d{'lang_sk'} = $lang{'sk'};
-			$d{'lang_pl'} = $lang{'pl'};
-			$d{'lang_hu'} = $lang{'hu'};
-			$d{'lang_de'} = $lang{'de'};
-			$d{'lang_nl'} = $lang{'nl'};
-			$d{'lang_fr'} = $lang{'fr'};
-			$d{'lang_es'} = $lang{'es'};
-			$d{'lang_it'} = $lang{'it'};
-			$d{'lang_ru'} = $lang{'ru'};
+		elsif (/\[ireport] Languages: \?\?=(\d+) en=(\d+) cs=(\d+) sk=(\d+) pl=(\d+) hu=(\d+) de=(\d+) nl=(\d+) fr=(\d+) es=(\d+) it=(\d+)/) {
+			$d{'lang_unknown'} = $1;
+			$d{'lang_en'} = $2;
+			$d{'lang_cs'} = $3;
+			$d{'lang_sk'} = $4;
+			$d{'lang_pl'} = $5;
+			$d{'lang_hu'} = $6;
+			$d{'lang_de'} = $7;
+			$d{'lang_nl'} = $8;
+			$d{'lang_fr'} = $9;
+			$d{'lang_es'} = $10;
+			$d{'lang_it'} = $11;
 		}
 
 		if (!$sending && /\[send-indices]/) {
@@ -204,6 +205,7 @@ sub chew_indexerlog {
 		}
 		elsif ($sending && /\[scheduler] All indices send/) {
 			$d{'idxcopy_end'} = $time;
+			$sending = 0;
 		}
 	}
 	for my $k (sort @keys) {
